#include "ActionStuff.h"
#include "ObjectStuff.h"
#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "FeatureStuff.h"
#include "ForegroundStuff.h"
#include "MyMATFileInterface.h"
#include <iostream>
#include <fstream>

using namespace std;
using namespace cv;

/************************************************************************************/
Mat getDataActionClassifier_superpixel_frame_foreground(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int seqNumber, const int frame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  float sizeThresh = 100.0;

  string datasetName = db.getDatasetName();

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);

  Mat im = db.getIM(seqNumber, frame);
  Mat ucm = db.readUCM(seqNumber, frame);
  PriorH p(im, ucm, k, k2, k3, frame);
  Mat centers = p.getCenters();
  Mat sizes = p.getPrior_size();
  Mat F_O = hierarchicalObjectClassification(p, objTrainers, clusterCenters_lab, clusterCenters_hog);

  Mat mask = db.readForegroundMask(seqNumber, frame, trainVersion);
  //imshow("alaki", mask);
  //cvWaitKey(0);
  //cout << "Mask: " << mask.rows << " " << mask.cols << endl;
  //cout << "SeqNumber: " << seqNumber << " " << " frame: " << frame << endl;
  mask = cleanForegroundMask(im, mask);

  Mat F;


  if((featType & ACT_F_CIR) >= 1){
    Mat F1_ = p.getFeatureInMask_colorHOG(clusterCenters_lab, clusterCenters_hog, mask);
    //cout << F1_.rows << " " << F1_.cols << endl;
    F1_ = F1_.t();
    F.push_back(F1_);
  }

  if((featType & ACT_F_OBJ) >= 1){
    int m = F_O.rows;
    int n = F_O.cols;
    Mat F2_(1, n, CV_32F);
    for(int j = 0; j < n; j++)
      F2_.at<float>(0,j) = -10; //-inf
    float dist;
    float cy,cx;
    for(int i = 0; i < m; i++){
      cy = centers.at<float>(i,0);
      cx = centers.at<float>(i,1);
      if(mask.at<uchar>(floor(cy),floor(cx)) == 0)
	continue;
      if(sizes.at<float>(i,0) < sizeThresh)
	continue;
      for(int j = 0; j < n; j++)
	F2_.at<float>(0,j) = max(F2_.at<float>(0,j), F_O.at<float>(i,j) );
    }
    F2_ = F2_.t();
    //cout << F2_.rows << " " << F2_.cols << endl;
    F.push_back(F2_);
  }

  F = F.t();

  //cout << F.rows << " " << F.cols << endl;

  return F;
}

/************************************************************************************/
Mat trainingDataActionClassifier_superpixel_interval_foreground(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/tmp/circle_superpixel/trainingDataActionClassifier_", dataDir.c_str());
  if((featType & ACT_F_CIR) >= 1)
    sprintf(buffer, "%scircle", buffer);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%sObject", buffer);

  if((featType & ACT_F_FG) >= 1)
    sprintf(buffer, "%sFG", buffer);

  if((featType & ACT_F_LOC) >= 1)
    sprintf(buffer, "%sLOC", buffer);

  sprintf(buffer, "%s_superpixel_interval_foreground_%d_%d_%d_%s_%d_%d", buffer, trainVersion, numBins_lab, numBins_hog, db.getSequence(seqNumber).getSeqName().c_str(), startFrame, endFrame);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%s_%d_%d_%d", buffer, k, k2, k3);

  sprintf(buffer, "%s.mat", buffer);

  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "trainData");
    return centers;
  }
  ifile.close();

  int fstep = 1;

  int len = endFrame- startFrame;
  fstep = len / 50;
  if(fstep == 0)
    fstep = 1;

  Mat trainData;

  Mat im = db.getIM(seqNumber, startFrame);
  float gy = ((float)(im.rows)) / 2.0;
  float gx = ((float)(im.cols)) / 2.0;
  vector<pair<float,float> > locations;
  locations.push_back(pair<float,float>(gy,gx));

  for(int frame = startFrame; frame <= endFrame; frame += fstep){
    Mat F = getDataActionClassifier_superpixel_frame_foreground(db, trainVersion, numBins_lab, numBins_hog, seqNumber, frame, k, k2, k3, objTrainers, featType);
    trainData.push_back(F);
  }

  writeMatrixToMatFile(trainData, "trainData", buffer);
  return trainData; 
}

/************************************************************************************/
Mat testingDataActionClassifier_superpixel_interval_foreground(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){
  return trainingDataActionClassifier_superpixel_interval_foreground(db, trainVersion, numBins_lab, numBins_hog, seqNumber, startFrame, endFrame, k, k2, k3, objTrainers, featType);
}

/************************************************************************************/
Mat getDataActionClassifier_superpixel_frame_location(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int frame, vector<pair<float,float> > locations, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  //cout << "hello" << endl;

  float sizeThresh = 100.0;

  string datasetName = db.getDatasetName();

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);

  Mat im = db.getIM(seqNumber, frame);
  Mat ucm = db.readUCM(seqNumber, frame);
  PriorH p(im, ucm, k, k2, k3, frame);
  Mat centers = p.getCenters();
  Mat sizes = p.getPrior_size();
  Mat F_O = hierarchicalObjectClassification(p, objTrainers, clusterCenters_lab, clusterCenters_hog);

  Mat mask;
  if((featType & ACT_F_FG) >= 1){
    //cout << seqNumber << " " << frame << endl;
    mask = db.readForegroundMask(seqNumber, frame, trainVersion);
    mask = cleanForegroundMask(im, mask);
    //cout << "Mask size: " << mask.rows << " " << mask.cols << endl;
  }

  Mat F;

  //cout << "Size of the locations: " << locations.size() << endl; 

  for(vector<pair<float,float> >::iterator it = locations.begin(); it != locations.end(); it++){
    float gy = it->first;
    float gx = it->second;

    if(gy < 0)
      continue;
    if(gx < 0)
      continue;
    if(gy >= im.rows)
      continue;
    if(gx >= im.cols)
      continue;

    Mat F_;
    
    if((featType & ACT_F_CIR) >= 1){
      Mat F1_ = p.getFeatureAroundPoint_colorHOG(clusterCenters_lab, clusterCenters_hog, gy, gx, radius);
      F1_ = F1_.t();
      //cout << "F1_: " << F1_.rows << " " << F1_.cols << endl;
      F_.push_back(F1_);
      //cout << "F_: " << F_.rows << " " << F_.cols << endl;
    }

    if((featType & ACT_F_OBJ) >= 1){
      int m = F_O.rows;
      int n = F_O.cols;
      Mat F2_(1, n, CV_32F);
      for(int j = 0; j < n; j++)
	F2_.at<float>(0,j) = -10; //-inf
      float dist;
      float cy,cx;
      for(int i = 0; i < m; i++){
	cy = centers.at<float>(i,0);
	cx = centers.at<float>(i,1);
	dist = sqrt(pow(cy-gy,2) + pow(cx-gx,2));
	if(dist > radius)
	  continue;
	if(sizes.at<float>(i,0) < sizeThresh)
	  continue;
	for(int j = 0; j < n; j++)
	  F2_.at<float>(0,j) = max(F2_.at<float>(0,j), F_O.at<float>(i,j) );
      }
      F2_ = F2_.t();
      //cout << "F2_: " << F2_.rows << " " << F2_.cols << endl;
      F_.push_back(F2_);
      //cout << "F_: " << F_.rows << " " << F_.cols << endl;
    }

    if((featType & ACT_F_FG) >= 1){
      Mat F3_(1, 1, CV_32F);
      F3_.at<float>(0,0) = 0.0;
      int ggy = floor(gy);
      int ggx = floor(gx);
      if(mask.at<uchar>(ggy,ggx) > 0)
	F3_.at<float>(0,0) = 1.0;
      F_.push_back(F3_);
    }

    if((featType & ACT_F_LOC) >= 1){
      Mat F4_(2, 1, CV_32F);
      F4_.at<float>(0,0) = gy;
      F4_.at<float>(1,0) = gx;
      F_.push_back(F4_);
    }

    F_ = F_.t();
    F.push_back(F_);
    //cout << "F: " << F.rows << " " << F.cols << endl;
  }

  //cout << "(Seq:" << seqNumber << ", Frame:" << frame << ") --> " << F.rows << " " << F.cols << endl;

  return F;
}


/************************************************************************************/
Mat trainingDataActionClassifier_superpixel_interval(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const bool pos_neg, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  // This function contains a bunch of magic numbers!!! I am sorry!

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/tmp/circle_superpixel/trainingDataActionClassifier_", dataDir.c_str());
  if((featType & ACT_F_CIR) >= 1)
    sprintf(buffer, "%scircle", buffer);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%sObject", buffer);

  if((featType & ACT_F_FG) >= 1)
    sprintf(buffer, "%sFG", buffer);

  if((featType & ACT_F_LOC) >= 1)
    sprintf(buffer, "%sLOC", buffer);

  sprintf(buffer, "%s_superpixel_interval_%d_%d_%d_%d_%s_%d_%d", buffer, trainVersion, numBins_lab, numBins_hog, radius, db.getSequence(seqNumber).getSeqName().c_str(), startFrame, endFrame);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%s_%d_%d_%d", buffer, k, k2, k3);

  sprintf(buffer, "%s_%d.mat", buffer, (int)(pos_neg));

  //cout << buffer << endl;
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "trainData");
    return centers;
  }
  ifile.close();

  vector<pair<float,float> > gazeLocations = db.getGazeLocations(seqNumber);

  int fstep = 1;

  if(pos_neg){
    int len = endFrame- startFrame;
    fstep = len / 50;
    if(fstep == 0)
      fstep = 1;
  }
  else{
    int len = endFrame- startFrame;
    fstep = len / 4;
    if(fstep == 0)
      fstep = 1;
  }

  Mat trainData;

  for(int frame = startFrame; frame <= endFrame; frame += fstep){
    float gy = gazeLocations[frame-1].first;
    float gx = gazeLocations[frame-1].second;

    vector<pair<float,float> > locations;

    if(pos_neg){
      int step = 20;
      for(int ii = -step; ii <= step; ii+= step)
	for(int jj = -step; jj <= step; jj+= step)
	  locations.push_back(pair<float,float>(gy+ii,gx+jj));
    }
    else{
      int sampleNum = 10;
      Mat im = db.getIM(seqNumber, frame);
      vector<int> ys = randsample(im.rows-1, sampleNum);
      vector<int> xs = randsample(im.cols-1, sampleNum);
      for(int i = 0; i < sampleNum; i++)
	locations.push_back(pair<float,float>((float)(ys[i]), (float)(xs[i]) ) );
    }
    
    Mat F = getDataActionClassifier_superpixel_frame_location(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, frame, locations, k, k2, k3, objTrainers, featType);
    trainData.push_back(F);
  }

  //cout << "trainData Matrix of Seq " << seqNumber << ", (" << startFrame << "-" << endFrame << "): " << trainData.rows << "x" << trainData.cols << endl;
  //cout << trainData.colRange(0,10) << endl;

  writeMatrixToMatFile(trainData, "trainData", buffer);
  return trainData;
}

/************************************************************************************/
Mat testingDataActionClassifier_superpixel_interval(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  // This function contains a bunch of magic numbers!!! I am sorry!

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/tmp/circle_superpixel/testingDataActionClassifier_", dataDir.c_str());
  if((featType & ACT_F_CIR) >= 1)
    sprintf(buffer, "%scircle", buffer);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%sObject", buffer);

  if((featType & ACT_F_FG) >= 1)
    sprintf(buffer, "%sFG", buffer);

  if((featType & ACT_F_LOC) >= 1)
    sprintf(buffer, "%sLOC", buffer);

  sprintf(buffer, "%s_superpixel_interval_%d_%d_%d_%d_%s_%d_%d", buffer, trainVersion, numBins_lab, numBins_hog, radius, db.getSequence(seqNumber).getSeqName().c_str(), startFrame, endFrame);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%s_%d_%d_%d", buffer, k, k2, k3);

  sprintf(buffer, "%s.mat", buffer);

  //cout << buffer << endl;
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "testData");
    return centers;
  }
  ifile.close();

  vector<pair<float,float> > gazeLocations = db.getGazeLocations(seqNumber);

  int fstep = 1;
  int len = endFrame- startFrame;
  fstep = len / 50;
  if(fstep == 0)
    fstep = 1;
  

  Mat testData;

  for(int frame = startFrame; frame <= endFrame; frame += fstep){
    float gy = gazeLocations[frame-1].first;
    float gx = gazeLocations[frame-1].second;

    vector<pair<float,float> > locations;

    locations.push_back(pair<float,float>(gy,gx));
    
    Mat F = getDataActionClassifier_superpixel_frame_location(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, frame, locations, k, k2, k3, objTrainers, featType);
    testData.push_back(F);
  }

  writeMatrixToMatFile(testData, "testData", buffer);
  return testData;
}

/************************************************************************************/
Mat trainingDataActionClassifier_superpixel_interval_center(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const bool pos_neg, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  if(!pos_neg)
    return trainingDataActionClassifier_superpixel_interval(db, trainVersion, numBins_lab, numBins_hog, radius, pos_neg, seqNumber, startFrame, endFrame, k, k2, k3, objTrainers, featType);


  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/tmp/circle_superpixel/trainingDataActionClassifier_", dataDir.c_str());
  if((featType & ACT_F_CIR) >= 1)
    sprintf(buffer, "%scircle", buffer);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%sObject", buffer);

  if((featType & ACT_F_FG) >= 1)
    sprintf(buffer, "%sFG", buffer);

  if((featType & ACT_F_LOC) >= 1)
    sprintf(buffer, "%sLOC", buffer);

  sprintf(buffer, "%s_superpixel_interval_center_%d_%d_%d_%d_%s_%d_%d", buffer, trainVersion, numBins_lab, numBins_hog, radius, db.getSequence(seqNumber).getSeqName().c_str(), startFrame, endFrame);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%s_%d_%d_%d", buffer, k, k2, k3);

  sprintf(buffer, "%s_%d.mat", buffer, (int)(pos_neg));

  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "trainData");
    return centers;
  }
  ifile.close();

  int fstep = 1;

  int len = endFrame- startFrame;
  fstep = len / 50;
  if(fstep == 0)
    fstep = 1;

  Mat trainData;

  Mat im = db.getIM(seqNumber, startFrame);
  float gy = ((float)(im.rows)) / 2.0;
  float gx = ((float)(im.cols)) / 2.0;
  vector<pair<float,float> > locations;
  locations.push_back(pair<float,float>(gy,gx));

  for(int frame = startFrame; frame <= endFrame; frame += fstep){
    Mat F = getDataActionClassifier_superpixel_frame_location(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, frame, locations, k, k2, k3, objTrainers, featType);
    trainData.push_back(F);
  }

  writeMatrixToMatFile(trainData, "trainData", buffer);
  return trainData; 
}


/************************************************************************************/
Mat testingDataActionClassifier_superpixel_interval_center(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType){

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/tmp/circle_superpixel/testingDataActionClassifier_", dataDir.c_str());
  if((featType & ACT_F_CIR) >= 1)
    sprintf(buffer, "%scircle", buffer);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%sObject", buffer);

  if((featType & ACT_F_FG) >= 1)
    sprintf(buffer, "%sFG", buffer);

  if((featType & ACT_F_LOC) >= 1)
    sprintf(buffer, "%sLOC", buffer);

  sprintf(buffer, "%s_superpixel_interval_center_%d_%d_%d_%d_%s_%d_%d", buffer, trainVersion, numBins_lab, numBins_hog, radius, db.getSequence(seqNumber).getSeqName().c_str(), startFrame, endFrame);

  if((featType & ACT_F_OBJ) >= 1)
    sprintf(buffer, "%s_%d_%d_%d", buffer, k, k2, k3);

  sprintf(buffer, "%s.mat", buffer);

  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "testData");
    return centers;
  }
  ifile.close();

  int fstep = 1;
  int len = endFrame- startFrame;
  fstep = len / 50;
  if(fstep == 0)
    fstep = 1;

  Mat testData;

  Mat im = db.getIM(seqNumber, startFrame);
  float gy = ((float)(im.rows)) / 2.0;
  float gx = ((float)(im.cols)) / 2.0;
  vector<pair<float,float> > locations;
  locations.push_back(pair<float,float>(gy,gx));

  for(int frame = startFrame; frame <= endFrame; frame += fstep){
    Mat F = getDataActionClassifier_superpixel_frame_location(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, frame, locations, k, k2, k3, objTrainers, featType);
    testData.push_back(F);
  }

  writeMatrixToMatFile(testData, "testData", buffer);
  return testData; 
}

/************************************************************************************/
pair<Mat,Mat> trainingDataActionClassifier(string datasetName, Action action, vector<Action>& freqActions, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int pointType){
  Dataset db = dataset_setup(datasetName);

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);

  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);

  vector<string> objNames = db.getObjectNames();
  vector<LinearClassifier> objTrainers;
  for(int o = 0; o < objNames.size(); o++){
    string objectName = objNames[o];
    LinearClassifier objTrainer;
    trainHierarchicalObjectClassifier_svmlight(datasetName, objectName, trainVersion, objTrainer, numBins_lab, numBins_hog, k, k2, k3);
    objTrainers.push_back(objTrainer);
  }

  Mat trainData;
  Mat labels;

  //cout << "Loading Action Data:" << endl;
  //action.print();
  //cout << endl;

  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    vector<pair<float,float> > gazeLocations = db.getGazeLocations(seqNumber);
    vector<Action> actions = db.getActions(seqNumber);
    for(int a = 0; a < actions.size(); a++){
      // check if it is frequent
      bool isFreq = false;
      for(int af = 0; af < freqActions.size(); af++){
	if(areEqualActions(actions[a], freqActions[af])){
	  isFreq = true;
	  break;
	}
      }
      if(!isFreq)
	continue;
      // explore
      cout << ".";
      cout.flush();
      bool posAction = areEqualActions(actions[a], action);

      Mat F;
      if(pointType == ACT_GAZE)
	F = trainingDataActionClassifier_superpixel_interval(db, trainVersion, numBins_lab, numBins_hog, radius, posAction, seqNumber, actions[a].startFrame, actions[a].endFrame, k, k2, k3, objTrainers, featureType);
      if(pointType == ACT_CENTER)
	F = trainingDataActionClassifier_superpixel_interval_center(db, trainVersion, numBins_lab, numBins_hog, radius, posAction, seqNumber, actions[a].startFrame, actions[a].endFrame, k, k2, k3, objTrainers, featureType);
      if(pointType == ACT_FG)
	F = trainingDataActionClassifier_superpixel_interval_foreground(db, trainVersion, numBins_lab, numBins_hog, seqNumber, actions[a].startFrame, actions[a].endFrame, k, k2, k3, objTrainers, featureType);

      trainData.push_back(F);

      Mat labels_ = Mat::ones(F.rows, 1, CV_32F);
      if(!posAction)
	labels_ *= (-1);
      labels.push_back(labels_);
      //cout << trainData.rows << " " << trainData.cols << endl;
      //cout << labels.rows << " " << labels.cols << endl;
    }
  }
  cout << endl;

  pair<Mat,Mat> output;
  output.first = trainData;
  output.second = labels;

  return output;
}

/************************************************************************************/
/*void trainActionClassifier(string datasetName, Action action, vector<Action>& freqActions, const int trainVersion, CvSVM * trainer, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int kernelType){

  Dataset db = dataset_setup(datasetName);
  
  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/Action/trainActionClassifier_%d_%d_%s", dataDir.c_str(), featureType, kernelType, action.verb.c_str());
  for(int i = 0; i < action.names.size(); i++)
    sprintf(buffer, "%s_%s", buffer, action.names[i].c_str());
  sprintf(buffer, "%s_%d_%d_%d_%d.bin", buffer, trainVersion, numBins_lab, numBins_hog, radius);

  //cout << buffer << endl;
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer->load(buffer);
    //cout << "hello " << trainer->get_support_vector_count() << endl;
    return;
  }
  ifile.close();

  pair<Mat, Mat> trainingData = trainingDataActionClassifier(datasetName, action, freqActions, trainVersion, numBins_lab, numBins_hog, radius, k, k2, k3, featureType);
  Mat trainData = trainingData.first;
  Mat labels = trainingData.second;

  CvSVMParams params;
  params.svm_type = CvSVM::C_SVC;
  params.kernel_type = kernelType;
  params.term_crit = cvTermCriteria(CV_TERMCRIT_ITER, 1000, 1e-6);
  
  trainer->train(trainData, labels, Mat(), Mat(), params);
  
  trainer->save(buffer);

  return;
  }*/

/************************************************************************************/
void trainActionClassifier_svmlight(string datasetName, Action action, vector<Action>& freqActions, const int trainVersion, LinearClassifier& trainer, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int pointType){

  Dataset db = dataset_setup(datasetName);
  
  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  if(pointType == ACT_GAZE)
    sprintf(buffer, "%s/opencv/Action/trainActionClassifier_svmlight_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());
  if(pointType == ACT_CENTER)
    sprintf(buffer, "%s/opencv/Action/trainActionClassifier_center_svmlight_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());
  if(pointType == ACT_FG)
    sprintf(buffer, "%s/opencv/Action/trainActionClassifier_foreground_svmlight_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());

  for(int i = 0; i < action.names.size(); i++)
    sprintf(buffer, "%s_%s", buffer, action.names[i].c_str());
  sprintf(buffer, "%s_%d_%d_%d_%d.bin", buffer, trainVersion, numBins_lab, numBins_hog, radius);

  //cout << buffer << endl;
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer.load(buffer);
    trainer.w = -trainer.w;
    trainer.b = -trainer.b;
    return;
  }
  ifile.close();

  char * params = "-t 0";

  char tempFile[500];
  tempFile[0] = 0;
  if(pointType == ACT_GAZE)
    sprintf(tempFile, "%s/opencv/Action/svm_light/trainActionClassifier_svmlight_tempFile_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());
  if(pointType == ACT_CENTER)
    sprintf(tempFile, "%s/opencv/Action/svm_light/trainActionClassifier_center_svmlight_tempFile_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());

  if(pointType == ACT_FG)
    sprintf(tempFile, "%s/opencv/Action/svm_light/trainActionClassifier_foreground_svmlight_tempFile_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());
  
  for(int i = 0; i < action.names.size(); i++)
    sprintf(tempFile, "%s_%s", tempFile, action.names[i].c_str());
  sprintf(tempFile, "%s_%d_%d_%d_%d.bin", tempFile, trainVersion, numBins_lab, numBins_hog, radius);

  //cout << tempFile << endl;
  
  char modelFile[500];
  modelFile[0] = 0;
  if(pointType == ACT_GAZE)
    sprintf(modelFile, "%s/opencv/Action/svm_light/trainActionClassifier_svmlight_modelFile_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());
  if(pointType == ACT_CENTER)
    sprintf(modelFile, "%s/opencv/Action/svm_light/trainActionClassifier_center_svmlight_modelFile_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());

  if(pointType == ACT_FG)
    sprintf(modelFile, "%s/opencv/Action/svm_light/trainActionClassifier_foreground_svmlight_modelFile_%d_%s", dataDir.c_str(), featureType, action.verb.c_str());

  for(int i = 0; i < action.names.size(); i++)
    sprintf(modelFile, "%s_%s", modelFile, action.names[i].c_str());
  sprintf(modelFile, "%s_%d_%d_%d_%d.bin", modelFile, trainVersion, numBins_lab, numBins_hog, radius);

  //cout << modelFile << endl;


  pair<Mat, Mat> trainingData = trainingDataActionClassifier(datasetName, action, freqActions, trainVersion, numBins_lab, numBins_hog, radius, k, k2, k3, featureType, pointType);
  Mat trainData = trainingData.first;
  Mat labels = trainingData.second;

  trainer = svm_light_train(trainData, labels, tempFile, modelFile, params);

  trainer.save(buffer);

  char command[500];
  command[0] = 0;
  sprintf(command, "rm -f %s", tempFile);
  system(command);

  command[0] = 0;
  sprintf(command, "rm -f %s", modelFile);
  system(command);

  trainer.w = -trainer.w;
  trainer.b = -trainer.b;

  return;
}

/************************************************************************************/
Mat testActionClassifier_svmlight_interval(Dataset& db, const int trainVersion, vector<LinearClassifier>& trainers, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, vector<LinearClassifier>& objTrainers, const int featType, const int pointType){

  Mat data;
  if(pointType == ACT_GAZE)
    data = testingDataActionClassifier_superpixel_interval(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, startFrame, endFrame, k, k2, k3, objTrainers, featType);
  if(pointType == ACT_CENTER)
    data = testingDataActionClassifier_superpixel_interval_center(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, startFrame, endFrame, k, k2, k3, objTrainers, featType);
  if(pointType == ACT_FG)
    data = testingDataActionClassifier_superpixel_interval_foreground(db, trainVersion, numBins_lab, numBins_hog, seqNumber, startFrame, endFrame, k, k2, k3, objTrainers, featType);

  //cout << data.rows << " " << data.cols << endl;

  Mat output;
  for(int i = 0; i < trainers.size(); i++){
    LinearClassifier trainer = trainers[i];
    Mat output_ = -(data * trainer.w.t() + trainer.b);
    //cout << output_.rows << " " << output_.cols << endl;
    float total = 0;
    if(output_.rows > 0){
      for(int j = 0; j < output_.rows; j++)
	total += output_.at<float>(j,0);
      total /= output_.rows;
    }
    output.push_back(total);
  }
  output = output.t();

  return output;
}

/************************************************************************************/
pair<Mat, Mat> testActionClassifier_svmlight(string datasetName, vector<LinearClassifier>& trainers, vector<Action>& freqActions, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int pointType){

  Dataset db = dataset_setup(datasetName);

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);

  vector<int> testSeqs = db.getTestSeqs(trainVersion);

  vector<string> objNames = db.getObjectNames();
  vector<LinearClassifier> objTrainers;
  for(int o = 0; o < objNames.size(); o++){
    string objectName = objNames[o];
    LinearClassifier objTrainer;
    trainHierarchicalObjectClassifier_svmlight(datasetName, objectName, trainVersion, objTrainer, numBins_lab, numBins_hog, k, k2, k3);
    objTrainers.push_back(objTrainer);
  }

  int numActions = trainers.size();
  Mat classifications;
  Mat labels;

  for(int i = 0; i < testSeqs.size(); i++){
    int seqNumber = testSeqs[i];
    cout << "Seq Number: " << seqNumber << endl;
    vector<Action> actions = db.getActions(seqNumber);
    int gtActNumber = -1;
    for(int a = 0; a < actions.size(); a++){
      bool isFreq = false;
      for(int af = 0; af < freqActions.size(); af++){
	if(areEqualActions(actions[a], freqActions[af])){
	  isFreq = true;
	  gtActNumber = af;
	  break;
	}
      }
      if(!isFreq){
	continue;
      }

      actions[a].print();
      cout << endl;

      Mat label_ = Mat::ones(1, 1, CV_32S);
      label_ = label_ * gtActNumber;
      labels.push_back(label_);

      actions[a].print();
      cout << " " << seqNumber << " (" << actions[a].startFrame << ", " << actions[a].endFrame << ")";
      cout << " , Action Number: " << gtActNumber << endl;

      Mat output = testActionClassifier_svmlight_interval(db, trainVersion, trainers, numBins_lab, numBins_hog, radius, seqNumber, actions[a].startFrame, actions[a].endFrame, k, k2, k3, objTrainers, featureType, pointType);
      cout << output << endl;

      classifications.push_back(output);
      // cout << endl;
      // cout << output << endl;
      cout << classifications.rows << " " << classifications.cols << endl;
      cout << labels.rows << " " << labels.cols << endl;
    }
  }

  pair<Mat, Mat> answer;
  answer.first = classifications;
  answer.second = labels;
  return answer;
}

/************************************************************************************/
Mat actionLocalization_frame(Dataset& db, const int trainVersion, LinearClassifier& trainer, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int frame, vector<LinearClassifier>& objTrainers, const int k, const int k2, const int k3, const int featureType, int& locY, int& locX){


  int step = 20;
  int margin = 20;
  Mat im = db.getIM(seqNumber, frame);
  vector<pair<float, float> > locations;
  for(int i = margin; i < im.rows-margin; i+= step)
    for(int j = margin; j < im.cols-margin; j+= step)
      locations.push_back(pair<float, float>(i, j));

  //cout << "Number of locations: " << locations.size() << endl;

  Mat data = getDataActionClassifier_superpixel_frame_location(db, trainVersion, numBins_lab, numBins_hog, radius, seqNumber, frame, locations, k, k2, k3, objTrainers, featureType);
  
  //cout << "Data is loaded..." << endl;

  Mat F = -(data * trainer.w.t() + trainer.b);
  int maxY, maxX;
  float bestScore = matrixMaxValue<float>(F, maxY, maxX);
  locY = locations[maxY].first;
  locY = locations[maxY].second;

  Mat answer = Mat::zeros(im.rows, im.cols, CV_32F);
  for(int i = 0; i < locations.size(); i++){
    int y_ind = locations[i].first;
    int x_ind = locations[i].second;
    float value = F.at<float>(i,0);
    for(int y = y_ind-step/2; y < y_ind+step/2; y++)
      for(int x = x_ind-step/2; x < x_ind+step/2; x++)
	answer.at<float>(y,x) = value;
  }
    
  return answer;
}

/************************************************************************************/
void actionLocalization_interval(Dataset& db, const int trainVersion, LinearClassifier& trainer, Action action, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, const int featureType, vector<pair<int,int> >& locations, vector<Mat>& responseMaps){

  cout << "Localizing action in frames..." << endl;

  vector<string> objNames = db.getObjectNames();
  vector<LinearClassifier> objTrainers;
  for(int o = 0; o < objNames.size(); o++){
    string objectName = objNames[o];
    LinearClassifier objTrainer;
    trainHierarchicalObjectClassifier_svmlight(db.getDatasetName(), objectName, trainVersion, objTrainer, numBins_lab, numBins_hog, k, k2, k3);
    objTrainers.push_back(objTrainer);
  }
  
  locations.clear();
  responseMaps.clear();
  for(int frame = startFrame; frame <= endFrame; frame++){
    cout << ".";
    cout.flush();
    int locY, locX;
    Mat response = actionLocalization_frame(db, trainVersion, trainer, numBins_lab, numBins_hog, radius, seqNumber, frame, objTrainers, k, k2, k3, featureType, locY, locX);
    responseMaps.push_back(response);
    locations.push_back(pair<int,int>(locY, locX));
  }
  cout << endl;

  return;
}

/************************************************************************************/
pair<float, float> gazeTransition_1dGaussianForAction(Dataset& db, Action action, const int trainVersion){
  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);
  float miu = 0;
  float var = 0;
  vector<float> shifts;
  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    vector<pair<float,float> > gazeLocations = db.getGazeLocations(seqNumber);
    vector<Action> tempActions = db.getActions(seqNumber);
    for(int a = 0; a < tempActions.size(); a++){
      if(areEqualActions(tempActions[a], action)){
	Mat im = db.getIM(seqNumber, tempActions[a].startFrame);
	int im_rows = im.rows;
	int im_cols = im.cols;
	for(int frame = tempActions[a].startFrame; frame <= tempActions[a].endFrame-1; frame++){
	  float gy1 = gazeLocations[frame].first;
	  float gx1 = gazeLocations[frame].second;
	  float gy2 = gazeLocations[frame+1].first;
	  float gx2 = gazeLocations[frame+1].second;
	  if(gy1 < 0 || gy2 < 0)
	    continue;
	  if(gx1 < 0 || gx2 < 0)
	    continue;
	  if(gy1 >= im_rows || gy2 >= im_rows)
	    continue;
	  if(gx1 >= im_cols || gx2 >= im_cols)
	    continue;
	  shifts.push_back(sqrt(pow(gy2-gy1,2.0)+pow(gx2-gx1,2.0)));
	}
      }
    }
  }

  //cout << "Number of transitions between gaze points: " << shifts.size() << endl;
  if(shifts.size() > 2){
    for(int i = 0; i < shifts.size(); i++)
      miu += shifts[i];
    miu /= shifts.size();
    for(int i = 0; i < shifts.size(); i++)
      var += pow(shifts[i]-miu, 2.0);
    var /= shifts.size();
  }

  pair<float, float> answer;
  answer.first = miu;
  answer.second = var;
  return answer;
}

/************************************************************************************/
pair<Mat, Mat > gazeLocation_2dGaussianForAction(Dataset& db, Action action, const int trainVersion){
  
  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);
  Mat miu = Mat::zeros(1, 2, CV_32F);
  Mat Sigma = Mat::zeros(2, 2, CV_32F);

  vector<Mat > points;

  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    vector<pair<float,float> > gazeLocations = db.getGazeLocations(seqNumber);
    vector<Action> tempActions = db.getActions(seqNumber);
    for(int a = 0; a < tempActions.size(); a++){
      if(areEqualActions(tempActions[a], action)){
	Mat im = db.getIM(seqNumber, tempActions[a].startFrame);
	int im_rows = im.rows;
	int im_cols = im.cols;
	for(int frame = tempActions[a].startFrame; frame <= tempActions[a].endFrame; frame++){
	  float gy = gazeLocations[frame].first;
	  float gx = gazeLocations[frame].second;
	  if(gy < 0)
	    continue;
	  if(gx < 0)
	    continue;
	  if(gy >= im_rows)
	    continue;
	  if(gx >= im_cols)
	    continue;
	  Mat point_(2, 1, CV_32F);
	  point_.at<float>(0,0) = gy;
	  point_.at<float>(1,0) = gx;
	  points.push_back(point_);
	}
      }
    }
  }

  if(points.size() > 2){
    /*for(int i = 0; i < points.size(); i++)
      miu += points[i];
      miu /= points.size();*/
    Mat points2(points.size(), 2, CV_32F);
    for(int i = 0; i < points.size(); i++){
      points2.at<float>(i,0) = points[i].at<float>(0,0);
      points2.at<float>(i,1) = points[i].at<float>(1,0);
    }
    //cout << points2 << endl;
    //cout << points2.rows << " " << points2.cols << endl;
    
    calcCovarMatrix(points2, Sigma, miu, CV_COVAR_NORMAL | CV_COVAR_ROWS, CV_32F);
  }

  pair<Mat, Mat> answer;
  answer.first = miu;
  answer.second = Sigma;
  return answer;
}
/************************************************************************************/





