#ifndef ACTION_STUFF_H
#define ACTION_STUFF_H

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/ml/ml.hpp>

#include "DatasetStuff.h"
#include "SequenceStuff.h"
#include "TrainStuff.h"


//enum{ACT_F_CIR, ACT_F_CIR_OBJ, ACT_F_CIR_OBJ_FG, ACT_F_CIR_OBJ_LOC};
//enum{ACT_GAZE, ACT_CENTER};
static int ACT_F_CIR = 1;
static int ACT_F_OBJ = 2;
static int ACT_F_FG = 4;
static int ACT_F_LOC = 8;

static int ACT_GAZE = 1;
static int ACT_CENTER = 2;
static int ACT_FG = 4;

/**********************************************************************/
/* Action given foreground */

cv::Mat getDataActionClassifier_superpixel_frame_foreground(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int seqNumber, const int frame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

cv::Mat trainingDataActionClassifier_superpixel_interval_foreground(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

cv::Mat testingDataActionClassifier_superpixel_interval_foreground(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

/**********************************************************************/
/*  Action given gaze */

cv::Mat getDataActionClassifier_superpixel_frame_location(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int frame, std::vector<std::pair<float,float> > locations, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

cv::Mat trainingDataActionClassifier_superpixel_interval(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const bool pos_neg, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

cv::Mat testingDataActionClassifier_superpixel_interval(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

cv::Mat trainingDataActionClassifier_superpixel_interval_center(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const bool pos_neg, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);

cv::Mat testingDataActionClassifier_superpixel_interval_center(Dataset& db, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType);


std::pair<cv::Mat,cv::Mat> trainingDataActionClassifier(std::string datasetName, Action action, std::vector<Action>& freqActions, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int pointType);

//void trainActionClassifier(std::string datasetName, Action action, std::vector<Action>& freqActions, const int trainVersion, CvSVM * trainer, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int kernelType);

void trainActionClassifier_svmlight(std::string datasetName, Action action, std::vector<Action>& freqActions, const int trainVersion, LinearClassifier& trainer, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int pointType);

/* returns a 1xm matrix which contains the response of m action classifiers on the data of the sequence */
cv::Mat testActionClassifier_svmlight_interval(Dataset& db, const int trainVersion, std::vector<LinearClassifier>& trainers, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, std::vector<LinearClassifier>& objTrainers, const int featType, const int pointType);

/* returns the confusion matrix */
std::pair<cv::Mat, cv::Mat> testActionClassifier_svmlight(std::string datasetName, std::vector<LinearClassifier>& trainers, std::vector<Action>& freqActions, const int trainVersion, const int numBins_lab, const int numBins_hog, const int radius, const int k, const int k2, const int k3, const int featureType, const int pointType);



/**********************************************************************/
/* Gaze Prediction given Action */

cv::Mat actionLocalization_frame(Dataset& db, const int trainVersion, LinearClassifier& trainer, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int frame, std::vector<LinearClassifier>& objTrainers, const int k, const int k2, const int k3, const int featureType, int& locY, int& locX);

void actionLocalization_interval(Dataset& db, const int trainVersion, LinearClassifier& trainer, Action action, const int numBins_lab, const int numBins_hog, const int radius, const int seqNumber, const int startFrame, const int endFrame, const int k, const int k2, const int k3, const int featureType, std::vector<std::pair<int,int> >& locations, std::vector<cv::Mat>& responseMaps);

std::pair<float, float> gazeTransition_1dGaussianForAction(Dataset& db, Action action, const int trainVersion);

std::pair<cv::Mat, cv::Mat > gazeLocation_2dGaussianForAction(Dataset& db, Action action, const int trainVersion);


#endif //ACTION_STUFF_H
