#include "DatasetStuff.h"
#include "GazeStuff.h"
#include "InputOutputFile.h"
#include "MyMATFileInterface.h"
#include <iostream>
#include <fstream>

using namespace std;
using namespace cv;

/*****************************************************************************************/
void Action::removeBadChars(){
  for(int i = 0; i < verb.length(); i++)
    if(verb[i] == ' ' || verb[i] == '-')
      verb[i] = '_';
  for(int o = 0; o < names.size(); o++)
    for(int i = 0; i < names[o].length(); i++)
      if(names[o][i] == ' ' || names[o][i] == '-')
	names[o][i] = '_';
  return;
}

/*****************************************************************************************/
void Action::makeAllCharsSmall(){
  for(int i = 0; i < verb.length(); i++)
    if(verb[i] >= 'A' && verb[i] <= 'Z')
      verb[i] = verb[i] + 'a' - 'A';
  for(int o = 0; o < names.size(); o++)
    for(int i = 0; i < names[o].length(); i++)
      if(names[o][i] >= 'A' && names[o][i] <= 'Z')
	names[o][i] = names[o][i] + 'a' - 'A';
  return;  
}

/*****************************************************************************************/
string Action::getActionName() const{
  string output("");
  output.append("<");
  output.append(verb);
  output.append("><");
  for(int i = 0; i < names.size()-1; i++){
    output.append(names[i]);
    output.append(",");
  }
  output.append(names[names.size()-1]);
  output.append(">");
    
  return output;
}

/*****************************************************************************************/
void Action::print() const{
  /*cout << "<" << verb.c_str() << "><";
  for(int i = 0; i < names.size()-1; i++)
    cout << names[i].c_str() << ",";
    cout << names[names.size()-1].c_str() << ">";;*/
  cout << getActionName().c_str();
  cout.flush();
  return;
}


/*****************************************************************************************/
bool areEqualActions(Action a1, Action a2){
  if(strcmp(a1.verb.c_str(), a2.verb.c_str()) != 0)
    return false;
  if(a1.names.size() != a2.names.size())
    return false;
  for(int i = 0; i < a1.names.size(); i++)
    if(strcmp(a1.names[i].c_str(), a2.names[i].c_str()) != 0) 
      return false;
  return true;
}

/*****************************************************************************************/
bool areEqualActions_verb(Action a1, Action a2){
  if(strcmp(a1.verb.c_str(), a2.verb.c_str()) != 0)
    return false;
  return true;
}

/*****************************************************************************************/
bool Action_classcomp::operator() (const Action& a1, const Action& a2) const{
  if(a1.verb.compare(a2.verb) < 0)
    return true; 
  if(a1.verb.compare(a2.verb) > 0)
    return false;
  if(a1.names.size() < a2.names.size())
    return true;
  if(a1.names.size() > a2.names.size())
    return false;
  for(int i = 0; i < a1.names.size(); i++){
    if(a1.names[i].compare(a2.names[i]) < 0)
      return true;
    if(a1.names[i].compare(a2.names[i]) > 0)
      return false;
  }
  return false;
}

/*****************************************************************************************/
vector<Action> readActionAnnotations_SMI(const char * filename, const int fps){
  vector<Action> actions;
  
  ifstream infile(filename, ios::in);
  int limit = 500;
  char line[limit];
  while(infile){
    line[0] = 0;
    infile.getline(line, limit);
    if(!infile)
      break;
    Action tempAction;
    char * tier = strtok(line, "\t");
    //cout << tier << endl;
    char * startTime = strtok(NULL, "\t");
    //cout << startTime << endl;
    char * endTime = strtok(NULL, "\t");
    //cout << endTime << endl;
    char * action = strtok(NULL, "\t");
    char * actionVerb = strtok(action, "(");
    tempAction.verb = string(actionVerb);
    char * actionNames = strtok(NULL, ")");

    char * name = strtok(actionNames, ",");
    //cout << name << " ";
    while(name != NULL){
      string namestr(name);
      while(namestr[0] == ' '){
	namestr = namestr.substr(1);
	if(namestr.length() < 1)
	  break;
      }
      tempAction.names.push_back(namestr);
      if(tempAction.names.size() >= 3)
	break;
      //cout << namestr.c_str() << ",";
      name = strtok(NULL, ",");
    }
    //cout << endl;

    char * hour_s = strtok(startTime, ":");
    char * minute_s = strtok(NULL, ":");
    char * second_s = strtok(NULL, ".");
    char * milisecond_s = strtok(NULL, ",");
    int h_s = atoi(hour_s);
    int m_s = atoi(minute_s);
    int s_s = atoi(second_s);
    int ml_s = atoi(milisecond_s);
    //cout << hour_s << "," << minute_s << "," << second_s << "," << milisecond_s << endl;

    char * hour_t = strtok(endTime, ":");
    char * minute_t = strtok(NULL, ":");
    char * second_t = strtok(NULL, ".");
    char * milisecond_t = strtok(NULL, ",");
    int h_t = atoi(hour_t);
    int m_t = atoi(minute_t);
    int s_t = atoi(second_t);
    int ml_t = atoi(milisecond_t);
    //cout << h_t << "," << m_t << "," << s_t << "," << ml_t << ": ";

    tempAction.startFrame = timeToFrame(h_s, m_s, s_s, ml_s, fps);
    tempAction.endFrame = timeToFrame(h_t, m_t, s_t, ml_t, fps);
    //cout << tempAction.endFrame << endl;
    tempAction.removeBadChars();
    tempAction.makeAllCharsSmall();
    actions.push_back(tempAction);
  }

  /*for(int i = 0; i < actions.size(); i++){
    cout << actions[i].verb.c_str() << "(";
    for(int j = 0; j < actions[i].names.size(); j++){
      cout << actions[i].names[j].c_str() << ",";
    }
    cout << ") ";
    cout << actions[i].startFrame << "-" << actions[i].endFrame << endl;
    }*/

  infile.close();
  return actions;
}

/*****************************************************************************************/
Mat Dataset::getIM(const int seqNumber, const int frameNumber){
  string impath = getImagePath(seqNumber, frameNumber);
  Mat im = imread(impath.c_str());
  if(strcmp(dbname_.c_str(), "GTDA_v3") == 0){
    int w_rem = 90;
    int h_rem = 45;
    int wdth = im.cols;
    im = im.colRange(w_rem, wdth);
    wdth = im.cols;
    im = im.colRange(0, wdth-w_rem);
    int hght = im.rows;
    im = im.rowRange(h_rem, hght);
    hght = im.rows;
    im = im.rowRange(0, hght-h_rem);
  }
  if(scale_ < 1){
    Mat im2(im.rows*scale_, im.cols*scale_, CV_8UC3);
    resize(im, im2, im2.size(), 0, 0, INTER_LINEAR);
    return im2;
  }
  //cout << im.rows << " " << im.cols << endl;
  return im;
}

/*****************************************************************************************/
Mat Dataset::readUCM(const int seqNumber, const int frameNumber){
  string ucmPath = getContourPath(seqNumber, frameNumber);
  Mat UCM = ::readUCM(ucmPath.c_str());
  if(strcmp(dbname_.c_str(), "GTDA_v3") == 0){
    int w_rem = 180;
    int h_rem = 90;
    int wdth = UCM.cols;
    UCM = UCM.colRange(w_rem, wdth);
    wdth = UCM.cols;
    UCM = UCM.colRange(0, wdth-w_rem);
    int hght = UCM.rows;
    UCM = UCM.rowRange(h_rem, hght);
    hght = UCM.rows;
    UCM = UCM.rowRange(0, hght-h_rem);
  }
  //cout << UCM.rows << " " << UCM.cols << endl;
  return UCM;
}

/*****************************************************************************************/
string Dataset::getImagePath(const int seqNumber, const int frameNumber){
  return ::getImagePath(dataDir_.c_str(), sequences_[seqNumber].getSeqName().c_str(), imType_.c_str(), frameNumber);
}

/*****************************************************************************************/
string Dataset::getContourPath(const int seqNumber, const int frameNumber){
  return ::getContourPath(dataDir_.c_str(), sequences_[seqNumber].getSeqName().c_str(), frameNumber);
}

/*****************************************************************************************/
string Dataset::getGTObjectMaskPath(const std::string& objectName, const int seqNumber, const int frameNumber){
  char buffer[300];
  buffer[0] = 0;
  string seqName = sequences_[seqNumber].getSeqName();
  sprintf(buffer, "%s/opencv/groundtruth/%s/%s/%s_%s_%010d.pgm", dataDir_.c_str(), seqName.c_str(), objectName.c_str(), objectName.c_str(), seqName.c_str(), frameNumber);
  return string(buffer);
}

Mat Dataset::getGTObjectMask(const std::string& objectName, const int seqNumber, const int frameNumber){
  string gtPath = getGTObjectMaskPath(objectName, seqNumber, frameNumber);
  Mat mask = readMaskFromImage(gtPath.c_str());
  if(scale_ < 1 && mask.rows > 1){
    //cout << scale_ << endl;
    Mat mask2(mask.rows*scale_, mask.cols*scale_, mask.type());
    //cout << mask2.rows << " " << mask2.cols << endl;
    resize(mask, mask2, mask2.size(), 0, 0, INTER_NEAREST);
    return mask2;
  }
  return mask;
}

void Dataset::writeGTObjectMask(const string& objectName, const int seqNumber, const int frameNumber, const Mat& mask){
  string gtPath = getGTObjectMaskPath(objectName, seqNumber, frameNumber);
  writeMaskAsImage(gtPath.c_str(), mask);
  return;
}

/******************************************************************************/
string Dataset::getFGSegmentationPath(const int seqNumber, const int frameNumber, const int trainVersion){
  string seqName = getSequence(seqNumber).getSeqName();
  char buffer[300];
  buffer[0] = 0;
  sprintf(buffer, "%s/opencv/%s/FGSegmentation_%d_%g_0/%010d.pgm", dataDir_.c_str(), seqName.c_str(), trainVersion, scale_, frameNumber);
  return string(buffer);
}

/******************************************************************************/
Mat Dataset::readForegroundMask(const int seqNumber, const int frame, const int trainVersion){
  string path = getFGSegmentationPath(seqNumber, frame, trainVersion);
  Mat mask;
  if(fileExists(path.c_str()))
    mask = ::readMaskFromImage(path.c_str());
  else
    cout << "Mask file does not exist: " << path.c_str() << endl;
  return mask;
}


/*****************************************************************************************/
string Dataset::getGazePath(const int seqNumber){
  char buffer[300];
  buffer[0] = 0;
  string seqName = sequences_[seqNumber].getSeqName();
  sprintf(buffer, "%s/Gaze/%s.txt", dataDir_.c_str(), seqName.c_str());
  return string(buffer);
}

/*****************************************************************************************/
vector<Action> Dataset::getActions(const int seqNumber){
  if(actions_.find(seqNumber) != actions_.end())
    return actions_[seqNumber];

  vector<Action> seqActions;
  string seqName = sequences_[seqNumber].getSeqName();

  if(strcmp(dbname_.c_str(), "GTDA_v3") == 0){
    char filename[500];
    filename[0] = 0;
    sprintf(filename, "%s/ELAN/%s.txt", dataDir_.c_str(), seqName.c_str());
    seqActions = readActionAnnotations_SMI(filename, fps_);
  }

  actions_[seqNumber] = seqActions;
  return actions_[seqNumber];
}

/*****************************************************************************************/
int Dataset::getActionFreqInTraining(const Action action, const int trainVersion){
  vector<int> seqs = getTrainSeqs(trainVersion);
  int answer = 0;
  for(int i = 0; i < seqs.size(); i++){
    int seqNumber = seqs[i];
    vector<Action> actions_ = getActions(seqNumber);
    for(int a = 0; a < actions_.size(); a++)
      if(areEqualActions(actions_[a], action))
	answer++;
  }
  return answer;
}

/*****************************************************************************************/
int Dataset::getActionFreqInTesting(const Action action, const int trainVersion){
  vector<int> seqs = getTestSeqs(trainVersion);
  int answer = 0;
  for(int i = 0; i < seqs.size(); i++){
    int seqNumber = seqs[i];
    vector<Action> actions_ = getActions(seqNumber);
    for(int a = 0; a < actions_.size(); a++)
      if(areEqualActions(actions_[a], action))
	answer++;
  }
  return answer;
}

/*****************************************************************************************/
vector<Action> Dataset::frequentActionsInTraining(const int trainVersion, const int minFreq){
  vector<int> seqs = getTrainSeqs(trainVersion);
  map<Action, int, Action_classcomp> actionCount;
  for(int i = 0; i < seqs.size(); i++){
    int seqNumber = seqs[i];
    vector<Action> actions_ = getActions(seqNumber);
    for(int a = 0; a < actions_.size(); a++){
      if(actionCount.find(actions_[a]) != actionCount.end())
	actionCount[actions_[a]]++;
      else
	actionCount[actions_[a]] = 1;
    }
  }
  vector<Action> output;
  for(map<Action,int>::iterator it = actionCount.begin(); it != actionCount.end(); it++){
    if(it->second >= minFreq)
      output.push_back(it->first);
  }
  return output;
}

/*****************************************************************************************/
vector<Action> Dataset::frequentActionsInTesting(const int trainVersion, const int minFreq){
  vector<int> seqs = getTestSeqs(trainVersion);
  map<Action, int, Action_classcomp> actionCount;
  for(int i = 0; i < seqs.size(); i++){
    int seqNumber = seqs[i];
    vector<Action> actions_ = getActions(seqNumber);
    for(int a = 0; a < actions_.size(); a++){
      if(actionCount.find(actions_[a]) != actionCount.end())
	actionCount[actions_[a]]++;
      else
	actionCount[actions_[a]] = 1;
    }
  }
  vector<Action> output;
  for(map<Action,int>::iterator it = actionCount.begin(); it != actionCount.end(); it++){
    if(it->second >= minFreq)
      output.push_back(it->first);
  }
  return output;
}

/*****************************************************************************************/
vector<Action> Dataset::frequentActions(const int trainVersion, const int minFreqTraining, const int minFreqTesting){
  vector<Action> actionsTraining = frequentActionsInTraining(trainVersion, minFreqTraining);
  vector<Action> actionsTesting = frequentActionsInTesting(trainVersion, minFreqTesting);
  vector<Action> output;
  for(int a1 = 0; a1 < actionsTraining.size(); a1++){
    bool existInTest = false;
    for(int a2 = 0; a2 < actionsTesting.size(); a2++){
      if(areEqualActions(actionsTraining[a1], actionsTesting[a2])){
	//actionsTraining[a1].print();
	//cout << endl;
	existInTest = true;
	break;
      }
    }
    if(existInTest)
      output.push_back(actionsTraining[a1]);
  }

  vector<Action> freqActions;

  for(int i = 0; i < output.size(); i++){
    //if(strcmp(output[i].verb.c_str(), "put") == 0)
    //continue;
    freqActions.push_back(output[i]);
  }

  return freqActions;
}


/*****************************************************************************************/
vector<Action> Dataset::getSeqFreqActions(const int seqNumber, const vector<Action> freqActions_){
  vector<Action> output;

  vector<Action> freqActions;
  for(int i = 0; i < freqActions_.size(); i++)
    freqActions.push_back(freqActions_[i]);

  vector<Action> actions = getActions(seqNumber);
  for(int a = 0; a < actions.size(); a++){
    bool isFreq = false;
    for(int af = 0; af < freqActions.size(); af++){
      if(areEqualActions(actions[a], freqActions[af])){
	isFreq = true;
	break;
      }
    }
    if(!isFreq)
      continue;
    output.push_back(actions[a]);
  }

  return output;
}


/*****************************************************************************************/
vector<pair<float,float> > Dataset::getGazeLocations(const int seqNumber){
  if(gazeLocations_.find(seqNumber) != gazeLocations_.end())
    return gazeLocations_[seqNumber];

  int numFrames = sequences_[seqNumber].getSeqLength();
  string gazePath = getGazePath(seqNumber);

  int marginY = 0;
  int marginX = 0;
  if(strcmp(dbname_.c_str(), "GTDA_v3") == 0){
    marginY = 45;
    marginX = 90;
  }

  Mat data = readGazePoints_SMI(gazePath.c_str(), scale_*resized_, marginY, marginX);
  vector<pair<float,float> > gazes = rawGazePointsToFrameLocations_SMI(data, numFrames, fps_);
  gazeLocations_[seqNumber] = gazes;
  return gazes;
}

/*****************************************************************************************/
pair<float,float> Dataset::getGazeLocation(const int seqNumber, const int frame){
  if(gazeLocations_.find(seqNumber) != gazeLocations_.end())
    return gazeLocations_[seqNumber][frame];

  vector<pair<float,float> > answer_ = getGazeLocations(seqNumber);
  pair<float,float> answer = answer_[frame];
  return answer;
}

/*****************************************************************************************/
Dataset dataset_setup(const std::string& datasetName){
  if(strcmp(datasetName.c_str(), "GTDA_v3") == 0){
    return dataset_setup_GTDA_v3();
  }

  if(strcmp(datasetName.c_str(), "GTDA_v4") == 0){
    return dataset_setup_GTDA_v4();
  }
  
}

/*****************************************************************************************/
Dataset dataset_setup_GTDA_v3(){
  Dataset output("GTDA_v3", string("/media/V3/GTDA_v3"), 1.0, 0.75, 24, string("png"));
  output.addSequence(1, DatasetSeq(string("SMI_Yin_Turkey"), 12046)); // 1
  output.addSequence(2, DatasetSeq(string("SMI_Yin_American"), 18615)); // 2
  output.addSequence(3, DatasetSeq(string("SMI_Yin_Snack"), 10451)); // 3
  output.addSequence(4, DatasetSeq(string("SMI_Yin_Hamburger"), 16329)); // 4
  output.addSequence(5, DatasetSeq(string("SMI_Alireza_Turkey"), 9180)); // 5
  output.addSequence(6, DatasetSeq(string("SMI_Alireza_American"), 19844)); // 6
  output.addSequence(7, DatasetSeq(string("SMI_Alireza_Snack"), 11255)); // 7
  output.addSequence(8, DatasetSeq(string("SMI_Alireza_Hamburger"), 10727)); // 8
  output.addSequence(9, DatasetSeq(string("SMI_Alireza_Greek"), 11199)); // 9
  output.addSequence(10, DatasetSeq(string("SMI_Alireza_Pizza"), 16632)); // 10
  output.addSequence(11, DatasetSeq(string("SMI_Alireza_Pasta"), 13024)); // 11
  output.addSequence(12, DatasetSeq(string("SMI_Yin_Pizza"), 15757)); // 12
  output.addSequence(13, DatasetSeq(string("SMI_Yin_Greek"), 11614)); // 13
  output.addSequence(14, DatasetSeq(string("SMI_Alireza_Pizza2"), -1)); // 14
  output.addSequence(15, DatasetSeq(string("SMI_Yin_Tilapia2"), -1)); // 15
  output.addSequence(16, DatasetSeq(string("SMI_Yin_Tilapia3"), -1)); // 16
  output.addSequence(17, DatasetSeq(string("SMI_Shaghayegh_Pizza"), 9725)); // 17
  output.addSequence(18, DatasetSeq(string("SMI_Shaghayegh_American"), 17216)); // 18
  output.addSequence(19, DatasetSeq(string("SMI_Shaghayegh_Snack"), 11385)); // 19
  output.addSequence(20, DatasetSeq(string("SMI_Rahul_Pizza"), 15178)); // 20
  output.addSequence(21, DatasetSeq(string("SMI_Rahul_American"), 20536)); // 21
  output.addSequence(22, DatasetSeq(string("SMI_Rahul_Snack"), 13027)); // 22
  output.addSequence(23, DatasetSeq(string("SMI_Carlos_Pizza"), 17929)); // 23
  output.addSequence(24, DatasetSeq(string("SMI_Carlos_American"), 20120)); // 24
  output.addSequence(25, DatasetSeq(string("SMI_Carlos_Snack"), 10674)); // 25

  // trainVersion 1
  vector<int> trainS;
  trainS.push_back(2);
  trainS.push_back(3);
  trainS.push_back(12);
  output.setTrainSeqs(1, trainS);
  output.setTestSeqs(1, trainS);

  // trainVersion 2
  trainS.clear();
  trainS.push_back(2);
  trainS.push_back(3);
  trainS.push_back(12);
  trainS.push_back(17);
  trainS.push_back(18);
  trainS.push_back(19);
  output.setTrainSeqs(2, trainS);
  vector<int> testS;
  testS.push_back(6);
  testS.push_back(7);
  testS.push_back(10);
  output.setTestSeqs(2, testS);

  // trainVersion 3
  trainS.clear();
  trainS.push_back(2);
  trainS.push_back(3);
  trainS.push_back(12);
  trainS.push_back(17);
  trainS.push_back(18);
  trainS.push_back(19);
  trainS.push_back(20);
  trainS.push_back(21);
  trainS.push_back(22);
  output.setTrainSeqs(3, trainS);
  testS.clear();
  testS.push_back(6);
  testS.push_back(7);
  testS.push_back(10);
  testS.push_back(23);
  testS.push_back(24);
  testS.push_back(25);
  output.setTestSeqs(3, testS);
  

  output.pushObjectName("Bacon");
  output.pushObjectName("Bacon_Container");
  output.pushObjectName("Bagel");
  output.pushObjectName("Bagel_Container");
  output.pushObjectName("Blunt_Knife");
  output.pushObjectName("Bowl");
  output.pushObjectName("Bread");
  output.pushObjectName("Bread_Container");
  output.pushObjectName("Catsup_Container");
  output.pushObjectName("Cereal");
  output.pushObjectName("Cereal_Container");
  output.pushObjectName("Cream_Cheese_Container");
  output.pushObjectName("Cup");
  output.pushObjectName("Egg");
  output.pushObjectName("Egg_Container");
  output.pushObjectName("Freezer");
  output.pushObjectName("Fridge");
  output.pushObjectName("Green_Pepper");
  output.pushObjectName("Hand");
  output.pushObjectName("Honey_Container");
  output.pushObjectName("Jam_Container");
  output.pushObjectName("Kettle");
  output.pushObjectName("Microwave");
  output.pushObjectName("Milk_Container");
  output.pushObjectName("Mozzarella_Container");
  output.pushObjectName("Mushroom");
  output.pushObjectName("Oil_Container");
  output.pushObjectName("Olive_Oil_Container");
  output.pushObjectName("Oraneg_Juice");
  output.pushObjectName("Oraneg_Juice_Container");
  output.pushObjectName("Oven");
  output.pushObjectName("Peanut_Butter_Container");
  output.pushObjectName("Pizza_Crust");
  output.pushObjectName("Pizza_Crust_Container");
  output.pushObjectName("Plastic_Spoon");
  output.pushObjectName("Plate");
  output.pushObjectName("Salt_Container");
  output.pushObjectName("Sink");
  output.pushObjectName("Skillet");

  return output;
}

/*****************************************************************************************/
Dataset dataset_setup_GTDA_v4(){
  Dataset output("GTDA_v4", string("/home/alireza/Data/GTDA_v4"), 1.0, 0.5, 15, string("png"));
  output.addSequence(3, DatasetSeq(string("003"), 4957)); // 1

  // trainVersion 1
  vector<int> trainS;
  trainS.push_back(3);
  output.setTrainSeqs(1, trainS);
  output.setTestSeqs(1, trainS);
  
  return output;
}

/*****************************************************************************************/
Scalar Dataset::getObjectColor(const std::string& objName){
  Scalar output = cvScalar(128, 128, 128); //unknown
  if(strcmp(objName.c_str(),"Bacon") == 0)
    output = cvScalar(42, 123, 255);
  if(strcmp(objName.c_str(),"Bacon_Container") == 0)
    output = cvScalar(88, 166, 255);
  if(strcmp(objName.c_str(),"Bagel") == 0)
    output = cvScalar(46, 119, 228);
  if(strcmp(objName.c_str(),"Bagel_Container") == 0)
    output = cvScalar(3, 45, 125);
  if(strcmp(objName.c_str(),"Blunt_Knife") == 0)
    output = cvScalar(23, 80, 143);
  if(strcmp(objName.c_str(),"Bowl") == 0)
    output = cvScalar(32, 126, 224);
  if(strcmp(objName.c_str(),"Bowl_Container") == 0)
    output = cvScalar(32, 126, 224);
  if(strcmp(objName.c_str(),"Bread") == 0)
    output = cvScalar(41, 96, 179);
  if(strcmp(objName.c_str(),"Bread_Container") == 0)
    output = cvScalar(98, 140, 211);
  if(strcmp(objName.c_str(),"Catsup_Container") == 0)
    output = cvScalar(0, 11, 66);
  if(strcmp(objName.c_str(),"Cereal") == 0)
    output = cvScalar(0, 65, 168);
  if(strcmp(objName.c_str(),"Cereal_Container") == 0)
    output = cvScalar(44, 84, 147);
  if(strcmp(objName.c_str(),"Cream_Cheese_Container") == 0)
    output = cvScalar(74, 182, 255);
  if(strcmp(objName.c_str(),"Cup") == 0)
    output = cvScalar(32, 73, 120);
  if(strcmp(objName.c_str(),"Egg") == 0)
    output = cvScalar(129, 190, 255);
  if(strcmp(objName.c_str(),"Egg_Container") == 0)
    output = cvScalar(79, 84, 88);
  if(strcmp(objName.c_str(),"Egg_Mix") == 0)
    output = cvScalar(77, 214, 255);
  if(strcmp(objName.c_str(),"Freezer") == 0)
    output = cvScalar(228, 209, 182);
  if(strcmp(objName.c_str(),"Fridge") == 0)
    output = cvScalar(9, 40, 69);
  if(strcmp(objName.c_str(),"Green_Pepper") == 0)
    output = cvScalar(46, 77, 59);
  if(strcmp(objName.c_str(),"Hand") == 0)
    output = cvScalar(13, 58, 159);
  if(strcmp(objName.c_str(),"Honey_Container") == 0)
    output = cvScalar(15, 65, 193);
  if(strcmp(objName.c_str(),"Jam_Container") == 0)
    output = cvScalar(13, 72, 255);
  if(strcmp(objName.c_str(),"Kettle") == 0)
    output = cvScalar(12, 45, 90);
  if(strcmp(objName.c_str(),"Microwave") == 0)
    output = cvScalar(104, 155, 209);
  if(strcmp(objName.c_str(),"Milk_Container") == 0)
    output = cvScalar(224, 200, 172);
  if(strcmp(objName.c_str(),"Mozzarella_Container") == 0)
    output = cvScalar(122, 183, 255);
  if(strcmp(objName.c_str(),"Mushroom") == 0)
    output = cvScalar(207, 211, 204);
  if(strcmp(objName.c_str(),"Oil_Container") == 0)
    output = cvScalar(56, 133, 212);
  if(strcmp(objName.c_str(),"Olive_Oil_Container") == 0)
    output = cvScalar(81, 161, 240);
  if(strcmp(objName.c_str(),"Oraneg_Juice") == 0)
    output = cvScalar(48, 138, 225);
  if(strcmp(objName.c_str(),"Oraneg_Juice_Container") == 0)
    output = cvScalar(48, 138, 255);
  if(strcmp(objName.c_str(),"Oven") == 0)
    output = cvScalar(19, 59, 93);
  if(strcmp(objName.c_str(),"Peanut_Butter_Container") == 0)
    output = cvScalar(18, 86, 192);
  if(strcmp(objName.c_str(),"Pizza_Crust") == 0)
    output = cvScalar(60, 123, 212);
  if(strcmp(objName.c_str(),"Pizza_Crust_Container") == 0)
    output = cvScalar(41, 70, 153);
  if(strcmp(objName.c_str(),"Plastic_Spoon") == 0)
    output = cvScalar(0, 19, 40);
  if(strcmp(objName.c_str(),"Plate") == 0)
    output = cvScalar(46, 91, 161);
  if(strcmp(objName.c_str(),"Plate_Container") == 0)
    output = cvScalar(46, 91, 161);
  if(strcmp(objName.c_str(),"Salt_Container") == 0)
    output = cvScalar(18, 109, 198);
  if(strcmp(objName.c_str(),"Sink") == 0)
    output = cvScalar(0, 26, 46);
  if(strcmp(objName.c_str(),"Skillet") == 0)
    output = cvScalar(0, 39, 81);
  
  
  return output;
}

/*****************************************************************************************/
Mat Dataset::getObjectColors(const std::vector<std::string>& objNames){
  int n = objNames.size();
  Mat colors(n, 3, CV_8U);
  for(int i = 0; i < n; i++){
    Scalar color = getObjectColor(objNames[i]);
    colors.at<uchar>(i,0) = color(0);
    colors.at<uchar>(i,1) = color(1);
    colors.at<uchar>(i,2) = color(2);    
  }

  return colors;
}

/*****************************************************************************************/
void writeGTActions_foreground(Action action, const string& datasetName, const int trainVersion, vector<int> sequences, const bool doFG){

  Dataset db = dataset_setup(datasetName);

  string dataDir = db.getDataDir();

  char actName[500];
  actName[0] = 0;
  sprintf(actName, "%s", action.verb.c_str());
  for(int n = 0; n < action.names.size(); n++)
    sprintf(actName, "%s_%s", actName, action.names[n].c_str());
  
  char rootDir[500];
  rootDir[0] = 0;
  sprintf(rootDir, "%s/opencv/Results/%s", dataDir.c_str(), actName);
  
  char command[500];
  command[0] = 0;
  sprintf(command, "mkdir %s", rootDir);
  system(command);

  for(int s = 0; s < sequences.size(); s++){
    int seqNumber = sequences[s];
    string seqName = db.getSequence(seqNumber).getSeqName();
    vector<Action> actions = db.getActions(seqNumber);
    for(int a = 0; a < actions.size(); a++){
      if(!areEqualActions(action, actions[a]))
	continue;
      char fileName[500];
      fileName[0] = 0;
      if(doFG)
	sprintf(fileName, "%s/FG_%s_%s_%d_%d.avi", rootDir, actName, seqName.c_str(), actions[a].startFrame, actions[a].endFrame);
      else
	sprintf(fileName, "%s/%s_%s_%d_%d.avi", rootDir, actName, seqName.c_str(), actions[a].startFrame, actions[a].endFrame);
      //cout << fileName << endl;
      Mat im = db.getIM(seqNumber, actions[a].startFrame);
      Size ss = im.size();
      VideoWriter vw;
      vw.open(fileName, CV_FOURCC('M', 'J', 'P', 'G'), db.getFps(), ss);
      for(int f = actions[a].startFrame; f <= actions[a].endFrame; f++){
	Mat im = db.getIM(seqNumber, f);
	if(doFG){
	  Mat mask = db.readForegroundMask(seqNumber, f, trainVersion);
	  im = maskOnImage2(mask, im);
	}
	vw << im;
      }
    }
  }


  return;
}

/*****************************************************************************************/
/*****************************************************************************************/
/*****************************************************************************************/
/*****************************************************************************************/
/*****************************************************************************************/
