#include "FeatureStuff.h"
#include "ImageStuff.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "InputOutputFile.h"

using namespace std;
using namespace cv;

int main(int argc, char **argv) {

  if (argc != 4) {
    cout << "usage: " << argv[0] << " <input image> <output image> <resizeFactor>" << endl;
    return 1;
  }

  string imname(argv[1]);
  float resizeFactor = atof(argv[3]);
  Mat im = imread(imname);
  int height = (int)((float)(im.rows) / resizeFactor);
  int width = (int)((float)(im.cols) / resizeFactor);
  Mat inputImage(height, width, CV_8UC3);
  resize(im, inputImage, Size(width,height));
  int binSize = 16;

  //imshow("inputImage", inputImage);

  Mat colorHist = getColorHistogram(inputImage, binSize);
  
  Mat colorImage = myImagesc(colorHist);
  //cout << colorImage.rows << " " << colorImage.cols << endl;
  //imshow("Imagesc", colorImage);  

  float sigma = 0.4;
  float k = 100;
  int min_size = 100;
  int num_ccs;
  Mat segmentation = felzenszwalb_segment(inputImage, sigma, k, min_size, &num_ccs);
  int m = sortSegmentNumbers(segmentation);

  //showSegmentationRandom(segmentation, "Felzenszwalb");  

  vector< vector<float> > featureVectors = getFeatureVectors_colorHist(inputImage, binSize, segmentation);
  /*vector<float> temp = featureVectors[0];
  for(int i = 0; i < temp.size(); i++)
    cout << temp[i] << " ";
    cout << endl;*/

  SparseMat adjacencyMatrix = getWeightedAdjacencyMatrix(segmentation);

  SparseMat spatialMatrix = getSpatial_colorHist(featureVectors, adjacencyMatrix, 4, 0.5);

  Mat W(m, m, CV_32F);
  spatialMatrix.copyTo(W);
  int numOfClasses = 2;
  Mat Labels(m, 1, CV_32S);
  for(int i = 0; i < m; i++)
      Labels.at<int>(i,0) = 0;
  Mat fLabels(m, numOfClasses, CV_32F);
  for(int i = 0; i < m; i++)
    for(int j = 0; j < numOfClasses; j++)
      fLabels.at<float>(i,j) = 0;

  Mat contourImage_ = getSpatialOnContours(inputImage, segmentation, spatialMatrix);
  Mat contourImage(height, width, CV_8UC3);
  cvtColor(contourImage_, contourImage, CV_GRAY2BGR);
  contourImage += inputImage;

  //showSpatialOnContours(inputImage, segmentation, spatialMatrix);

  ImIntSeg * myImIntSeg = new ImIntSeg();
  myImIntSeg->image = inputImage;
  myImIntSeg->W = W;
  myImIntSeg->Labels = Labels;
  myImIntSeg->fLabels = fLabels;
  myImIntSeg->numOfClasses = numOfClasses;
  myImIntSeg->segmentation = segmentation;
  myImIntSeg->contourImage = contourImage;
  myImIntSeg->labelsOnContourImage = contourImage;

  imshow("FathiBMVC11", contourImage);

  cvSetMouseCallback("FathiBMVC11", interactiveImageSegmentation, (void *) myImIntSeg);

  cvWaitKey(0);

  Mat mask = labelsToMask(myImIntSeg->segmentation, myImIntSeg->fLabels);
  writeMaskAsImage(argv[2], mask);

  return 0;

}
