#ifndef FEATURE_STUFF_H
#define FEATURE_STUFF_H

#include <opencv2/opencv.hpp>

#include <vector>

#include "global.h"
#include "ImageStuff.h"

/* binSize should be between 1 to 256, I suggest 16 or 32 */
cv::Mat getColorHistogram(const cv::Mat & colorImage, const int binSize);

/* returns a feature vector for each region
   the feature vector of each region is normalized and sums up to 1
   It is assumed that region numbers are from 1 to m in segmentation matrix
 */
std::vector< std::vector<float> > getFeatureVectors_colorHist(const cv::Mat& inputImage, const int binSize, const cv::Mat& segmentation);

cv::SparseMat getSpatial_colorHist(const std::vector< std::vector<float> > & featureVectors_colorHist, const cv::SparseMat & adjacencyMatrix, float var1, float var2);

/*
  spatialWeighted is the result of element-wise multiplication of a spatialMatrix (that contains weights between adjacent super-pixels) and and the adjacencyWeighted matrix which contains the length of edges between adjacent super-pixels
 */
cv::SparseMat getSpatialWeighted(const cv::SparseMat & spatialMatrix, const cv::SparseMat & adjacencyWeighted);

cv::Mat getFeatureMap(const cv::Mat& fvs, const cv::Mat& centers);

cv::Mat getHOG(const cv::Mat& im);
cv::Mat featureMapHOG_reshape(const cv::Mat& featureMap, const int imrows, const int imcols);

cv::Mat getColorLab(const cv::Mat& im);
cv::Mat featureMapLab_reshape(const cv::Mat& featureMap, const int imrows, const int imcols);

cv::Mat featureMap2regionFeatures(const cv::Mat& featureMap, const cv::Mat& segmentation, const int numBins);

/*
  given two segmentations (segmentation1,m1,fv1,centers1 and segmentation2,m2,fv2,centers2), 
  returns m1 feature vectors each replaced by the element in fv2 at the region from segment1
 */
cv::Mat getHierarchicalFeature(const cv::Mat& fv2, const cv::Mat& centers1, const cv::Mat& segmentation2);

cv::Mat featureAroundPoint(const cv::Mat& featureMap, const float y, const float x, const int radius, const int numBins);

cv::Mat featureInMask(const cv::Mat& featureMap, const cv::Mat& mask, const int numBins);

#endif //FEATURE_STUFF_H


