#include "TrainStuff.h"
#include "ForegroundStuff.h"
#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "FeatureStuff.h"
#include "MyMATFileInterface.h"
#include <iostream>
#include <fstream>

using namespace std;
using namespace cv;


/******************************************************************************/
Mat trainForegroundLocationPattern(string datasetName, const int trainVersion, const float scale){
  cout << "Training Foreground Location Pattern" << endl;
  Dataset db = dataset_setup(datasetName);
  db.setScale(scale);

  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);
  int count = 0;
  Mat output;
  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    int step = 10;
    int seqLength = db.getSequence(seqNumber).getSeqLength();
    //string seqName = db.getSequence(seqNumber).getSeqName();
    //cout << seqName.c_str() << " " << seqNumber << " " << seqLength << endl;
    for(int f = 10; f <= seqLength; f+=step){
      Mat mask = db.getGTObjectMask("Foreground", seqNumber, f);
      if(mask.rows < 1)
	continue;
      cout << f << ", ";
      cout.flush();
      if(count == 0){
	output = Mat::zeros(mask.rows, mask.cols, CV_32F);
	for(int y = 0; y < mask.rows; y++)
	  for(int x = 0; x < mask.cols; x++)
	      output.at<float>(y,x) = 0;
      }

      for(int y = 0; y < mask.rows; y++)
	for(int x = 0; x < mask.cols; x++)
	  if(mask.at<uchar>(y,x) > 0)
	    output.at<float>(y,x) += 1;
      count++;
    }
  }
  cout << endl;
  output /= (float)(count);
  return output;
}


/******************************************************************************/
pair<Mat, Mat> trainingDataXiaofengClassifier(string datasetName, const int trainVersion, const float scale, const int segType){
  Dataset db = dataset_setup(datasetName);
  db.setScale(scale);

  Mat locationPattern = trainForegroundLocationPattern(datasetName, trainVersion, scale);

  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);

  Mat trainData;
  Mat labels;

  for(int i = 0; i < trainSeqs.size(); i++){
  //for(int i = 0; i < 1; i++){
    int seqNumber = trainSeqs[i];
    int step = 10;
    int seqLength = db.getSequence(seqNumber).getSeqLength();
    string dataDir = db.getDataDir();
    string imType = db.getImType();
    string seqName = db.getSequence(seqNumber).getSeqName();
    //cout << seqName.c_str() << " " << seqNumber << " " << seqLength << endl;
    
    for(int f = 10; f <= seqLength; f+=step){
    //for(int f = 10; f < 700; f+=step){
      Mat mask = db.getGTObjectMask("Foreground", seqNumber, f);
      if(mask.rows < 1)
	continue;
      //cout << gtPath << endl;
      //imshow("mask", mask);

      Sequence sq(db, seqNumber, f-10, f+5, segType);
      Prior p = sq.getPrior(f);
      Mat labels_ = p.maskToLabel(mask);
      Mat xfv = sq.xiaofengFeatureVector(f, locationPattern);
      //cout << labels_.rows << " " << labels_.cols << endl;
      //cout << xfv.rows << " " << xfv.cols << endl;
      trainData.push_back(xfv);
      labels.push_back(labels_);
      //cout << trainData.rows << " " << trainData.cols << endl;
      //cout << labels.rows << " " << labels.cols << endl;
    }
  }
  pair<Mat,Mat> output;
  output.first = trainData;
  output.second = labels;

  return output;
}

/******************************************************************************/
void trainXiaofengClassifier(string datasetName, const int trainVersion, CvSVM * trainer, const float scale, const int segType){
  Dataset db = dataset_setup(datasetName);
  db.setScale(scale);

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/trainXiaofengClassifier_%d_%g_%d.bin", dataDir.c_str(), trainVersion, scale, segType);
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer->load(buffer);
    //cout << "hello " << trainer->get_support_vector_count() << endl;
    return;
  }
  ifile.close();

  //cout << "hello" << endl;
  pair<Mat, Mat> trainingData = trainingDataXiaofengClassifier(datasetName, trainVersion, scale, segType);
  Mat trainData = trainingData.first;
  Mat labels = trainingData.second;
  //cout << trainData.rows << " " << trainData.cols << endl;

  CvSVMParams params;
  params.svm_type = CvSVM::C_SVC;
  params.kernel_type = CvSVM::LINEAR;
  params.term_crit   = cvTermCriteria(CV_TERMCRIT_ITER, 1000, 1e-6);
  trainer->train(trainData, labels, Mat(), Mat(), params);
  
  trainer->save(buffer);

  return;
}

/******************************************************************************/
void trainXiaofengClassifier_svmlight(std::string datasetName, const int trainVersion, LinearClassifier& trainer, const float scale, const int segType){
  Dataset db = dataset_setup(datasetName);
  db.setScale(scale);

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/trainXiaofengClassifier_svmlight_%d_%g_%d.bin", dataDir.c_str(), trainVersion, scale, segType);
  //cout << buffer << endl;
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer.load(buffer);
    trainer.w = -trainer.w;
    trainer.b = -trainer.b;
    return;
  }
  ifile.close();

  char * params = "-t 0";

  char tempFile[500];
  tempFile[0] = 0;
  sprintf(tempFile, "%s/opencv/trainXiaofengClassifier_svmlight_tempFile_%d_%g_%d.bin", dataDir.c_str(), trainVersion, scale, segType);

  cout << tempFile << endl;

  char modelFile[500];
  modelFile[0] = 0;
  sprintf(modelFile, "%s/opencv/trainXiaofengClassifier_svmlight_modelFile_%d_%g_%d.bin", dataDir.c_str(), trainVersion, scale, segType);

  cout << modelFile << endl;

  pair<Mat, Mat> trainingData = trainingDataXiaofengClassifier(datasetName, trainVersion, scale, segType);
  Mat trainData = trainingData.first;
  Mat labels = trainingData.second;
  
  trainer = svm_light_train(trainData, labels, tempFile, modelFile, params);

  trainer.save(buffer);

  char command[500];
  command[0] = 0;
  sprintf(command, "rm -f %s", tempFile);
  system(command);

  command[0] = 0;
  sprintf(command, "rm -f %s", modelFile);
  system(command);

  trainer.w = -trainer.w;
  trainer.b = -trainer.b;

  return;
}

/******************************************************************************/
Mat fg_getUnary(Sequence& sq, CvSVM* trainer, const int fromFrame, const int toFrame, const Mat& locationPattern){
  vector<int> ms;
  for(int f=fromFrame; f <= toFrame; f++)
    ms.push_back(sq.getPrior(f).getM());

  int m = 0;
  for(int i = 0; i < ms.size(); i++)
    m += ms[i];

  Mat prior(m, 2, CV_32F);

  float const1 = -1;
  float const2 = 0;

  int m_start = 0;
  for(int i = 0; i < ms.size(); i++){
    Mat fv = sq.xiaofengFeatureVector(fromFrame+i, locationPattern);
    int m = ms[i];
    Mat temp;
    for(int j = 0; j < m; j++){
      temp = fv.rowRange(j,j+1);
      prior.at<float>(m_start+j,0) = 1 / (1+exp(const1*(-trainer->predict(temp, true)+const2)));
      prior.at<float>(m_start+j,1) = 1 - prior.at<float>(m_start+j,0);
    }
    m_start += m;
  }

  return prior;
}

/******************************************************************************/
Mat fg_getUnary_svmlight(Sequence& sq, LinearClassifier trainer, const int fromFrame, const int toFrame, const Mat& locationPattern){
  
  cout << locationPattern.rows << " " << locationPattern.cols << endl;
  
  vector<int> ms;
  for(int f=fromFrame; f <= toFrame; f++)
    ms.push_back(sq.getPrior(f).getM());

  int m = 0;
  for(int i = 0; i < ms.size(); i++){
    m += ms[i];
    cout << ms[i] << endl;
  }

  Mat prior(m, 2, CV_32F);

  float const1 = -1;
  float const2 = 0;

  int m_start = 0;
  for(int i = 0; i < ms.size(); i++){
    //cout << "fuck" << endl;
    Mat fv = sq.xiaofengFeatureVector(fromFrame+i, locationPattern);
    //cout << fv.rows << " " << fv.cols << endl;
    int m = ms[i];
    Mat temp = fv * trainer.w.t() + trainer.b;
    for(int j = 0; j < m; j++){
      prior.at<float>(m_start+j,0) = 1 / (1+exp(const1*(-temp.at<float>(j,0)+const2)));
      prior.at<float>(m_start+j,1) = 1 - prior.at<float>(m_start+j,0);
    }
    m_start += m;
  }

  return prior;
}

/******************************************************************************/
vector<Mat> fgSegmentation(Sequence& sq, CvSVM * trainer, const int fromFrame, const int toFrame, const Mat& locationPattern){

  // Pairwise terms
  float ratio = 1;
  SparseMat pairwise = sq.computeW(fromFrame, toFrame, ratio);

  // UNARY terms
  Mat prior = fg_getUnary(sq, trainer, fromFrame, toFrame, locationPattern);

  //cout << prior << endl;

  vector<int> ms;
  for(int f=fromFrame; f <= toFrame; f++)
    ms.push_back(sq.getPrior(f).getM());
  
  // GraphCut
  cout << "GraphCut...";
  cout.flush();
  Mat labels = graphCut(prior*1000, pairwise);
  cout << "Done." << endl;
  //cout << labels << endl;
  vector<Mat> output;
  int m_start = 0;
  for(int i = 0; i < ms.size(); i++){
    int m = ms[i];
    Mat temp = labels.rowRange(m_start, m_start+m);
    output.push_back(temp);
    m_start += m;
  }
  
  return output;
}

/******************************************************************************/
vector<Mat> fgSegmentation_svmlight(Sequence& sq, LinearClassifier& trainer, const int fromFrame, const int toFrame, const Mat& locationPattern){
  // Pairwise terms
  float ratio = 1;
  SparseMat pairwise = sq.computeW(fromFrame, toFrame, ratio);

  // UNARY terms
  Mat prior = fg_getUnary_svmlight(sq, trainer, fromFrame, toFrame, locationPattern);

  //cout << prior << endl;

  vector<int> ms;
  for(int f=fromFrame; f <= toFrame; f++)
    ms.push_back(sq.getPrior(f).getM());
  
  // GraphCut
  cout << "GraphCut...";
  cout.flush();
  Mat labels = graphCut(prior*1000, pairwise);
  cout << "Done." << endl;
  //cout << labels << endl;
  vector<Mat> output;
  int m_start = 0;
  for(int i = 0; i < ms.size(); i++){
    int m = ms[i];
    Mat temp = labels.rowRange(m_start, m_start+m);
    output.push_back(temp);
    m_start += m;
  }
  
  return output;
  
}

/******************************************************************************/
void batch_fgSegmentation(std::string datasetName, const int seqNumber, const int startFrame, const int endFrame, const int trainVersion, const float scale, const int segType){
  Dataset db = dataset_setup(datasetName);
  db.setScale(scale);
  DatasetSeq seq = db.getSequence(seqNumber);

  int stepGC = 50;
  int lengthGC = 100;
  int seqContain = 120;

  Mat locationPattern = trainForegroundLocationPattern(datasetName, trainVersion, scale);
  LinearClassifier trainer;
  trainXiaofengClassifier_svmlight(datasetName, trainVersion, trainer, scale, segType);

  // initializing the sequence
  string dataDir = db.getDataDir();
  string seqName = seq.getSeqName();
  string imType = db.getImType();
  int endFrame_ = startFrame+seqContain+1;
  int currentFrame = startFrame;
  Sequence sq(db, seqNumber, startFrame-10, endFrame_, BERKELEY_SEG);

  char rootFolder[300];
  rootFolder[0] = 0;
  sprintf(rootFolder, "%s/opencv/%s", dataDir.c_str(), seqName.c_str());
  char buffer[300];
  buffer[0] = 0;
  sprintf(buffer, "mkdir %s", rootFolder);
  system(buffer);

  rootFolder[0] = 0;
  sprintf(rootFolder, "%s/opencv/%s/FGSegmentation_%d_%g_%d", dataDir.c_str(), seqName.c_str(), trainVersion, scale, segType);
  buffer[0] = 0;
  sprintf(buffer, "mkdir %s", rootFolder);
  system(buffer);

  // Looping, adding more frames, graphcut, etc.
  do{
    cout << seqNumber << " " << currentFrame << endl;
    if((endFrame-currentFrame) % stepGC == 0 || endFrame_ >= endFrame || currentFrame == startFrame){
      vector<Mat> labels = fgSegmentation_svmlight(sq, trainer, currentFrame, currentFrame+lengthGC, locationPattern);
      for(int f = currentFrame; f <= currentFrame+lengthGC; f++){
	cout << f << " ";
	char buffer[200];
	buffer[0] = 0;
	sprintf(buffer, "%s/%010d.pgm", rootFolder, f);
	//cout << buffer << endl;
	Mat mask = sq.getPrior(f).labelsToMask(labels[f-currentFrame]);
	writeMaskAsImage(buffer, mask);
      }
      cout << endl;
    }
    sq.push_back(db, seqNumber, segType);
    sq.pop_front();

    currentFrame++;
    endFrame_++;
  }
  while(endFrame_ <= endFrame);


  return;
}


/******************************************************************************/
Mat futureManipulationFeatureVector(Sequence& sq, Dataset& db, const int seqNumber, const int frame, const int trainVersion){
  Prior p = sq.getPrior(frame);
  Mat output;

  vector<int> margins;
  margins.push_back(0);
  margins.push_back(2);
  margins.push_back(5);
  margins.push_back(10);
  
  for(int i = 0; i < margins.size(); i++){
    int f = frame + margins[i];

    Mat fgMask = db.readForegroundMask(seqNumber, f, trainVersion);
    cout << fgMask.rows << " " << fgMask.cols << endl;
    Mat fgPrior = sq.getPrior(f).maskToLabel(fgMask);
    Mat tffgPrior = sq.transferPriorFromFrameToFrame(fgPrior, f, frame);

    /*char buffer[100];
    buffer[0] = 0;
    sprintf(buffer, "fgPrior %d", margins[i]);
    Mat seg = sq.getPrior(frame).getSegmentation();
    showPrior(tffgPrior, seg, buffer);*/

    Mat temp = tffgPrior.t();
    output.push_back(temp);
  }
  output = output.t();

  return output;
}

/******************************************************************************/
Mat cleanForegroundMask(const Mat& image, const Mat& mask){
  imshow("mask", mask);
  int h = mask.rows;
  int w = mask.cols;
  Mat seg(h, w, CV_32S);
  for(int i = 0; i < h; i++)
    for(int j = 0; j < w; j++)
      seg.at<int>(i,j) = (int)(mask.at<uchar>(i,j));

  sortSegmentNumbers(seg);
  Mat ucm = getContours_ucm(seg);

  Prior p(image, ucm, 1, 1);
  Mat labels = p.getSegmentation();
  Mat sizes = p.getPrior_size();

  Mat outputMask(h, w, CV_8U);
  int sizeThresh = 2000;
  int s;
  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      if(mask.at<uchar>(i,j) == 0){
	outputMask.at<uchar>(i,j) = 0;
	continue;
      }
      s = sizes.at<float>(labels.at<int>(i,j)-1,0);
      if(s < sizeThresh)
	outputMask.at<uchar>(i,j) = 0;
      else
	outputMask.at<uchar>(i,j) = 255;
    }
  }
  
  return outputMask;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
