#include "GazeStuff.h"
#include <iostream>
#include <fstream>

#include <opencv2/gpu/gpu.hpp>

using namespace std;
using namespace cv;
using namespace cv::gpu;

/****************************************************************************/
Mat readGazePoints_SMI(const char * filename, const float scale, const int marginY, const int marginX){
  Mat output;
  ifstream infile(filename, ios::in);
  if(!infile){
    cout << "readGazePoints_SMI: problem reading from file..." << endl;
    exit(1);
  }
  
  int limit = 500;
  char line[limit];
  for(int i = 0; i < 34; i++)
    infile.getline(line, limit);
  
  infile.getline(line, limit);

  float timestamp, y, x;
  Mat temp(1, 3, CV_32F);
  while(infile){
    infile.getline(line, limit);
    if(!infile)
      break;
    char * token = strtok(line, "\t");
    timestamp = (float)(atoi(token));
    token = strtok(NULL, "\t");
    token = strtok(NULL, "\t");
    token = strtok(NULL, "\t");
    x = atof(token);
    token = strtok(NULL, "\t");
    y = atof(token);
    temp.at<float>(0,0) = timestamp;
    temp.at<float>(0,1) = (y-marginY) * scale;
    temp.at<float>(0,2) = (x-marginX) * scale;
    output.push_back(temp);
  }

  return output;
}

/****************************************************************************/
vector<pair<float,float> > rawGazePointsToFrameLocations_SMI(const Mat& rawData, const int numFrames, const int fps){
  
  Mat queryMat(numFrames, 1, CV_32F);
  float step = (1000000/fps);
  queryMat.at<float>(0,0) = 0;
  for(int f = 0; f < numFrames; f++)
    queryMat.at<float>(f,0) = queryMat.at<float>(f-1,0) + step;
  
  Mat dataMat = rawData.colRange(0,1);
  float dcTerm = dataMat.at<float>(0,0);
  dataMat -= dcTerm;
  
  GpuMat queryMat_gpu;
  GpuMat dataMat_gpu;
  queryMat_gpu.upload(queryMat);
  dataMat_gpu.upload(dataMat);

  vector<DMatch> matches;
  BruteForceMatcher_GPU<L2<float> > matcher;
  matcher.match(queryMat_gpu, dataMat_gpu, matches);
  
  if(matches.size() != queryMat.rows){
    cout << "rawGazePointsToFrameLocations_SMI: Not all gaze points are matched!" << endl;
    exit(1);
  }

  vector<pair<float,float> > output;
  int idx;
  pair<float,float> tempLocation;
  for(int i = 0; i < queryMat.rows; i++){
    idx = matches[i].trainIdx;
    tempLocation.first = rawData.at<float>(idx, 1);
    tempLocation.second = rawData.at<float>(idx, 2);
    output.push_back(tempLocation);
  }
  
  return output;
}
/****************************************************************************/
/****************************************************************************/
/****************************************************************************/
/****************************************************************************/
