#include "InputOutputFile.h"

#include <fstream>
#include <vector>

using namespace std;
using namespace cv;

/*************************************************************************************/
bool fileExists(const char * filename){
  bool exists = false;
  ifstream ifile(filename);
  exists = ifile;
  ifile.close();
  return exists;
}

/*************************************************************************************/
void writeMatrixToFile(const char * filename, const Mat & matrix){
  ofstream outfile(filename, ios::out);
  Size s = matrix.size();
  int height = s.height;
  int width = s.width;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      outfile << matrix.at<float>(i,j);
      if(j < width-1)
	outfile << " ";
    }
    outfile << endl;
  }
  outfile.close();
}

/*************************************************************************************/
Mat readMatrixFromFile(const char * filename){
  vector< vector<float> > tempMat;
  ifstream infile(filename, ios::in);
  while(infile){
    int limit = 20000;
    char line[limit];
    infile.getline(line, limit);
    if(!infile)
      break;
    //cout << line << endl;
    char * tok;
    vector<float> lineVec;
    tok = strtok(line, " ");
    while(tok != NULL){
      float temp;
      sscanf(tok, "%f", &temp);
      lineVec.push_back((float)(temp));
      tok = strtok(NULL, " ");
    }
    tempMat.push_back(lineVec);
  }
  int height = (int)(tempMat.size());
  if(height == 0){
    Mat answer(0,0,CV_32F);
    return answer;
  }
  int width = (int)(tempMat[0].size());
  Mat answer(height, width, CV_32F);
  for(int i = 0; i < (int)(tempMat.size()); i++){
    vector<float> lineVec = tempMat[i];
    for(int j = 0; j < (int)(lineVec.size()); j++)
      answer.at<float>(i,j) = lineVec[j];
  }
  return answer;
}

/*************************************************************************************/
char * getBSDS500TrainImageName(int index){
  ifstream infile("BSDS500_train_image_list.txt", ios::in);
  int limit = 200;
  char * line = new char[limit];
  line[0] = 0;
  for(int i = 1; i <= index; i++){
    infile.getline(line, limit);
    //cout << line << endl;
    if(!infile){
      cout << "getBSRTrainImageName: Index out of bound, returning the last image name" << endl;
      break;
    }
    if(i == index)
      break;
  }
  line = strtok(line, ".");
  return line;
}

/*************************************************************************************/
void writeMaskAsImage(const char * filename, const cv::Mat & mask){
  int height = mask.rows;
  int width = mask.cols;
  Mat output(height, width, CV_8UC1);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      output.at<uchar>(i,j) = (uchar)(mask.at<int>(i,j))*255;
  vector<int> params;
  params.push_back(CV_IMWRITE_PXM_BINARY);
  params.push_back(0);
  imwrite(filename, output);
  
  return;
}

/*************************************************************************************/
Mat readMaskFromImage(const char * filename){
  Mat im = imread(filename, 0);
  return im;
}

/*************************************************************************************/
string getImagePath(const char * dataDir, const char * seqName, const char * imType, const int frameNumber){
  char buffer[200];
  buffer[0] = 0;
  sprintf(buffer, "%s/%s/%s/%010d.%s", dataDir, imType, seqName, frameNumber, imType);
  string output(buffer);
  return output;
}

/*************************************************************************************/
string getContourPath(const char * dataDir, const char * seqName, const int frameNumber){
  char buffer[200];
  buffer[0] = 0;
  sprintf(buffer, "%s/segment_half/%s/%010d_contour.mat", dataDir, seqName, frameNumber);
  string output(buffer);
  return output;
}


/*************************************************************************************/
void write_svm_light_input(const Mat& data, const Mat& labels, const char * filename){
  int m = data.rows;
  int n = data.cols;

  ofstream outfile(filename, ios::out);

  for(int i = 0; i < m; i++){
    outfile << (int)(labels.at<float>(i,0)) << " ";
    for(int j = 0; j < n; j++){
      if(data.at<float>(i,j) != 0)
	outfile << j+1 << ":" << data.at<float>(i,j) << " ";
    }
    outfile << endl;
  }

  outfile.close();

  return;
}

/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
