#include "MLStuff.h"

using namespace std;
using namespace cv;

/*************************************************************************************/
Mat harmonicSemiSupervised(const Mat & W, const Mat & Labels, const int numClasses){
  int m = W.rows;

  // cout << "m is: " << m << endl;
  // cout << "Labels: " << endl << Labels << endl;
  
  if(Labels.rows != m){
    cout << "harmonicSemiSupervised: The dimension of Labels do not match the dimension of W" << endl;
    exit(1);
  }

  int numL = 0; // number of labeled nodes
  int numU = 0; // number of unlabeled nodes

  Mat prU(m,1,CV_32S);
  Mat prL(m,1,CV_32S);
  for(int i = 0; i < m; i++){
    if(Labels.at<int>(i,0) == 0){
      prU.at<int>(i,0) = 1;
      prL.at<int>(i,0) = 0;
      numU++;
    }
    else{
      prU.at<int>(i,0) = 0;
      prL.at<int>(i,0) = 1;
      numL++;
    }
  }

  //cout << "numL: " << numL << ", numU: " << numU << endl;
  //cout << prL << endl;

  Mat output(m, numClasses, CV_32F);

  if(numL == 0){
    for(int i = 0; i < m; i++)
      for(int j = 0; j < numClasses; j++)
	output.at<float>(i,j) = 0;
    return output;
  }

  Mat fl(numL, numClasses, CV_32F);
  for(int i = 0; i < numL; i++)
    for(int j = 0; j < numClasses; j++)
      fl.at<float>(i,j) = 0;

  int countL = 0;
  for(int i = 0; i < m; i++){
    if(Labels.at<int>(i,0) != 0){
      fl.at<float>(countL, Labels.at<int>(i,0)-1) = 1;
      countL++;
    }
  }

  // cout << "fl: " << endl << fl << endl;

  Mat Wul = subMatrix<float>(W, prU, prL);

  Mat L = getLaplacianMatrix(W);

  Mat Luu = subMatrix<float>(L, prU, prU);

  Mat part1(numU, numL, CV_32F);

  bool didSolve = solve(Luu, Wul, part1);

  if(!didSolve){
    cout << "harmonicSemiSupervised: Could not solve the linear equation" << endl;
  }

  Mat fu = part1 * fl;

  countL = -1;
  int countU = -1;
  for(int i = 0; i < m; i++){
      if(Labels.at<int>(i,0) == 0){
	countU++;
	for(int j = 0; j < numClasses; j++)
	  output.at<float>(i,j) = fu.at<float>(countU,j);
      }
      else{
	countL++;
	for(int j = 0; j < numClasses; j++)
	  output.at<float>(i,j) = fl.at<float>(countL,j);
      }
  }

  return output;
}

/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
/*************************************************************************************/
