/****

     This function Makes the patches!

 ***/

#include <iostream>

#include "MakePatches.h"

using namespace cv;
using namespace std;

/*******************************************************************************************************************/

void makePatches_RGB(uchar ** answer, const cv::Mat& image, const int patchRadius){
  Size wh = image.size();
  int height = wh.height;
  int width = wh.width;
  int channels = image.channels();
  if(channels != 3){
    cout << "The image should have RGB channels in order to be able to transform it into CIE XYZ space" << endl;
    exit(0);
  }

  int h = height - patchRadius*2;
  int w = width - patchRadius*2;
  int patchSize = patchRadius * 2 + 1;

  typedef Vec<uchar,3> Vec3u; 

  int count = 0;
  for(int i = patchRadius; i < height-patchRadius; i++){
    for(int j = patchRadius; j < width-patchRadius; j++){
      int count2 = 0;
      for(int ii = i-patchRadius; ii <= i+patchRadius; ii++){
	for(int jj = j-patchRadius; jj <= j+patchRadius; jj++){
	  answer[count][count2++] = image.at<Vec3u>(ii,jj)[0];
	  answer[count][count2++] = image.at<Vec3u>(ii,jj)[1];
	  answer[count][count2++] = image.at<Vec3u>(ii,jj)[2];
	}
      }
      count++;
    }
  }
  

  return;
}

/*******************************************************************************************************************/

void makePatches_HOG(uchar ** answer, const cv::Mat& image, const int patchRadius){
}

/*******************************************************************************************************************/

void makePatches_SIFT(uchar ** answer, const cv::Mat& image, const int patchRadius){
}

/*******************************************************************************************************************/

void makePatches_XYZ(uchar ** answer, const cv::Mat& image, const int patchRadius){
  // channels should be equal to 3
  
  Size wh = image.size();
  int height = wh.height;
  int width = wh.width;
  int channels = image.channels();
  if(channels != 3){
    cout << "The image should have RGB channels in order to be able to transform it into CIE XYZ space" << endl;
    exit(0);
  }

  Mat luv(height, width, CV_8UC3);
  cvtColor(image, luv, CV_BGR2Luv, 3);

  typedef Vec<uchar,3> Vec3u; 

  int count = 0;
  for(int i = patchRadius; i < height-patchRadius; i++){
    for(int j = patchRadius; j < width-patchRadius; j++){
      int count2 = 0;
      for(int ii = i-patchRadius; ii <= i+patchRadius; ii++){
	for(int jj = j-patchRadius; jj <= j+patchRadius; jj++){
	  answer[count][count2++] = luv.at<Vec3u>(ii,jj)[0];
	  answer[count][count2++] = luv.at<Vec3u>(ii,jj)[1];
	  answer[count][count2++] = luv.at<Vec3u>(ii,jj)[2];
	}
      }
      count++;
    }
  }
  
  return;
}

/*******************************************************************************************************************/

void makePatches_RGB_WEIGHTED(uchar ** answer, const cv::Mat& image, const int patchRadius){
}

/*******************************************************************************************************************/

void makePatches_GRADIENT(uchar ** answer, const cv::Mat& image, const int patchRadius){
  Size wh = image.size();
  int height = wh.height;
  int width = wh.width;
  int channels = image.channels();
  int h = height - patchRadius*2;
  int w = width - patchRadius*2;
  int patchSize = patchRadius * 2 + 1;

  Mat grayImage(height, width, CV_8UC1);
  cvtColor(image, grayImage, CV_BGR2GRAY, 1);

  Mat gradientMatH(height, width, CV_8UC1);
  Mat gradientMatW(height, width, CV_8UC1);
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      if(i == 0)
	gradientMatH.at<uchar>(i,j) = 128;
      else{
	int dif = (int)(grayImage.at<uchar>(i,j)) - (int)(grayImage.at<uchar>(i-1,j));
	dif += 128;
	if(dif > 255)
	  dif = 255;
	if(dif < 0)
	  dif = 0;
	gradientMatH.at<uchar>(i,j) = (uchar)(dif);
      }
      
      if(j == 0)
	gradientMatW.at<uchar>(i,j) = 128;
      else{
	int dif = (int)(grayImage.at<uchar>(i,j)) - (int)(grayImage.at<uchar>(i,j-1));
	dif += 128;
	if(dif > 255)
	  dif = 255;
	if(dif < 0)
	  dif = 0;
	gradientMatW.at<uchar>(i,j) = (uchar)(dif);
      }
    }
  }

  int count = 0;
  for(int i = patchRadius; i < height-patchRadius; i++){
    for(int j = patchRadius; j < width-patchRadius; j++){
      int count2 = 0;
      for(int ii = i-patchRadius; ii <= i+patchRadius; ii++){
	for(int jj = j-patchRadius; jj <= j+patchRadius; jj++){
	  answer[count][count2++] = gradientMatH.at<uchar>(ii,jj);
	  answer[count][count2++] = gradientMatW.at<uchar>(ii,jj);	  
	}
      }
      count++;
    }
  }

  return;
}

/*******************************************************************************************************************/
/* if patchRadius is entered 0, it will return the index for image */
void indexToYXLocation(int& Y, int& X, const long int index, const int height, const int width, const int patchRadius){
  int ww = width - patchRadius*2;
  X = index % ww;
  Y = index / ww;
  X = X + patchRadius;
  Y = Y + patchRadius;
  return;
}
/*******************************************************************************************************************/
/* if patchRadius is entered 0, it will return the index for image */
void YXLocationToIndex(long int& index, const int Y, const int X, const int height, const int width, const int patchRadius)
{
  int ww = width - patchRadius*2;
  index = (Y-patchRadius)*ww + (X-patchRadius);
  return;
}

/*******************************************************************************************************************/



