#ifndef MAKE_PATCHES_H
#define MAKE_PATCHES_H

#include <cstdlib>
#include <cstdio>

#include "global.h"

/* 
   Represents each patch (height x width) with its raw R, G and B values.
*/
void makePatches_RGB(uchar ** answer, const cv::Mat& image, const int patchRadius);

/*
  Represents each patch with its HOG descriptor.
 */
void makePatches_HOG(uchar ** answer, const cv::Mat& image, const int patchRadius);

/*
  Represents each patch with its SIFT descriptor.
 */
void makePatches_SIFT(uchar ** answer, const cv::Mat& image, const int patchRadius);

/*
  Represents each patch with its XYZ color-space values.
 */
void makePatches_XYZ(uchar ** answer, const cv::Mat& image, const int patchRadius);

/*
  Represents each patch with its weighted RGB values. Wights are the same as ones used to map the image into gray scale.
 */
void makePatches_RGB_WEIGHTED(uchar ** answer, const cv::Mat& image, const int patchRadius);

/*
  Represents each patch its gradient values.
 */
void makePatches_GRADIENT(uchar ** answer, const cv::Mat& image, const int patchRadius);

void indexToYXLocation(int& Y, int& X, const long int index, const int height, const int width, const int patchRadius);

void YXLocationToIndex(long int& index, const int Y, const int X, const int height, const int width, const int patchRadius);

#endif

