#ifndef MOTION_STUFF_H
#define MOTION_STUFF_H

#include <opencv2/opencv.hpp>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

#include <vector>

#include "global.h"
#include "ImageStuff.h"
#include "SparseStuff.h"

#include "flow_functions.h"

/****************************************************************************/
/*
  Computes the dense optical flow using the method of Brox et al., eccv04.
  It runs on GPU.
  It resizes the image using resizeFactor, but then resizes the answer back to the original size of the image.
 */
std::pair<cv::Mat,cv::Mat> getDenseFlow_eccv04(const cv::Mat& imag1, const cv::Mat& image2, double resizeFactor); 

/****************************************************************************/
cv::Mat affineEstimate(const cv::Mat & P, const cv::Mat & V);

/****************************************************************************/
cv::Mat affineMotionRansac(const cv::Mat & P, const cv::Mat & V, double thresh, int numIter, std::vector<int>& returnInliers);

/****************************************************************************/
/*
  This doesn't work yet!!! I gave up on working more on that for now!
  Returns num motion channels given the dense optical flow vectors
  It subsamples the flow vectors every st pixels
  A good value for num is 5 usually
 */
void getMotionChannels_spectralClustering(const cv::Mat& VX, const cv::Mat& VY, const int st, const int num, cv::Mat& returnLABELS, cv::SparseMat& returnAdj);

/****************************************************************************/
/*
  Simply clusters optical flow vectors
 */
cv::Mat getMotionChannels_clustering(const cv::Mat& VX, const cv::Mat& VY, const int st, const int num);

/****************************************************************************/
/*
  Computes the dense optical flow between im1 and im2
  returns the mask of pixels for which it could fit an affine transformation
  st is the step size for subsampling the images
 */
cv::Mat xiaofeng_getInliersOpticalFlowAffine(const cv::Mat& im1, const cv::Mat& im2, const int st);
cv::Mat xiaofeng_getInliersOpticalFlowAffine(const std::pair<cv::Mat,cv::Mat>& flow, const int st);

/****************************************************************************/
/*
  images is a vector of images including the current frame and its adjacent frames
  refIndex is the index of the current frame in the vector
  It returns a probabilistic map of background regions, based on affine motion to those frames
 */
cv::Mat getBackgroundMaskPb(const std::vector<cv::Mat>& images, const int refIndex);
cv::Mat getBackgroundMaskPb(const std::vector<std::pair<cv::Mat,cv::Mat> >& flows);

/****************************************************************************/
cv::SparseMat getTransitionFlow_unnormalizedM(const cv::Mat& VX, const cv::Mat& VY, const cv::Mat& segmentation1, const cv::Mat& segmentation2);

#endif //MOTION_STUFF_H


