
#include "MyMATFileInterface.h"

using namespace std;
using namespace cv;

/*************************************************************************************************/
Mat readBSDSegmentation(char * file){
  MATFile * pmat;
  const char ** dir;
  int  ndir;
  mxArray *pa;
  const char *name;
  
  //printf("Reading file %s...\n\n", file);

  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error opening file %s\n", file);
    Mat answer;
    return answer;
  }

  dir = (const char **)matGetDir(pmat, &ndir);
  if (dir == NULL) {
    printf("Error reading directory of file %s\n", file);
    matClose(pmat);
    Mat answer;
    return answer;;
  } else {
    //printf("Directory of %s:\n", file);
    //for (int i=0; i < ndir; i++)
    //printf("%s\n",dir[i]);
  }
  mxFree(dir);
  
  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    Mat answer;
    return answer;
  }
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error reopening file %s\n", file);
    Mat answer;
    return answer;
  }

  /* Get headers of all variables */
  //printf("\nExamining the header for each variable:\n");
  for (int i=0; i < ndir; i++) {
    pa = matGetNextVariableInfo(pmat, &name);
    if(strcmp(name, "Segmentation") == 0 || strcmp(name, "labels") == 0){
      if (pa == NULL) {
	printf("Error reading in file %s\n", file);
	Mat answer;
	mxDestroyArray(pa);
	matClose(pmat);
	return answer;
      }
      /* Diagnose header pa */
      //printf("According to its header, array %s has %d dimensions\n", name, (int)(mxGetNumberOfDimensions(pa)));
      /*if (mxIsFromGlobalWS(pa))
	printf("  and was a global variable when saved\n");
      else
	printf("  and was a local variable when saved\n");
      */
    }
    mxDestroyArray(pa);
  }

  /* Reopen file to read in actual arrays. */
  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    mxDestroyArray(pa);
    matClose(pmat);
    Mat answer;
    return answer;
  }
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error reopening file %s\n", file);
    mxDestroyArray(pa);
    matClose(pmat);
    Mat answer;
    return answer;
  }

  /* Read in each array. */
  //printf("\nReading in the actual array contents:\n");
  for (int i=0; i<ndir; i++) {
    pa = matGetNextVariable(pmat, &name);
    if(strcmp(name, "Segmentation") == 0 || strcmp(name, "labels") == 0){
      if (pa == NULL) {
	//printf("Error reading in file %s\n", file);
	mxDestroyArray(pa);
	matClose(pmat);
	Mat answer;
	return answer;
      } 
      /*
       * Diagnose array pa
       */
      //printf("According to its contents, array %s has %d dimensions\n", name, (int)(mxGetNumberOfDimensions(pa)));
      /*if (mxIsFromGlobalWS(pa))
	printf("  and was a global variable when saved\n");
      else
      printf("  and was a local variable when saved\n");*/
      const mwSize * dims = mxGetDimensions(pa);
      int dims0 = (int)dims[0];
      int dims1 = (int)dims[1];
      Mat answer(dims0, dims1, CV_32S);
      if(strcmp(name, "Segmentation") == 0){
	ushort * pa_ptr = (ushort *)mxGetPr(pa);
	for(int i = 0; i < dims0; i++){
	  for(int j = 0; j < dims1; j++){
	    answer.at<int>(i,j) = (int)(pa_ptr[i+dims0*j]);
	  }
	}
      }
      
      if(strcmp(name, "labels") == 0){
	double * pa_ptr = (double *)mxGetPr(pa);
	for(int i = 0; i < dims0; i++){
	  for(int j = 0; j < dims1; j++){
	    answer.at<int>(i,j) = (int)(pa_ptr[i+dims0*j]);
	  }
	}
      }
	
      //cout << "Hello......................" << endl;
      //answer = answer.t();
      mxDestroyArray(pa);
      matClose(pmat);
      return answer;
    }
    mxDestroyArray(pa);
  }

  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    Mat answer;
    return answer;
  }
  //printf("Done\n");

  Mat answer;
  return answer;
}

/*************************************************************************************************/
Mat readUCM(const char * file){
  MATFile * pmat;
  const char ** dir;
  int  ndir;
  mxArray *pa;
  const char *name;
  
  //printf("Reading file %s...\n\n", file);

  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error opening file %s\n", file);
    Mat answer;
    return answer;
  }

  dir = (const char **)matGetDir(pmat, &ndir);
  if (dir == NULL) {
    printf("Error reading directory of file %s\n", file);
    matClose(pmat);
    Mat answer;
    return answer;;
  } else {
    //printf("Directory of %s:\n", file);
    //for (int i=0; i < ndir; i++)
    //printf("%s\n",dir[i]);
  }
  mxFree(dir);
  
  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    Mat answer;
    return answer;
  }
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error reopening file %s\n", file);
    Mat answer;
    return answer;
  }

  /* Get headers of all variables */
  //printf("\nExamining the header for each variable:\n");
  for (int i=0; i < ndir; i++) {
    pa = matGetNextVariableInfo(pmat, &name);
    if(strcmp(name, "ucm") == 0){
      if (pa == NULL) {
	printf("Error reading in file %s\n", file);
	Mat answer;
	mxDestroyArray(pa);
	matClose(pmat);
	return answer;
      }
      /* Diagnose header pa */
      //printf("According to its header, array %s has %d dimensions\n", name, (int)(mxGetNumberOfDimensions(pa)));
      /*if (mxIsFromGlobalWS(pa))
	printf("  and was a global variable when saved\n");
      else
	printf("  and was a local variable when saved\n");
      */
    }
    mxDestroyArray(pa);
  }

  /* Reopen file to read in actual arrays. */
  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    mxDestroyArray(pa);
    matClose(pmat);
    Mat answer;
    return answer;
  }
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error reopening file %s\n", file);
    mxDestroyArray(pa);
    matClose(pmat);
    Mat answer;
    return answer;
  }

  /* Read in each array. */
  //printf("\nReading in the actual array contents:\n");
  for (int i=0; i<ndir; i++) {
    pa = matGetNextVariable(pmat, &name);
    if(strcmp(name, "ucm") == 0){
      if (pa == NULL) {
	//printf("Error reading in file %s\n", file);
	mxDestroyArray(pa);
	matClose(pmat);
	Mat answer;
	return answer;
      } 
      /*
       * Diagnose array pa
       */
      //printf("According to its contents, array %s has %d dimensions\n", name, (int)(mxGetNumberOfDimensions(pa)));
      /*if (mxIsFromGlobalWS(pa))
	printf("  and was a global variable when saved\n");
      else
      printf("  and was a local variable when saved\n");*/
      const mwSize * dims = mxGetDimensions(pa);
      int dims0 = (int)dims[0];
      int dims1 = (int)dims[1];
      Mat answer(dims0, dims1, CV_8UC1);
      
      if(strcmp(name, "ucm") == 0){
	uchar * pa_ptr = (uchar *)mxGetPr(pa);
	for(int i = 0; i < dims0; i++){
	  for(int j = 0; j < dims1; j++){
	    answer.at<uchar>(i,j) = pa_ptr[i+dims0*j];
	  }
	}
      }
	
      //cout << "Hello......................" << endl;
      //answer = answer.t();
      mxDestroyArray(pa);
      matClose(pmat);
      return answer;
    }
    mxDestroyArray(pa);
  }

  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    Mat answer;
    return answer;
  }
  //printf("Done\n");

  Mat answer;
  return answer;
}


/*************************************************************************************************/
Mat readFloatMatrix(char * file, char * varName){
  MATFile * pmat;
  const char ** dir;
  int  ndir;
  mxArray *pa;
  const char *name;
  
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error opening file %s\n", file);
    Mat answer;
    return answer;
  }

  dir = (const char **)matGetDir(pmat, &ndir);
  if (dir == NULL) {
    printf("Error reading directory of file %s\n", file);
    matClose(pmat);
    Mat answer;
    return answer;;
  } else {
  }
  mxFree(dir);
  
  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    Mat answer;
    return answer;
  }
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error reopening file %s\n", file);
    Mat answer;
    return answer;
  }

  /* Get headers of all variables */
  for (int i=0; i < ndir; i++) {
    pa = matGetNextVariableInfo(pmat, &name);
    if(strcmp(name, varName) == 0){
      if (pa == NULL) {
	printf("Error reading in file %s\n", file);
	Mat answer;
	mxDestroyArray(pa);
	matClose(pmat);
	return answer;
      }
    }
    mxDestroyArray(pa);
  }

  /* Reopen file to read in actual arrays. */
  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    mxDestroyArray(pa);
    matClose(pmat);
    Mat answer;
    return answer;
  }
  pmat = matOpen(file, "r");
  if (pmat == NULL) {
    printf("Error reopening file %s\n", file);
    mxDestroyArray(pa);
    matClose(pmat);
    Mat answer;
    return answer;
  }

  /* Read in each array. */
  for (int i=0; i<ndir; i++) {
    pa = matGetNextVariable(pmat, &name);
    if(strcmp(name, varName) == 0){
      if (pa == NULL) {
	mxDestroyArray(pa);
	matClose(pmat);
	Mat answer;
	return answer;
      } 
      /*
       * Diagnose array pa
       */
      const mwSize * dims = mxGetDimensions(pa);
      int dims0 = (int)dims[0];
      int dims1 = (int)dims[1];
      Mat answer(dims0, dims1, CV_32F);
      
      if(strcmp(name, varName) == 0){
	double * pa_ptr = (double *)mxGetPr(pa);
	for(int i = 0; i < dims0; i++){
	  for(int j = 0; j < dims1; j++){
	    answer.at<float>(i,j) = (float)(pa_ptr[i+dims0*j]);
	  }
	}
      }
	
      //cout << "Hello......................" << endl;
      //answer = answer.t();
      mxDestroyArray(pa);
      matClose(pmat);
      return answer;
    }
    mxDestroyArray(pa);
  }

  if (matClose(pmat) != 0) {
    printf("Error closing file %s\n",file);
    Mat answer;
    return answer;
  }
  //printf("Done\n");

  Mat answer;
  return answer;
}

/*************************************************************************************************/
void writeMatrixToMatFile(cv::Mat matrix, char * variableName, char * fileName){
  MATFile * pmat;
  mxArray * pa;
  //cout << "Creating MAT file..." << endl;
  
  pmat = matOpen(fileName, "w");
  if (pmat == NULL) {
    cout << "writeMatrix: Error creating MAT file " << endl;
    exit(0);
  }
  
  Size wh = matrix.size();
  int height = wh.height;
  int width = wh.width;
  int channels = matrix.channels();
  if(channels != 1){
    cout << "writeMatrix: number of channels should be equal to 1" << endl;
    exit(0);
  }

  pa = mxCreateDoubleMatrix(height, width, mxREAL);
  if(pa == NULL){
    cout << "writeMatrix: Unable to create the mat file" << endl;
    exit(0);
  }

  double * data = new double[height*width];
  int count = 0;
  for(int j = 0; j < width; j++){
    for(int i = 0; i < height; i++){
      data[count++] = (double)(matrix.at<float>(i,j));
    }
  }

  memcpy((void*)(mxGetPr(pa)), (void*)(data), sizeof(double)*height*width);

  int status = matPutVariable(pmat, variableName, pa);
  if(status != 0){
    cout << "writeMatrix: error writing matrix to file" << endl;
    exit(0);
  }

  if(matClose(pmat) != 0){
    cout << "writeMatrix: could not close the mat file" << endl;
    exit(0);
  }

  mxDestroyArray(pa);
  delete [] data;
  
  //cout << "Done" << endl;
  
  return;
}


/*************************************************************************************************/
/*************************************************************************************************/
/*************************************************************************************************/
