#include "ObjectStuff.h"
#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "FeatureStuff.h"
#include "MyMATFileInterface.h"
#include <iostream>
#include <fstream>

using namespace std;
using namespace cv;

/******************************************************************************/
pair<Mat,Mat> trainingDataHierarchicalObjectClassifier(string datasetName, string objectName, const int trainVersion, const int numBins_lab, const int numBins_hog, const int k, const int k2, const int k3){
  Dataset db = dataset_setup(datasetName);

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);

  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);

  Mat trainData;
  Mat labels;

  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    int step = 10;
    int seqLength = db.getSequence(seqNumber).getSeqLength();
    string dataDir = db.getDataDir();
    string imType = db.getImType();
    string seqName = db.getSequence(seqNumber).getSeqName();
    //cout << seqName.c_str() << " " << seqNumber << " " << seqLength << endl;
    
    for(int f = 10; f <= seqLength; f+=step){
    //for(int f = 10; f < 700; f+=step){
      string gtPath = db.getGTObjectMaskPath(objectName.c_str(), seqNumber, f);
      Mat mask = readMaskFromImage(gtPath.c_str());
      if(mask.rows < 1)
	continue;
      //cout << gtPath << endl;
      //imshow("mask", mask);

      Mat im = db.getIM(seqNumber, f);
      Mat ucm = db.readUCM(seqNumber, f);
      PriorH ph(im, ucm, k, k2, k3, f);
      Mat F_ = ph.getHierarchicalFeature_colorHOG(clusterCenters_lab, clusterCenters_hog);

      Mat labels_ = ph.maskToLabel(mask);

      //cout << labels_.rows << " " << labels_.cols << endl;
      //cout << xfv.rows << " " << xfv.cols << endl;
      trainData.push_back(F_);
      labels.push_back(labels_);
      //cout << trainData.rows << " " << trainData.cols << endl;
      //cout << labels.rows << " " << labels.cols << endl;
      cout << ".";
      cout.flush();
    }
  }
  cout << endl;
  cout << "Size of the training data: " << trainData.rows << " " << trainData.cols << endl;

  pair<Mat,Mat> output;
  output.first = trainData;
  output.second = labels;

  return output;
}

/******************************************************************************/
void trainHierarchicalObjectClassifier(string datasetName, string objectName, const int trainVersion, CvSVM * trainer, const int numBins_lab, const int numBins_hog, const int k, const int k2, const int k3, const int kernelType){
  Dataset db = dataset_setup(datasetName);

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  if(kernelType == CvSVM::LINEAR)
    sprintf(buffer, "%s/opencv/Object/trainHierarchicalObjectClassifier_Linear_%s_%d_%d_%d_%d_%d_%d.bin", dataDir.c_str(), objectName.c_str(), trainVersion, numBins_lab, numBins_hog, k, k2, k3);
  if(kernelType == CvSVM::RBF)
    sprintf(buffer, "%s/opencv/Object/trainHierarchicalObjectClassifier_%s_%d_%d_%d_%d_%d_%d.bin", dataDir.c_str(), objectName.c_str(), trainVersion, numBins_lab, numBins_hog, k, k2, k3);
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer->load(buffer);
    //cout << "hello " << trainer->get_support_vector_count() << endl;
    return;
  }
  ifile.close();

  pair<Mat, Mat> trainingData = trainingDataHierarchicalObjectClassifier(datasetName, objectName, trainVersion, numBins_lab, numBins_hog, k,k2, k3);
  Mat trainData = trainingData.first;
  Mat labels = trainingData.second;

  Mat class_weights;
  class_weights = Mat::zeros(2, 1, CV_32F);
  for(int i = 0; i < labels.rows; i++)
    if(labels.at<float>(i,0) == 1)
      class_weights.at<float>(0,0)++;
    else
      class_weights.at<float>(1,0)++;
  float sum_weights = class_weights.at<float>(0,0) + class_weights.at<float>(1,0);
  class_weights.at<float>(0,0) /= sum_weights;
  class_weights.at<float>(1,0) /= sum_weights;

  cout << "class_weights: " << class_weights << endl;

  CvMat class_weights_[1];
  class_weights_[0] = class_weights;

  CvSVMParams params;
  params.svm_type = CvSVM::C_SVC;
  params.kernel_type = kernelType;
  //params.class_weights = class_weights_;
  params.term_crit = cvTermCriteria(CV_TERMCRIT_ITER, 1000, 1e-6);
  
  trainer->train(trainData, labels, Mat(), Mat(), params);
  
  trainer->save(buffer);

  return;
}

/******************************************************************************/
void trainHierarchicalObjectClassifier_linear(string datasetName, string objectName, const int trainVersion, LinearClassifier& trainer, const int numBins_lab, const int numBins_hog, const int k, const int k2, const int k3){
  Dataset db = dataset_setup(datasetName);

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/Object/trainHierarchicalObjectClassifier_wb_%s_%d_%d_%d_%d_%d_%d.bin", dataDir.c_str(), objectName.c_str(), trainVersion, numBins_lab, numBins_hog, k, k2, k3);

  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer.load(buffer);
    //cout << "hello " << trainer->get_support_vector_count() << endl;
    return;
  }
  ifile.close();

  CvSVM * trainer_ = new CvSVM();
  
  trainHierarchicalObjectClassifier(datasetName, objectName, trainVersion, trainer_, numBins_lab, numBins_hog, k, k2, k3, CvSVM::LINEAR);

  trainer = getWeightVectorLinearSVM(trainer_);

  delete trainer_;

  trainer.save(buffer);

  return;
}

/******************************************************************************/
void trainHierarchicalObjectClassifier_svmlight(string datasetName, string objectName, const int trainVersion, LinearClassifier& trainer, const int numBins_lab, const int numBins_hog, const int k, const int k2, const int k3){
  Dataset db = dataset_setup(datasetName);

  char buffer[500];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/Object/svm_light/trainHierarchicalObjectClassifier_svmlight_%s_%d_%d_%d_%d_%d_%d.bin", dataDir.c_str(), objectName.c_str(), trainVersion, numBins_lab, numBins_hog, k, k2, k3);

  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    trainer.load(buffer);
    trainer.w = -trainer.w;
    trainer.b = -trainer.b;
    //cout << "hello " << trainer->get_support_vector_count() << endl;
    return;
  }
  ifile.close();

  char * params = "-t 0";

  char tempFile[500];
  tempFile[0] = 0;
  sprintf(tempFile, "%s/opencv/Object/svm_light/trainHierarchicalObjectClassifier_svmlight_tempFile_%s_%d_%d_%d_%d_%d_%d.bin", dataDir.c_str(), objectName.c_str(), trainVersion, numBins_lab, numBins_hog, k, k2, k3);

  cout << tempFile << endl;

  char modelFile[500];
  modelFile[0] = 0;
  sprintf(modelFile, "%s/opencv/Object/svm_light/trainHierarchicalObjectClassifier_svmlight_modelFile_%s_%d_%d_%d_%d_%d_%d.bin", dataDir.c_str(), objectName.c_str(), trainVersion, numBins_lab, numBins_hog, k, k2, k3);

  cout << modelFile << endl;

  pair<Mat, Mat> trainingData = trainingDataHierarchicalObjectClassifier(datasetName, objectName, trainVersion, numBins_lab, numBins_hog, k,k2, k3);
  Mat trainData = trainingData.first;
  Mat labels = trainingData.second;

  trainer = svm_light_train(trainData, labels, tempFile, modelFile, params);

  trainer.save(buffer);

  char command[500];
  command[0] = 0;
  sprintf(command, "rm -f %s", tempFile);
  system(command);

  command[0] = 0;
  sprintf(command, "rm -f %s", modelFile);
  system(command);

  trainer.w = -trainer.w;
  trainer.b = -trainer.b;

  return;
}

/******************************************************************************/
Mat hierarchicalObjectClassification(PriorH& ph, CvSVM * trainer, const Mat& clusterCenters_lab, const Mat& clusterCenters_hog){
  Mat F = ph.getHierarchicalFeature_colorHOG(clusterCenters_lab, clusterCenters_hog);
  int m = F.rows;
  Mat output(m, 1, CV_32F);

  Mat temp;
  for(int i = 0; i < m; i++){
    temp = F.rowRange(i,i+1);
    output.at<float>(i,0) = -trainer->predict(temp, true);
  }
  
  return output;
}

/******************************************************************************/
Mat hierarchicalObjectClassification(PriorH& ph, const LinearClassifier& trainer, const Mat& clusterCenters_lab, const Mat& clusterCenters_hog){
  Mat F = ph.getHierarchicalFeature_colorHOG(clusterCenters_lab, clusterCenters_hog);
  int m = F.rows;

  Mat output = F * trainer.w.t();
  output += trainer.b;
  output = -output;

  return output;
}

/******************************************************************************/
Mat hierarchicalObjectClassification(PriorH& ph, std::list<CvSVM *>& trainers, const Mat& clusterCenters_lab, const Mat& clusterCenters_hog){
  //cout << "hello" << endl;
  Mat F = ph.getHierarchicalFeature_colorHOG(clusterCenters_lab, clusterCenters_hog);
  //cout << F.rows << " " << F.cols << endl;
  int n = trainers.size();
  int m = F.rows;
  Mat output(m, n, CV_32F);

  Mat temp;
  int j = 0;
  for(list<CvSVM *>::iterator it = trainers.begin(); it != trainers.end(); it++){
    //cout << j << endl;
    for(int i = 0; i < m; i++){
      temp = F.rowRange(i,i+1);
      output.at<float>(i,j) = -(*it)->predict(temp, true);
      //cout << i << " " << j << " " << output.at<float>(i,j) << endl;
    }
    j++;
  }
  
  return output;  
}

/******************************************************************************/
Mat hierarchicalObjectClassification(PriorH& ph, vector<LinearClassifier>& trainers, const Mat& clusterCenters_lab, const Mat& clusterCenters_hog){
  Mat F = ph.getHierarchicalFeature_colorHOG(clusterCenters_lab, clusterCenters_hog);
  int n = trainers.size();
  int m = F.rows;

  Mat output;
  for(int t = 0; t < n; t++){
    LinearClassifier trainer = trainers[t];
    Mat output_ = F * trainer.w.t();
    output_ += trainer.b;
    output_ = -output_.t();
    output.push_back(output_);
  }
  output = output.t();
  
  return output;
}

/******************************************************************************/
Mat classificationToLabels(const Mat& classificationMatrix){
  float coef1 = 0.0; /* if max value is smaller than this, then it is unknown */
  float coef2 = 0.1; /* if the difference between max value and second max value is smaller than this, then unknown */
  int m = classificationMatrix.rows;
  int n = classificationMatrix.cols;
  Mat output(m, 1, CV_32S);
  int maxY,maxX;
  float maxVal;
  for(int i = 0; i < m; i++){
    maxVal = matrixMaxValue<float>(classificationMatrix.rowRange(i,i+1), maxY, maxX);
    if(maxVal < coef1){
      output.at<int>(i,0) = n;
      continue;
    }
    output.at<int>(i,0) = maxX;
  }

  return output;
}

/******************************************************************************/
/******************************************************************************/
