#ifndef PRIOR_STUFF_H
#define PRIOR_STUFF_H

#include <opencv2/opencv.hpp>

class Prior{

 public:
  Prior();
  Prior(const Prior& p);
  Prior(const cv::Mat& image, const int frameNumber); // felzenzswalb
  Prior(const cv::Mat& image){Prior(image, -1);} // felzenzswalb
  Prior(const cv::Mat& image, const cv::Mat& ucm, const int k, const int frameNumber); // Berkeley
  Prior(const cv::Mat& image, const cv::Mat& ucm, const int k){Prior(image, ucm, k, -1);} // Berkeley

  void setSegmentation(const cv::Mat& segmentation); // Never use this, unless you know what you are doing!
  void setSegmentation_felz(const cv::Mat& segmentation); // felzenszwalb
  void setSegmentation_berkeley(const cv::Mat& ucm, const int k); // Berkeley

  void setFrameNumber(const int frameNumber);
  int getFrameNumber() const;
  cv::Mat getImage() const;
  cv::Mat getSegmentation() const;
  cv::SparseMat getAdjacencyWeights() const;
  cv::SparseMat getAdjacencyLengths() const;
  cv::SparseMat getW() const;
  int getNumSegments() const;
  cv::Mat getCenters();
  cv::Mat getPrior_size();
  cv::Mat getPrior_location(const cv::Mat& locationPattern);
  cv::Mat getPrior_motionChannel(std::map<int, Prior>& priors, const std::vector<int>& adjFrames);
  cv::Mat getPrior_motionChannel(const std::vector<std::pair<cv::Mat,cv::Mat> >& flows);

  cv::Mat getColorMap(const cv::Mat& clusterCenters);
  cv::Mat getHogMap(const cv::Mat& clusterCenters);

  cv::Mat getFeature_CLab(const cv::Mat& clusterCenters);
  cv::Mat getFeature_HOG(const cv::Mat& clusterCenters);

  cv::Mat getFeatureAroundPoint_CLab(const cv::Mat& clusterCenters, const float cy, const float cx, const int radius);
  cv::Mat getFeatureAroundPoint_HOG(const cv::Mat& clusterCenters, const float cy, const float cx, const int radius);
  cv::Mat getFeatureAroundPoint_colorHOG(const cv::Mat& clusterCenters_lab, const cv::Mat& clusterCenters_hog, const float cy, const float cx, const int radius);

  cv::Mat getFeatureInMask_CLab(const cv::Mat& clusterCenters, const cv::Mat& mask);
  cv::Mat getFeatureInMask_HOG(const cv::Mat& clusterCenters, const cv::Mat& mask);
  cv::Mat getFeatureInMask_colorHOG(const cv::Mat& clusterCenters_lab, const cv::Mat& clusterCenters_hog, const cv::Mat& mask);

  cv::Mat maskToLabel(const cv::Mat& mask);
  cv::Mat labelsToMask(const cv::Mat& labels);

  cv::Mat copyCenters() const {return centers_;}
  cv::Mat copyPrior_size() const {return prior_size_;}
  cv::Mat copyPrior_location() const {return prior_location_;}
  cv::Mat copyPrior_motionChannel() const {return prior_motionChannel_;}
  cv::Mat copyColorMap() const {return colorMap_;}
  cv::Mat copyHogMap() const {return hogMap_;}

  int getM() const{return m_;}

  bool isCentersComputed() const{return computed_centers_;}
  bool isSizeComputed() const{return computed_size_;}
  bool isLocationComputed() const{return computed_location_;}
  bool isMotionChannelComputed() const{return computed_motionChannel_;}
  bool isHogMapComputed() const{return computed_hogmap_;}
  bool isColorMapComputed() const{return computed_colormap_;}

 protected:
  int f_; // frame number
  int m_; // number of regions

  bool computed_size_;
  bool computed_centers_;
  bool computed_location_;
  bool computed_motionChannel_;
  bool computed_hogmap_;
  bool computed_colormap_;

  cv::Mat image_;
  cv::Mat segmentation_;
  cv::SparseMat adjacencyWeights_; // strength of contours between regions
  cv::SparseMat adjacencyLengths_; // length of contours between regions
  cv::Mat prior_size_;
  cv::Mat centers_;
  cv::Mat prior_location_;
  cv::Mat prior_motionChannel_;
  cv::Mat colorMap_;
  cv::Mat hogMap_;  
};





/******************************************/
class PriorH: public Prior{
 public:
  PriorH(const cv::Mat& image, const cv::Mat& ucm, const int k, const int k2, const int k3, const int frameNumber);
  cv::Mat getHierarchicalFeature_CLab(const cv::Mat& clusterCenters);
  cv::Mat getHierarchicalFeature_HOG(const cv::Mat& clusterCenters);
  cv::Mat getHierarchicalFeature_colorHOG(const cv::Mat& colorClusterCenters, const cv::Mat& hogClusterCenters);

 private:
  cv::Mat segmentation2_;
  cv::Mat segmentation3_;
};



#endif //PRIOR_STUFF_H
