#include "SegmentationStuff.h"
#include "graphcut/GCoptimization.h"

#include <list>
#include <map>

#include <cstdio>
#include <cstdlib>

extern "C" {
#include <vl/generic.h>
#include <vl/mathop.h>
#include <vl/quickshift.h>
}

using namespace std;
using namespace cv;


/*************************************************************************/
inline float diff(image<float> *r, image<float> *g, image<float> *b, int x1, int y1, int x2, int y2){
  return sqrt(square(imRef(r, x1, y1)-imRef(r, x2, y2)) +
	      square(imRef(g, x1, y1)-imRef(g, x2, y2)) +
	      square(imRef(b, x1, y1)-imRef(b, x2, y2)));
}

/*************************************************************************/
Mat segmentationForTest12(int height, int width){
  Mat answer(height, width, CV_32S);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width/2; j++)
      answer.at<int>(i,j) = 1;
  for(int i = 0; i < height; i++)
    for(int j = width/2; j < width; j++)
      answer.at<int>(i,j) = 2;
  return answer;
}

/*************************************************************************/
Mat segmentationForTest22(int height, int width){
  Mat answer(height, width, CV_32S);
  for(int i = 0; i < height/2; i++)
    for(int j = 0; j < width/2; j++)
      answer.at<int>(i,j) = 1;
  for(int i = 0; i < height/2; i++)
    for(int j = width/2; j < width; j++)
      answer.at<int>(i,j) = 2;
  for(int i = height/2; i < height; i++)
    for(int j = 0; j < width/2; j++)
      answer.at<int>(i,j) = 3;
  for(int i = height/2; i < height; i++)
    for(int j = width/2; j < width; j++)
      answer.at<int>(i,j) = 4;
  return answer;
}

/*************************************************************************/
Mat noisyImageForTestRedBlueRedBlue(int height, int width){
  Mat r1(height, width/4, CV_8UC3);
  Mat b1(height, width/4, CV_8UC3);
  Mat r2(height, width/4, CV_8UC3);
  Mat b2(height, width/4, CV_8UC3);
  typedef Vec<uchar,3> Vec3u;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width/4; j++){
      for(int ch = 0; ch < 3; ch++){
	r1.at<Vec3u>(i,j)[ch] = 0;
	r2.at<Vec3u>(i,j)[ch] = 0;
	b1.at<Vec3u>(i,j)[ch] = 0;
	b2.at<Vec3u>(i,j)[ch] = 0;
      }
      uchar temp = uchar(100+round(155.0*(double)(rand())/(double)(RAND_MAX)));
      //cout << (int)(temp) << endl;
      r1.at<Vec3u>(i,j)[2] = uchar(200+round(55.0*(double)(rand())/(double)(RAND_MAX)));
      r2.at<Vec3u>(i,j)[2] = uchar(200+round(55.0*(double)(rand())/(double)(RAND_MAX)));
      b1.at<Vec3u>(i,j)[0] = uchar(200+round(55.0*(double)(rand())/(double)(RAND_MAX)));
      b2.at<Vec3u>(i,j)[0] = uchar(200+round(55.0*(double)(rand())/(double)(RAND_MAX)));
    }
  }
  Mat answer(height, (width/4)*4, CV_8UC3);
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width/4; j++){
      for(int ch = 0; ch < 3; ch++){
	answer.at<Vec3u>(i,j)[ch] = r1.at<Vec3u>(i,j)[ch];
	answer.at<Vec3u>(i,j+1*(width/4))[ch] = b1.at<Vec3u>(i,j)[ch];
	answer.at<Vec3u>(i,j+2*(width/4))[ch] = r2.at<Vec3u>(i,j)[ch];
	answer.at<Vec3u>(i,j+3*(width/4))[ch] = b2.at<Vec3u>(i,j)[ch];
      }
    }
  }
  return answer;
}

/*************************************************************************/
Mat felzenszwalb_segment(const Mat& inputImage, float sigma, float c, int min_size, int *num_ccs){

  Size wh = inputImage.size();
  int width = wh.width;
  int height = wh.height;

  const uchar * rawPtr = inputImage.ptr();

  rgb * rawInput = new rgb[height*width];
  int count = 0;
  for(int i = 0; i < height*width; i++){
    rawInput[i].b = rawPtr[count++];
    rawInput[i].g = rawPtr[count++];
    rawInput[i].r = rawPtr[count++];
  }

  image<rgb>* im = new image<rgb>(width, height);
  for(int i = 0; i < height*width; i++)
    im->data[i] = rawInput[i];
  
  image<float> *r = new image<float>(width, height);
  image<float> *g = new image<float>(width, height);
  image<float> *b = new image<float>(width, height);

  // smooth each color channel  
  for (int y = 0; y < height; y++) {
    for (int x = 0; x < width; x++) {
      imRef(r, x, y) = imRef(im, x, y).r;
      imRef(g, x, y) = imRef(im, x, y).g;
      imRef(b, x, y) = imRef(im, x, y).b;
    }
  }
  image<float> *smooth_r = smooth(r, sigma);
  image<float> *smooth_g = smooth(g, sigma);
  image<float> *smooth_b = smooth(b, sigma);
  delete r;
  delete g;
  delete b;
 
  // build graph
  edge *edges = new edge[width*height*4];
  int num = 0;
  for (int y = 0; y < height; y++) {
    for (int x = 0; x < width; x++) {
      if (x < width-1) {
	edges[num].a = y * width + x;
	edges[num].b = y * width + (x+1);
	edges[num].w = diff(smooth_r, smooth_g, smooth_b, x, y, x+1, y);
	num++;
      }

      if (y < height-1) {
	edges[num].a = y * width + x;
	edges[num].b = (y+1) * width + x;
	edges[num].w = diff(smooth_r, smooth_g, smooth_b, x, y, x, y+1);
	num++;
      }

      if ((x < width-1) && (y < height-1)) {
	edges[num].a = y * width + x;
	edges[num].b = (y+1) * width + (x+1);
	edges[num].w = diff(smooth_r, smooth_g, smooth_b, x, y, x+1, y+1);
	num++;
      }

      if ((x < width-1) && (y > 0)) {
	edges[num].a = y * width + x;
	edges[num].b = (y-1) * width + (x+1);
	edges[num].w = diff(smooth_r, smooth_g, smooth_b, x, y, x+1, y-1);
	num++;
      }
    }
  }
  delete smooth_r;
  delete smooth_g;
  delete smooth_b;

  // segment
  universe *u = segment_graph(width*height, num, edges, c);
  
  // post process small components
  for (int i = 0; i < num; i++) {
    int a = u->find(edges[i].a);
    int b = u->find(edges[i].b);
    if ((a != b) && ((u->size(a) < min_size) || (u->size(b) < min_size)))
      u->join(a, b);
  }
  delete [] edges;
  *num_ccs = u->num_sets();

  Mat output = universe2segmentation(u, height, width);

  delete u;

  return output;
}

/*************************************************************************/
vector< pair<int,int> > getContours(const cv::Mat & segmentation){

  clock_t startC, endC;
  
  startC = clock();
  int num_ccs; 

  vector < pair<int,int> > answer;
  int h = segmentation.rows;
  int w = segmentation.cols;

  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      int sv = (int)(segmentation.at<int>(i,j));
      bool isContour = false;
      for(int ii=max(i-1,0); ii <= min(i+1,h-1); ii++){
	for(int jj=max(j-1,0); jj <= min(j+1,w-1); jj++){
	  if(isContour)
	    break;
	  int dv = (int)(segmentation.at<int>(ii,jj));
	  if(sv != dv)
	    isContour = true;
	}
      }
      if(isContour){
	pair<int,int> temp(i,j);
	answer.push_back(temp);
      }
    }
  }

  endC = clock();

  //cout << "Time required for getContours: " << (double)(endC-startC)/CLOCKS_PER_SEC << " seconds." << endl;

  return answer;
}

/*************************************************************************/
Mat getContours_ucm(const Mat & segmentation){
  int h = segmentation.rows;
  int w = segmentation.cols;
  Mat output(h*2+1, w*2+1, CV_8U);

  for(int i = 0; i < h*2+1; i++)
    for(int j = 0; j < w*2+1; j++)
      output.at<uchar>(i,j) = 0;

  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      int sv = (int)(segmentation.at<int>(i,j));
      for(int ii=max(i-1,0); ii <= min(i+1,h-1); ii++){
	for(int jj=max(j-1,0); jj <= min(j+1,w-1); jj++){
	  int dv = (int)(segmentation.at<int>(ii,jj));
	  if(sv != dv)
	    output.at<uchar>(i+ii+1, j+jj+1) = 255;
	}
      }
    }
  }

  for(int j = 0; j < w*2+1; j++){
    output.at<uchar>(h*2,j) = output.at<uchar>(h*2-1,j);
    output.at<uchar>(0,j) = output.at<uchar>(1,j);
  }


  for(int i = 0; i < h*2+1; i++){
    output.at<uchar>(i,w*2) = output.at<uchar>(i,w*2-1);
    output.at<uchar>(i,0) = output.at<uchar>(i,1);
  }

  return output;
}

/*************************************************************************/
vector< pair<int,int> > getContourBetweenTwoRegions(const cv::Mat & segmentation, int rg1, int rg2){
  vector < pair<int,int> > answer;
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;

  for(int i = 1; i < height-1; i++){
    for(int j = 1; j < width-1; j++){
      int sv = (int)(segmentation.at<int>(i,j));
      if(sv != rg1)
	continue;
      bool isContour = false;
      for(int ii=i-1; ii <= i+1; ii++){
	for(int jj=j-1; jj <= j+1; jj++){
	  if(isContour)
	    break;
	  int dv = (int)(segmentation.at<int>(ii,jj));
	  if(dv == rg2)
	    isContour = true;
	}
      }
      if(isContour){
	pair<int,int> temp(i,j);
	answer.push_back(temp);
      }
    }
  }
  
  return answer;
}

/*************************************************************************/
int sortSegmentNumbers(cv::Mat & segmentation){
  int count = 1;
  map<int, int> num2sorted;
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      int temp = (int)(segmentation.at<int>(i,j));
      if(num2sorted.find(temp) == num2sorted.end())
	num2sorted[temp] = count++;
    }
  }
  int m = count - 1;
  // re-assigning the segment labels
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      int temp = (int)(segmentation.at<int>(i,j));
      segmentation.at<int>(i,j) = num2sorted[temp];
    }
  }

  return m;
}


/*************************************************************************/
cv::SparseMat getWeightedAdjacencyMatrix(const cv::Mat & segmentation){
  // The assumption is that region numbers are sorted from 1 to m
  int dummyY, dummyX;
  int m = matrixMaxValue<int>(segmentation, dummyY, dummyX);
  int dims = 2;
  int sizes[2];
  sizes[0] = m;
  sizes[1] = m;
  int type = CV_32F;
  SparseMat output = SparseMat(dims, sizes, type);

  int height = segmentation.rows;
  int width = segmentation.cols;

  int idx[2];

  for(int i = 0; i < height-1; i++){
    for(int j = 0; j < width-1; j++){
      int sv = (int)(segmentation.at<int>(i,j));
      bool isContour = false;
      for(int ii=i; ii <= i+1; ii++){
	for(int jj=j; jj <= j+1; jj++){
	  if(isContour)
	    break;
	  int dv = (int)(segmentation.at<int>(ii,jj));
	  if(sv != dv){
	    isContour = true;
	    idx[0] = sv-1; idx[1] = dv-1;
	    output.ref<float>(idx)++;
	    idx[0] = dv-1; idx[1] = sv-1;
	    output.ref<float>(idx)++;
	  }
	}
      }
    }
  }
  
  return output;
}

/*************************************************************************/
std::vector< std::pair<float,float> > getSegmentCenters(const cv::Mat & segmentation){
  // The assumption is that region numbers are sorted from 1 to m
  int dummyY, dummyX;
  int m = matrixMaxValue<int>(segmentation, dummyY, dummyX);
  float * cy = new float[m];
  float * cx = new float[m];
  float * ccount = new float[m];
  for(int i = 0; i < m; i++){
    cy[i] = 0;
    cx[i] = 0;
    ccount[i] = 0;
  }
  
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;

  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      int s = segmentation.at<int>(i,j);
      cy[s-1] += (float)(i);
      cx[s-1] += (float)(j);
      ccount[s-1] += 1;
    }
  }

  vector< pair<float,float> > output;
  pair<float,float> temp;
  for(int i = 0; i < m; i++){
    temp.first = 0;
    temp.second = 0;
    output.push_back(temp);
    if(ccount[i] == 0){
      cout << "Segment " << i+1 << " is empty!!!" << endl;
      continue;
    }
    temp.first = cy[i] / ccount[i];
    temp.second = cx[i] / ccount[i];
    output[i] = temp;
  }
   
  delete [] cy;
  delete [] cx;
  delete [] ccount;

  return output;
}

/*************************************************************************/
edge * dadjacency2edges(const SparseMat& adjacency, int & edgeLength /*this is output*/){
  edgeLength = (int)(adjacency.nzcount());
  edge * edges = new edge[edgeLength];

  double s = 0;
  int ind = 0;
  for(SparseMat::const_iterator it = adjacency.begin(); it != adjacency.end(); ++it){
    edges[ind].a = it.node()->idx[0];
    edges[ind].b = it.node()->idx[1];
    edges[ind].w = it.value<float>();
    ind++;
  }
  return edges;
}

/*************************************************************************/
Mat universe2segmentation(universe* u, int height, int width, bool widthFirst){
  Mat answer(height, width, CV_32S);
  map<long int, int> ind2seg;
  int count = 1;
  long int ind1, ind2;
  if(!widthFirst){
    for(int Y = 0; Y < height; Y++){
      for(int X = 0; X < width; X++){
	YXLocationToIndex(ind1, Y, X, height, width, 0);
	ind2 = u->find(ind1);
	if(ind2seg.find(ind2) == ind2seg.end()){
	  ind2seg[ind2] = count;
	  answer.at<int>(Y,X) = count;
	  count++;
	}
	else{
	  answer.at<int>(Y,X) = ind2seg[ind2];
	}
      }
    }
  }
  if(widthFirst){
    for(int Y = 0; Y < height; Y++){
      for(int X = 0; X < width; X++){
	YXLocationToIndex(ind1, X, Y, width, height, 0);
	ind2 = u->find(ind1);
	if(ind2seg.find(ind2) == ind2seg.end()){
	  ind2seg[ind2] = count;
	  answer.at<int>(Y,X) = count;
	  count++;
	}
	else{
	  answer.at<int>(Y,X) = ind2seg[ind2];
	}
      }
    }
  }
  //cout << "Number of regions are: " << count-1 << endl;
  return answer;
}

/*************************************************************************/
cv::Mat parents2segmentation(int* parents, int height, int width){
  universe * u = new universe(height*width);
  int p;
  for(int i = 0; i < height*width; i++){
    p = parents[i];
    //int p1 = u->find(i);
    //int p2 = u->find(p);
    u->join(i, p);
  }

  for(int i = 0; i < height*width; i++){
    p = parents[i];
    int p1 = u->find(i);
    int p2 = u->find(p);
    u->join(p1,p2);
  }
  
  Mat answer = universe2segmentation(u, height, width, true);
  return answer;
}

/*************************************************************************/
vector<int> universe2segmentation(universe* u, int length){
  vector<int> segmentation;
  int count = 1;
  map<long int, int> ind2seg;
  long int ind1, ind2;
  for(ind1 = 0; ind1 < length; ind1++){
    ind2 = u->find(ind1);
    if(ind2seg.find(ind2) == ind2seg.end()){
      ind2seg[ind2] = count;
      segmentation.push_back(count);
      count++;
    }
    else{
      segmentation.push_back(ind2seg[ind2]);
    }
  }
  return segmentation;
}


/*************************************************************************/
cv::SparseMat dadjacency2sadjacency(const cv::SparseMat& adjacency, const double sigma){
  int dims = adjacency.dims();
  int idx[dims];
  const int * sizes = adjacency.size();
  int type = adjacency.type();
  SparseMat answer(dims, sizes, type);
  
  for(SparseMat::const_iterator it = adjacency.begin(); it != adjacency.end(); ++it){
    for(int i = 0; i < dims; i++)
      idx[i] = it.node()->idx[i];
    answer.ref<float>(idx) = exp(-(it.value<float>())/sigma);
  }
  return answer;
}

/*************************************************************************/
cv::Mat sadjacency2regionSadjacency(const cv::SparseMat& adjacency, const cv::Mat segmentation, bool symmetricAnswer){
  vector<int> seg;
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;
  
  int regionNum = 0;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      int temp = segmentation.at<int>(i,j);
      if(temp > regionNum)
	regionNum = temp;
      seg.push_back(temp);
    }
  }

  if(regionNum < 1){
    cout << "adjacency2regionAdjacency: Zero regions!" << endl;
  }

  //cout << "Number of regions are: " << regionNum << endl;

  float myEpsilon = 0.000001;

  int dims = 2;
  int size[dims];
  size[0] = regionNum;
  size[1] = regionNum;
  Mat answer(regionNum, regionNum, CV_32F);
  for(int i = 0; i < regionNum; i++)
    for(int j = 0; j < regionNum; j++)
      answer.at<float>(i,j) = myEpsilon;

  int ind1, ind2;
  int region1, region2;
  for(SparseMat::const_iterator it = adjacency.begin(); it != adjacency.end(); ++it){
    ind1 = it.node()->idx[0];
    ind2 = it.node()->idx[1];
    region1 = seg[ind1];
    region2 = seg[ind2];
    float value = it.value<float>();
    //cout << value << endl;
    answer.at<float>(region1-1, region2-1) += value;
    //answer.at<float>(region1-1, region2-1) = 1.0;
  }

  for(int i = 0; i < regionNum; i++)
    answer.at<float>(i,i) = 0;
  
  int otherIdx[2];
  if(symmetricAnswer){
    for(int i = 0; i < regionNum; i++){
      for(int j = i+1; j < regionNum; j++){
	float value1 = answer.at<float>(i,j);
	float value2 = answer.at<float>(j,i);
	if(value1 > value2)
	  answer.at<float>(i,j) = value2;
	if(value2 > value1)
	  answer.at<float>(j,i) = value1;
      }
    }
  }
  
  return answer;
}

/*************************************************************************/
cv::Mat sadjacency2regionSadjacency_mean(const cv::SparseMat& adjacency, const cv::Mat segmentation, bool symmetricAnswer){
  vector<int> seg;
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;
  
  int regionNum = 0;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      int temp = segmentation.at<int>(i,j);
      if(temp > regionNum)
	regionNum = temp;
      seg.push_back(temp);
    }
  }

  if(regionNum < 1){
    cout << "adjacency2regionAdjacency: Zero regions!" << endl;
  }

  //cout << "Number of regions are: " << regionNum << endl;

  float myEpsilon = 0.000001;

  int dims = 2;
  int size[dims];
  size[0] = regionNum;
  size[1] = regionNum;
  Mat answer(regionNum, regionNum, CV_32F);
  for(int i = 0; i < regionNum; i++)
    for(int j = 0; j < regionNum; j++)
      answer.at<float>(i,j) = 0;
  Mat connectionCount(regionNum, regionNum, CV_32F);
  for(int i = 0; i < regionNum; i++)
    for(int j = 0; j < regionNum; j++)
      connectionCount.at<float>(i,j) = myEpsilon;
  

  int ind1, ind2;
  int region1, region2;
  for(SparseMat::const_iterator it = adjacency.begin(); it != adjacency.end(); ++it){
    ind1 = it.node()->idx[0];
    ind2 = it.node()->idx[1];
    region1 = seg[ind1];
    region2 = seg[ind2];
    float value = it.value<float>();
    //cout << value << endl;
    answer.at<float>(region1-1, region2-1) += value;
    connectionCount.at<float>(region1-1, region2-1) += 1;
    //answer.at<float>(region1-1, region2-1) = 1.0;
  }

  for(int i = 0; i < regionNum; i++)
    for(int j = 0; j < regionNum; j++)
      answer.at<float>(i,j) /= connectionCount.at<float>(i,j);;
  
  for(int i = 0; i < regionNum; i++)
    answer.at<float>(i,i) = 0;

  int otherIdx[2];
  if(symmetricAnswer){
    for(int i = 0; i < regionNum; i++){
      for(int j = i+1; j < regionNum; j++){
	float value1 = answer.at<float>(i,j);
	float value2 = answer.at<float>(j,i);
	if(value1 > value2)
	  answer.at<float>(i,j) = value2;
	if(value2 > value1)
	  answer.at<float>(j,i) = value1;
      }
    }
  }
  
  return answer;
}

/*************************************************************************/
vector<int> presegmentImage(SparseMat& dadjacency, float thresh){
  vector< list<int> > neighbors;
  vector<float> distances;
  float veryBig = 10000000.0;
  int height = dadjacency.size()[0];
  list<int> emptyList;
  for(int i = 0; i < height; i++){
    distances.push_back(veryBig);
    neighbors.push_back(emptyList);
  }

  int ind1, ind2;
  float value;
  for(SparseMat::const_iterator it = dadjacency.begin(); it != dadjacency.end(); ++it){
    value = it.value<float>();
    ind1 = it.node()->idx[0];
    ind2 = it.node()->idx[1];
    if(value == distances[ind1]){
      neighbors[ind1].push_back(ind2);
    }
    if(value < distances[ind1]){
      distances[ind1] = value;
      neighbors[ind1].clear();
      neighbors[ind1].push_back(ind2);
    }
  }

  for(SparseMat::const_iterator it = dadjacency.begin(); it != dadjacency.end(); ++it){
    value = it.value<float>();
    ind1 = it.node()->idx[0];
    ind2 = it.node()->idx[1];
    if(value <= thresh)
      neighbors[ind1].push_back(ind2);
  }  

  universe * u = new universe(height);
  for(int ind1 = 0; ind1 < height; ind1++){
    list<int> lst = neighbors[ind1];
    for(list<int>::iterator it = lst.begin(); it != lst.end(); it++){
      int ind2 = *it;
      int p1 = u->find(ind1);
      int p2 = u->find(ind2);
      u->join(p1, p2);
    }
  }  

  vector<int> segmentation = universe2segmentation(u, height);

  return segmentation;
}

/*************************************************************************/
Mat meanShiftSegmentation(const Mat& inputImage, double kernel_size, double max_dist){
  Size wh = inputImage.size();
  int width = wh.width;
  int height = wh.height;
  int channels = inputImage.channels();
  
  int count = 0;
  typedef Vec<uchar,3> Vec3u;
  double * imdata = new double[height*width*channels];
  for(int k = 2; k >= 0; k--)
    for(int j = 0; j < width; j++)
      for(int i = 0; i < height; i++)
	imdata[count++] = (double)(inputImage.at<Vec3u>(i,j)[k]);

  VlQS * qsh = vl_quickshift_new(imdata, height, width, channels);
  vl_quickshift_set_kernel_size(qsh, kernel_size);
  vl_quickshift_set_max_dist(qsh, max_dist);
  vl_quickshift_process(qsh);
  int * parents = vl_quickshift_get_parents(qsh);

  Mat segmentation = parents2segmentation(parents, height, width);

  // delete things
  vl_quickshift_delete(qsh);
  delete [] imdata;

  return segmentation;
}


/*************************************************************************/
Mat berkeley_segment(const cv::Mat& ucm, const int k){
  int ysize = ucm.rows;
  int xsize = ucm.cols;
  Mat ucmK = Mat::zeros(ysize, xsize, CV_8UC1);
  for(int i = 0; i < ysize; i++)
    for(int j = 0; j < xsize; j++)
      if(ucm.at<uchar>(i,j) > k)
	ucmK.at<uchar>(i,j) = ucm.at<uchar>(i,j);

  Mat labels = bwLabel(ucmK);
  
  int h = (ysize-1)/2;
  int w = (xsize-1)/2;
  Mat segmentation(h, w, CV_32S);
  for(int i = 1; i < ysize; i+=2)
    for(int j = 1; j < xsize; j+=2)
      segmentation.at<int>((i-1)/2, (j-1)/2) = labels.at<int>(i,j);

  sortSegmentNumbers(segmentation);

  return segmentation;
}

/*************************************************************************/
Mat berkeley_ucmResize(const Mat& ucm){
  int ysize = ucm.rows;
  int xsize = ucm.cols;
  int h = (ysize-1)/2;
  int w = (xsize-1)/2;
  Mat ucms(h, w, CV_8UC1);
  for(int i = 0; i < ysize-1; i+=2)
    for(int j = 0; j < xsize-1; j+=2)
      ucms.at<uchar>(i/2, j/2) = ucm.at<uchar>(i,j);
  return ucms;
}

/*************************************************************************/
void berkeley_weightedAdjacencyMatrix(const Mat& ucm, const Mat & segmentation, SparseMat& returnContourLengths, SparseMat& returnContourWeights){
  int dummyY, dummyX;
  int m = matrixMaxValue<int>(segmentation, dummyY, dummyX);
  int dims = 2;
  int sizes[2];
  sizes[0] = m;
  sizes[1] = m;
  SparseMat contourLengths = SparseMat(dims, sizes, CV_32F);
  SparseMat contourWeights = SparseMat(dims, sizes, CV_32F);

  int height = segmentation.rows;
  int width = segmentation.cols;

  Mat ucms = berkeley_ucmResize(ucm);

  int idx[2];

  for(int i = 0; i < height-1; i++){
    for(int j = 0; j < width-1; j++){
      int sv = (int)(segmentation.at<int>(i,j));
      bool isContour = false;
      for(int ii=i; ii <= i+1; ii++){
	for(int jj=j; jj <= j+1; jj++){
	  if(isContour)
	    break;
	  int dv = (int)(segmentation.at<int>(ii,jj));
	  if(sv != dv){
	    isContour = true;
	    uchar temp = 0;
	    for(int iii=i; iii <= i+1; iii++)
	      for(int jjj=j; jjj <= j+1; jjj++)
		temp = max(temp, ucms.at<uchar>(iii,jjj));
	    idx[0] = sv-1; idx[1] = dv-1;
	    contourLengths.ref<float>(idx)++;
	    contourWeights.ref<float>(idx) += (float)(temp);
	    idx[0] = dv-1; idx[1] = sv-1;
	    contourLengths.ref<float>(idx)++;
	    contourWeights.ref<float>(idx) += (float)(temp);
	  }
	}
      }
    }
  }

  for(SparseMat::iterator it = contourWeights.begin(); it != contourWeights.end(); it++)
    contourWeights.ref<float>(it.node()->idx) /= contourLengths.value<float>(it.node()->idx);

  contourLengths.copyTo(returnContourLengths);
  contourWeights.copyTo(returnContourWeights);

  return;
}

/*************************************************************************/
SparseMat applyVarAdjacency(const SparseMat& adjacencyWeights, const float var1, const float var2){
  SparseMat output(adjacencyWeights.dims(), adjacencyWeights.size(), CV_32F);
  for(SparseMat::const_iterator it = adjacencyWeights.begin(); it != adjacencyWeights.end(); it++)
    output.ref<float>(it.node()->idx) = exp(-pow(it.value<float>(), var2) / var1);
  
  return output;
}


/*************************************************************************/
Mat graphCut(const Mat& prior, const SparseMat& pairwise){
  int m = prior.rows;
  cout << "m: " << m << endl;
  int numOfClasses = prior.cols;
  if(pairwise.size()[0] != m || pairwise.size()[1] != m){
    cout << "graphCut: pairwise and prior sizes do not match" << endl;
    exit(1);
  }

  Mat labels(m, 1, CV_32S);

  int num_sites = m;
  int num_labels = numOfClasses;

  GCoptimizationGeneralGraph gc(num_sites, num_labels);
  float tvalue;
  int idx0, idx1;
  for(SparseMat::const_iterator it = pairwise.begin(); it != pairwise.end(); it++){
    idx0 = it.node()->idx[0];
    idx1 = it.node()->idx[1];
    if(idx0 > idx1)
      continue;
    tvalue = (it.value<float>() + pairwise.value<float>(idx1, idx0)) / 2.0;
    gc.setNeighbors(idx0, idx1, tvalue);
  }
  
  for(int i = 0; i < m; i++)
    for(int j = 0; j < numOfClasses; j++)
      gc.setDataCost(i, j, prior.at<float>(i,j));

  for(int i = 0; i < numOfClasses; i++)
    for(int j = 0; j < numOfClasses; j++)
      if(i != j)
	gc.setSmoothCost(i, j, 1);
      else
	gc.setSmoothCost(i, j, 0);

  gc.setLabelCost(1.0);

  gc.expansion();

  for(int i = 0; i < m; i++)
    labels.at<int>(i,0) = gc.whatLabel(i);
  
  cout << "GraphCut Total Energy: " << gc.compute_energy() << endl;
  
  return labels;
}



