#ifndef SEGMENTATION_STUFF_H
#define SEGMENTATION_STUFF_H

#include <opencv2/opencv.hpp>

#include <vector>

#include "global.h"
#include "ImageStuff.h"
#include "MakePatches.h"

#include "filter.h"
#include "pnmfile.h"
#include "segment-graph.h"

// dissimilarity measure between pixels
static inline float diff(image<float> *r, image<float> *g, image<float> *b, int x1, int y1, int x2, int y2);


/* Segmentation for test, splits the image into two parts vertically */
cv::Mat segmentationForTest12(int height, int width);
cv::Mat segmentationForTest22(int height, int width);
cv::Mat noisyImageForTestRedBlueRedBlue(int height, int width);

/* Felzenszwalb Segmentation */
cv::Mat felzenszwalb_segment(const cv::Mat& inputImage, float sigma, float c, int min_size, int *num_ccs);

/* receives the segmentation matrix, and returns the (y,x) location of points which are at boundaries */
std::vector< std::pair<int,int> > getContours(const cv::Mat & segmentation);
cv::Mat getContours_ucm(const cv::Mat & segmentation);
/* returns the points which are in rg1, and at the boundary of rg1 and rg2 */
std::vector< std::pair<int,int> > getContourBetweenTwoRegions(const cv::Mat & segmentation, int rg1, int rg2);



/* sorts the number assigned to each segment from 1-m, and returns the number of segments */
int sortSegmentNumbers(cv::Mat & segmentation);

/* given a segmentation, build the weighted adjacency matrix 
   answer(i,j) = the length of the contour between i,j
*/
cv::SparseMat getWeightedAdjacencyMatrix(const cv::Mat & segmentation);

/* return center of segments */
std::vector< std::pair<float,float> > getSegmentCenters(const cv::Mat & segmentation);

/* returns edge * from the sparse matrix */
edge * dadjacency2edges(const cv::SparseMat& adjacency, int & edgeLength /*this is output*/);

/* returns a segmentation matrix of integers from 0 to k, with pixels assigned 0 to k values, by getting the universe* as input */
cv::Mat universe2segmentation(universe* u, int height, int width, bool widthFirst=false);
cv::Mat parents2segmentation(int* parents, int height, int width);
std::vector<int> universe2segmentation(universe* u, int length);

/* given a sparse adjacency matrix of differences, makes it an adjacency matrix of similarities, exp(-d/sigma) */
cv::SparseMat dadjacency2sadjacency(const cv::SparseMat& adjacency, const double sigma);

/* returns an adjacency matrix between regions, the weight between two regions is the sum of weights between the connections betwen their elements  */
cv::Mat sadjacency2regionSadjacency(const cv::SparseMat& adjacency, const cv::Mat segmentation, bool symmetrixAnswer);
/* returns an adjacency matrix between regions, the weight between two regions is the mean of weights between the connections betwen their elements */
cv::Mat sadjacency2regionSadjacency_mean(const cv::SparseMat& adjacency, const cv::Mat segmentation, bool symmetrixAnswer);


/* pre-segment the image, by connecting each pixel to its nearest neighbor(s) */
std::vector<int> presegmentImage(cv::SparseMat& dadjacency, float thresh);

/* image to superpixels, using mean shift */
cv::Mat meanShiftSegmentation(const cv::Mat& image, double kernel_size, double max_dist);

/****************** BERKELEY **********************************/

/* read contours from mat file */
cv::Mat berkeley_segment(const cv::Mat& ucm, const int k);

cv::Mat berkeley_ucmResize(const cv::Mat& ucm);

void berkeley_weightedAdjacencyMatrix(const cv::Mat& ucm, const cv::Mat & segmentation, cv::SparseMat& returnContourLengths, cv::SparseMat& returnContourWeights);

cv::SparseMat applyVarAdjacency(const cv::SparseMat& adjacencyWeights, const float var1, const float var2);

/*******************GRAPHCUT*********************************/
cv::Mat graphCut(const cv::Mat& prior, const cv::SparseMat& pairwise);

#endif
