#ifndef SEQUENCE_STUFF_H
#define SEQUENCE_STUFF_H

#include "DatasetStuff.h"
#include "PriorStuff.h"
#include "PairwiseStuff.h"

#include <list>

#include <opencv2/opencv.hpp>

enum {BERKELEY_SEG, FELZ_SEG};

class Sequence{
 public:
  Sequence();
  /* seg_type contains the type of segmentation (BERKELEY_SEG, FELZ_SEG */
  Sequence(Dataset& db, const int seqNumber, const int startFrame, const int endFrame, const int seg_type);
  void push_back(const Prior& p);
  void push_front(const Prior& p);
  void push_back(Dataset& db, const int seqNumber, const int seg_type);
  Prior getPrior(int frameNumber);
  Pairwise getPairwise(int frameNumber1, int frameNumber2);
  void pop_back();
  void pop_front();
  int getLength() const;
  cv::Mat transferPriorFromFrameToFrame(const cv::Mat& prior1, const int frame1, const int frame2);
  cv::Mat xiaofengFeatureVector(const int frame, const cv::Mat& locationPattern);
  int getStartFrame() const{return startFrame_;}
  int getEndFrame() const{return endFrame_;}
  cv::SparseMat computeW(const int fromFrame, const int toFrame, const float ratio);
  
  std::map<int, Prior> getPriors(){return priors_;};

 private:
  std::map<int, Prior> priors_;
  std::map<std::pair<int,int>, Pairwise> pairwises_;

  int startFrame_;
  int endFrame_;

};


#endif //SEQUENCE_STUFF_H


