#include "TrainStuff.h"
#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "FeatureStuff.h"
#include "MyMATFileInterface.h"
#include <iostream>
#include <fstream>

using namespace std;
using namespace cv;

/******************************************************************************/
void LinearClassifier::save(const char * filename){
  Mat output = w.t();
  output.push_back(b);
  output = output.t();
  writeMatrixToFile(filename, output);
  return;
}

/******************************************************************************/
void LinearClassifier::load(const char * filename){
  Mat wb = readMatrixFromFile(filename);
  int n = wb.cols;
  w = wb.colRange(0, n-1);
  b = wb.at<float>(0, n-1);
  return;
}

/******************************************************************************/
void LinearClassifier::print(){
  cout << "w:" << endl;
  cout << w << endl;
  cout << "b:" << endl;
  cout << b << endl;
  return;
}

/******************************************************************************/
Mat getHOGClusters(string datasetName, const int trainVersion, const int clusterNum){  
  Dataset db = dataset_setup(datasetName);

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/getHOGClusters_%d_%d.mat", dataDir.c_str(), trainVersion, clusterNum);
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "centers");
    return centers;
  }
  ifile.close();


  int step = 500;
  int imSampleSize = 200;

  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);
  Mat allFeatures;
  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    DatasetSeq seq = db.getSequence(seqNumber);
    int seqLength = seq.getSeqLength();
    

    for(int frame = step; frame < seqLength; frame+=step){
      Mat im = db.getIM(seqNumber, frame);
      Mat fv = getHOG(im);
      vector<int> ind = randsample(fv.rows, imSampleSize);
      Mat fvind = indMatrixRows<float>(fv, ind);
      allFeatures.push_back(fvind);
      cout << "allFeatures: " << allFeatures.rows << " " << allFeatures.cols << endl;
    }
  }

  int m = allFeatures.rows;
  int n = allFeatures.cols;

  double myepsilon = 0.0000001;
  Mat idx = Mat::zeros(m, 1, CV_32S);
  Mat centers = Mat::zeros(m, n, CV_32F);
  TermCriteria t(1,100,myepsilon);
  kmeans(allFeatures, clusterNum, idx, t, 5, KMEANS_PP_CENTERS, centers);

  writeMatrixToMatFile(centers, "centers", buffer);

  return centers;
}


/******************************************************************************/
Mat getLabClusters(string datasetName, const int trainVersion, const int clusterNum){
  Dataset db = dataset_setup(datasetName);

  char buffer[300];
  buffer[0] = 0;
  string dataDir = db.getDataDir();
  sprintf(buffer, "%s/opencv/getLabClusters_%d_%d.mat", dataDir.c_str(), trainVersion, clusterNum);
  
  ifstream ifile(buffer);
  if (ifile) {
    ifile.close();
    Mat centers = readFloatMatrix(buffer, "centers");
    return centers;
  }
  ifile.close();

  int step = 500;
  int imSampleSize = 200;

  vector<int> trainSeqs = db.getTrainSeqs(trainVersion);
  Mat allFeatures;
  for(int i = 0; i < trainSeqs.size(); i++){
    int seqNumber = trainSeqs[i];
    DatasetSeq seq = db.getSequence(seqNumber);
    int seqLength = seq.getSeqLength();
    
    for(int frame = step; frame < seqLength; frame+=step){
      Mat im = db.getIM(seqNumber, frame);
      Mat fv = getColorLab(im);
      vector<int> ind = randsample(fv.rows, imSampleSize);
      Mat fvind = indMatrixRows<float>(fv, ind);
      allFeatures.push_back(fvind);
      cout << "allFeatures: " << allFeatures.rows << " " << allFeatures.cols << endl;
    }
  }

  int m = allFeatures.rows;
  int n = allFeatures.cols;

  double myepsilon = 0.00001;
  Mat idx = Mat::zeros(m, 1, CV_32S);
  Mat centers = Mat::zeros(m, n, CV_32F);
  TermCriteria t(1,100,myepsilon);
  cout << "kmeans...";
  cout.flush();
  kmeans(allFeatures, clusterNum, idx, t, 5, KMEANS_PP_CENTERS, centers);
  cout << "Done." << endl;

  writeMatrixToMatFile(centers, "centers", buffer);

  return centers;
  
}

/******************************************************************************/
LinearClassifier getWeightVectorLinearSVM(CvSVM * trainer){
  LinearClassifier output;
  int n = trainer->get_var_count();
  Mat temp = Mat::zeros(1, n, CV_32F);
  output.b = trainer->predict(temp, true);
  output.w = Mat::zeros(1, n, CV_32F);
  for(int i = 0; i < n; i++){
    temp = Mat::zeros(1, n, CV_32F);
    temp.at<float>(0,i) = 1;
    float w_ = trainer->predict(temp, true);
    w_ -= output.b;
    output.w.at<float>(0,i) = w_;
  }

  return output;
}

/******************************************************************************/
LinearClassifier svm_light_train(const Mat& data, const Mat& labels, const char * tempFile, const char * modelFile, const char * params){
  int m = data.rows;
  int n = data.cols;

  float np = 0;
  float nn = 0;
  for(int i = 0; i < m; i++)
    if(labels.at<float>(i,0) > 0)
      np++;
    else if(labels.at<float>(i,0) < 0)
      nn++;

  cout << "Number of positive examples: " << np << endl;
  cout << "Number of negative examples: " << nn << endl;

  char allparams[500];
  allparams[0] = 0;
  float j = 1;
  if(np > 1 && nn > 1)
    j = nn/np;
  sprintf(allparams, "%s -j %f", params, j);

  write_svm_light_input(data, labels, tempFile);
  char command[500];
  command[0] = 0;
  sprintf(command, "./svm_light/svm_learn %s %s %s", allparams, tempFile, modelFile);
  cout << command << endl;
  system(command);

  LinearClassifier output = svm_light_read_model(modelFile);  
  
  return output;
}

/******************************************************************************/
LinearClassifier svm_light_read_model(const char * modelFile){
  LinearClassifier output;

  int nsv, n;
  Mat supvec, alpha;
  float b;

  ifstream infile(modelFile, ios::in);
  bool found = false;
  while(!found){
    int limit = 1000000;
    char line[limit];
    infile.getline(line, limit);
    if(!infile)
      break;
    if(!found){
      string linestr(line);
      int pos = linestr.find(string("highest feature index"));
      if(pos >= 0)
	found = true;
    }
    if(!found)
      continue;

    char * tok_n = strtok(line, "#");
    n = atoi(tok_n);
    cout << "n: " << n << endl;

    infile.getline(line, limit);
    infile.getline(line, limit);
    char * tok_nsv = strtok(line, "#");
    nsv = atoi(tok_nsv);
    nsv -= 1;
    cout << "nsv: " << nsv << endl;

    supvec = Mat::zeros(nsv, n, CV_32F);
    alpha = Mat::zeros(nsv, 1, CV_32F);

    infile.getline(line, limit);
    char * tok_b = strtok(line, "#");
    b = atof(tok_b);
    cout << "b: " << b << endl;

    for(int i = 0; i < nsv; i++){
      infile.getline(line, limit);
      char * tok = strtok(line, " :");
      alpha.at<float>(i,0) = atof(tok);
      int ind;
      float s;
      while(tok != NULL){
	tok = strtok(NULL, " :");
	if(tok == NULL)
	  break;
	ind = atoi(tok);
	tok = strtok(NULL, " :");
	if(tok == NULL)
	  break;
	s = atof(tok);
	supvec.at<float>(i, ind-1) = s;
      }
      //cout << supvec.rowRange(i,i+1) << endl;
      //cout << alpha.rowRange(0,1) << endl;
    }
  }

  infile.close();

  cout << supvec.rows << " " << supvec.cols << endl;

  Mat alpha_repmat = repmat(alpha, 1, n);
  Mat temp = Mat::zeros(nsv, n, CV_32F);
  multiply(supvec, alpha_repmat, temp);

  output.b = -b;
  Mat w = Mat::zeros(1, n, CV_32F);
  for(int i = 0; i < nsv; i++)
    for(int j = 0; j < n; j++)
      w.at<float>(0,j) += temp.at<float>(i,j);

  output.w = w;

  return output;
}


/******************************************************************************/
Mat classificationToConfusionMatrix(const Mat& classifications, const Mat& labels){
  int maxY, maxX;
  int maxLabel = matrixMaxValue<int>(labels, maxY, maxX);
  cout << "maxLabel: " << maxLabel << endl;
  Mat confusion = Mat::zeros(maxLabel+1, maxLabel+1, CV_32F);
  int n = classifications.rows;
  for(int i = 0; i < n; i++){
    int lgt = labels.at<int>(i,0);
    int lc, dummy;
    matrixMaxValue<float>(classifications.rowRange(i,i+1), dummy, lc);
    confusion.at<float>(lgt,lc) += 1.0;
  }

  //first make the sum of each row equal to 1.
  for(int i = 0; i < maxLabel+1; i++){
    float totalSum = 0;
    for(int j = 0; j < maxLabel+1; j++)
      totalSum += confusion.at<float>(i,j);
    totalSum = totalSum + 0.000001;
    //cout << i << ": " << totalSum << endl;
    for(int j = 0; j < maxLabel+1; j++)
      confusion.at<float>(i,j) = confusion.at<float>(i,j)/totalSum;
  }


  return confusion;
}
