#include "Visualizations.h"
#include "MLStuff.h"

#include "ImageStuff.h"
#include "MyMATFileInterface.h"
#include "MakePatches.h"

#include <stdarg.h>

using namespace std;
using namespace cv;

/**********************************************************************************/
void cvShowManyImages(char* title, int nArgs, ...) {

  // img - Used for getting the arguments 
  IplImage *img;

  // DispImage - the image in which input images are to be copied
  IplImage *DispImage;

  int size;
  int i;
  int m, n;
  int x, y;

  // w - Maximum number of images in a row 
  // h - Maximum number of images in a column 
  int w, h;

  // scale - How much we have to resize the image
  float scale;
  int max;

  // If the number of arguments is lesser than 0 or greater than 12
  // return without displaying 
  if(nArgs <= 0) {
    printf("Number of arguments too small....\n");
    return;
  }
  else if(nArgs > 12) {
    printf("Number of arguments too large....\n");
    return;
  }
  // Determine the size of the image, 
  // and the number of rows/cols 
  // from number of arguments 
  else if (nArgs == 1) {
    w = h = 1;
    size = 300;
  }
  else if (nArgs == 2) {
    w = 2; h = 1;
    size = 300;
  }
  else if (nArgs == 3 || nArgs == 4) {
    w = 2; h = 2;
    size = 300;
  }
  else if (nArgs == 5 || nArgs == 6) {
    w = 3; h = 2;
    size = 200;
  }
  else if (nArgs == 7 || nArgs == 8) {
    w = 4; h = 2;
    size = 200;
  }
  else {
    w = 4; h = 3;
    size = 150;
  }

  // Create a new 3 channel image
  DispImage = cvCreateImage( cvSize(100 + size*w, 60 + size*h), 8, 3 );

  // Used to get the arguments passed
  va_list args;
  va_start(args, nArgs);

  // Loop for nArgs number of arguments
  for (i = 0, m = 20, n = 20; i < nArgs; i++, m += (20 + size)) {

    // Get the Pointer to the IplImage
    img = va_arg(args, IplImage*);

    // Check whether it is NULL or not
    // If it is NULL, release the image, and return
    if(img == 0) {
      printf("Invalid arguments");
      cvReleaseImage(&DispImage);
      return;
    }

    // Find the width and height of the image
    x = img->width;
    y = img->height;

    // Find whether height or width is greater in order to resize the image
    max = (x > y)? x: y;

    // Find the scaling factor to resize the image
    scale = (float) ( (float) max / size );

    // Used to Align the images
    if( i % w == 0 && m!= 20) {
      m = 20;
      n+= 20 + size;
    }

    // Set the image ROI to display the current image
    cvSetImageROI(DispImage, cvRect(m, n, (int)( x/scale ), (int)( y/scale )));

    // Resize the input image and copy the it to the Single Big Image
    cvResize(img, DispImage);

    // Reset the ROI in order to display the next image
    cvResetImageROI(DispImage);
  }

  // Create a new window, and show the Single Big Image
  cvNamedWindow( title, 1 );
  cvShowImage( title, DispImage);
}

/**************************************************************************/
void showSegmentation(char * imname, char * segname){

  Mat segmentation = readBSDSegmentation(segname);
  Mat segim = myImagesc(segmentation);

  Mat im = imread(imname);
  
  IplImage segimg = segim;
  IplImage img = im;

  cvNamedWindow("Segmentation", 0);
  //cvNamedWindow("Image", 0);

  cvShowManyImages("Segmentation", 2, &img, &segimg);

  //cvShowImage("Image", &img);
  //cvShowImage("Segmentation", &segimg);
}

/**************************************************************************/
void showConfusionMatrix(Mat matrix, int blockSize, vector<string> labels){
  Size s = matrix.size();
  int height = s.height;
  int width = s.width;

  //first make the sum of each row equal to 1.
  for(int i = 0; i < height; i++){
    float totalSum = 0;
    for(int j = 0; j < width; j++)
      totalSum += matrix.at<float>(i,j);
    totalSum = totalSum + 0.0001;
    //cout << i << ": " << totalSum << endl;
    for(int j = 0; j < width; j++)
      matrix.at<float>(i,j) = matrix.at<float>(i,j)/totalSum;
  }

  //Now it is time to visualize Matrix
  Mat newMatrix(height*blockSize, width*blockSize, CV_8UC1);
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      double temp = matrix.at<float>(i,j)*255.0;
      for(int ii = i*blockSize; ii < (i+1)*blockSize; ii++){
	for(int jj = j*blockSize; jj < (j+1)*blockSize; jj++){
	  newMatrix.at<char>(ii,jj) = (char)(round(temp));
	}
      }
    }
  }

  int margin = 3;
  Mat finalMatrix = Mat::ones(height*blockSize + blockSize*margin, width*blockSize + blockSize*margin, CV_8UC1) * 255;
  int temp = margin * blockSize;
  for(int i = 0; i < height*blockSize; i++)
    for(int j = 0; j < width*blockSize; j++)
      finalMatrix.at<char>(i+temp,j+temp) = newMatrix.at<char>(i,j);

  for(int i = 0; i < height; i++){
    Point pt;
    pt.y = temp + i * blockSize + blockSize*3/4;
    pt.x = blockSize;

    char buffer[100];
    buffer[0] = 0;
    sprintf(buffer, "%d", i);
    string text(buffer);

    putText(finalMatrix, text, pt, FONT_HERSHEY_SIMPLEX, 0.3, cvScalar(0,0,0));

    pt.x = temp + i * blockSize + blockSize/4;
    pt.y = blockSize;
    putText(finalMatrix, text, pt, FONT_HERSHEY_SIMPLEX, 0.3, cvScalar(0,0,0));

  }

  //show Confusion matrix
  cvNamedWindow("Confusion", 0);

  IplImage img = finalMatrix;
  cvShowImage("Confusion", &img);
  
}
/**************************************************************************/
void showSegmentationConfusionMatrix(char * segname, cv::Mat matrix){
  
  Size wh = matrix.size();
  int height = wh.height;
  int width = wh.width;

  for(int i = 0; i < height; i++){
    double totalSum = 0;
    for(int j = 0; j < width; j++){
      totalSum += matrix.at<double>(i,j);
    }
    totalSum = totalSum + 0.01;
    for(int j = 0; j < width; j++){
      double temp = matrix.at<double>(i,j);
      temp = temp/totalSum;
      matrix.at<double>(i,j) = temp;
    }
  }

  //showConfusionMatrix(matrix, 20);

  uchar * confusions = new uchar[height];
  for(int i = 0; i < height; i++)
    confusions[i] = (uchar)(matrix.at<double>(i,i) * 255);

  Mat segmentation = readBSDSegmentation(segname);

  wh = segmentation.size();
  height = wh.height;
  width = wh.width;
  
  Mat answer(height, width, CV_8UC1);
  uchar * answer_ptr = answer.ptr();
  uchar * segment_ptr = segmentation.ptr();

  for(int count = 0; count < height*width; count++){
    uchar sind = segment_ptr[count];
    uchar temp = confusions[sind-1]; //because segmentation indices start from 1
    answer_ptr[count] = temp;
  }

  delete [] confusions;

  IplImage ansimg = answer;

  cvNamedWindow("Region Confusion", 0);

  cvShowImage("Region Confusion", &ansimg);
}

/**************************************************************************/
void showContours(vector< pair<int,int> > contours, int height, int width){
  Mat contourMatrix(height, width, CV_8UC1);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      contourMatrix.at<uchar>(i,j) = 0;
  int count = 0;
  for(vector< pair<int,int> >::iterator it = contours.begin(); it != contours.end(); it++)
    contourMatrix.at<uchar>(it->first, it->second) = 255;

  //for(int i = 0; i < (int)(contours.size()); i++)
  //contourMatrix.at<uchar>(contours[i].first, contours[i].second) = 255;

  IplImage ansimg = contourMatrix;

  cvNamedWindow("Contours", 0);

  cvShowImage("Contours", &ansimg);
}

/**************************************************************************/
void showContoursOnImage(const Mat & image, vector< pair<int,int> > contours, char * name){
  typedef Vec<uchar,3> Vec3u;

  int height = image.size().height;
  int width = image.size().width;

  Mat contourMatrix = image;

  int count = 0;
  for(vector< pair<int,int> >::iterator it = contours.begin(); it != contours.end(); it++){
    contourMatrix.at<Vec3u>(it->first, it->second)[0] = 255;
    contourMatrix.at<Vec3u>(it->first, it->second)[1] = 0;    
    contourMatrix.at<Vec3u>(it->first, it->second)[2] = 0;
  }

  IplImage ansimg = contourMatrix;

  cvNamedWindow(name, 0);

  cvShowImage(name, &ansimg);
}

/**************************************************************************/
void showFzsbSegmentation(const cv::Mat & image, universe * u, char * name){
  Size wh = image.size();
  int height = wh.height;
  int width = wh.width;
  Mat answer = getRandomImage(height, width);

  typedef Vec<uchar,3> Vec3u;

  long int ind1, ind2;
  int Ycor, Xcor;
  for(int Y = 0; Y < height; Y++){
    for(int X = 0; X < width; X++){
      YXLocationToIndex(ind1, Y, X, height, width, 0);
      ind2 = u->find(ind1);
      indexToYXLocation(Ycor, Xcor, ind2, height, width, 0);
      answer.at<Vec3u>(Y, X)[0] = answer.at<Vec3u>(Ycor, Xcor)[0];
      answer.at<Vec3u>(Y, X)[1] = answer.at<Vec3u>(Ycor, Xcor)[1];
      answer.at<Vec3u>(Y, X)[2] = answer.at<Vec3u>(Ycor, Xcor)[2];
    }
  }

  /*  for(int Y = 0; Y < height; Y+=15){
    for(int X = 0; X < width; X+=15){
      cout << (int)(answer.at<Vec3u>(Y, X)[0]) << " " << (int)(answer.at<Vec3u>(Y, X)[1]) << " " << (int)(answer.at<Vec3u>(Y, X)[2]) << endl;
    }
  }

  int Y = 50;
  int X = 50;
  cout << (int)(answer.at<Vec3u>(Y, X)[0]) << " " << (int)(answer.at<Vec3u>(Y, X)[1]) << " " << (int)(answer.at<Vec3u>(Y, X)[2]) << endl;
  Y = 51;
  X = 50;
  cout << (int)(answer.at<Vec3u>(Y, X)[0]) << " " << (int)(answer.at<Vec3u>(Y, X)[1]) << " " << (int)(answer.at<Vec3u>(Y, X)[2]) << endl;
  */
  IplImage ansimg = answer;
  IplImage img = image;

  cvNamedWindow(name, 0);
  cvNamedWindow("Image", 0);

  cvShowImage("Image", &img);
  cvShowImage(name, &ansimg);

  return;
}
/**************************************************************************/
void showNonzeroElementsMatrix(const cv::Mat & matrix){
  int ymax, xmax;
  float maxMatrixValue = matrixMaxValue<float>(matrix, ymax, xmax);
  cout << "Max matrix value: " << maxMatrixValue << endl;
  typedef Vec<uchar,3> Vec3u;
  Size wh = matrix.size();
  int height = wh.height;
  int width = wh.width;
  Mat answer(height, width, CV_8UC3);
  for(int i = 0; i < height; i++){
    float maxRowValue = -1;
    for(int j = 0; j < width; j++){
      float temp = matrix.at<float>(i,j);
      if(temp > maxRowValue)
	maxRowValue = temp;
    }
    //cout << maxRowValue << " | ";
    for(int j = 0; j < width; j++){
      //cout << matrix.at<float>(i,j) << " ";
      if(height < 20){
	float temp = matrix.at<float>(i,j) / maxRowValue;
	temp = temp*255;
	uchar uctemp = (uchar)(round(temp));
	answer.at<Vec3u>(i,j)[0] = uctemp;
	answer.at<Vec3u>(i,j)[1] = uctemp;
	answer.at<Vec3u>(i,j)[2] = uctemp;
      }
      else{
	if(matrix.at<float>(i,j) > 0){
	  answer.at<Vec3u>(i,j)[0] = 255;
	  answer.at<Vec3u>(i,j)[1] = 255;
	  answer.at<Vec3u>(i,j)[2] = 255;
	}
	else{
	  answer.at<Vec3u>(i,j)[0] = 0;
	  answer.at<Vec3u>(i,j)[1] = 0;
	  answer.at<Vec3u>(i,j)[2] = 0;	
	}
      }
    }
    //cout << endl;
  }

  IplImage ansimg = answer;

  cvNamedWindow("Nonzero Elements", 0);

  cvShowImage("Nonzero Elements", &ansimg);

  return;
}

/**************************************************************************/
void showSegmentationRandom(const cv::Mat& segmentation, char * name){
  typedef Vec<uchar,3> Vec3u;
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;
  Mat im(height, width, CV_8UC3);
  int maxy,maxx;
  int maxValue = matrixMaxValue<int>(segmentation, maxy, maxx);
  map<int, int> Rmap, Gmap, Bmap;
  //srand (time(NULL));
  srand(0);
  for(int i = 1; i <= maxValue; i++){
    Rmap[i] = (uchar)(rand()*255);
    Gmap[i] = (uchar)(rand()*255);
    Bmap[i] = (uchar)(rand()*255);
  }
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      im.at<Vec3u>(i,j)[0] = Bmap[segmentation.at<int>(i,j)];
      im.at<Vec3u>(i,j)[1] = Gmap[segmentation.at<int>(i,j)];
      im.at<Vec3u>(i,j)[2] = Rmap[segmentation.at<int>(i,j)];
    }
  }

  imshow(name, im);

  /*  IplImage img = im;
  cvNamedWindow(name, 0);
  cvShowImage(name, &img);*/
  
  return;

}

/**************************************************************************/
void showParticularRegion(const cv::Mat& segmentation, int regionNumber){
  Size wh = segmentation.size();
  int height = wh.height;
  int width = wh.width;
  Mat im(height, width, CV_8UC3); 
  typedef Vec<uchar,3> Vec3u; 
  cout << "List of pixels in the region:" << endl;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      if(segmentation.at<int>(i,j) == regionNumber){
	cout << i << " " << j << endl;
	im.at<Vec3u>(i,j)[0] = 255;
	im.at<Vec3u>(i,j)[1] = 255;
	im.at<Vec3u>(i,j)[2] = 255;	
      }
      else{
	im.at<Vec3u>(i,j)[0] = 0;
	im.at<Vec3u>(i,j)[1] = 0;
	im.at<Vec3u>(i,j)[2] = 0;		
      }
    }
  }
  
  char name[200];
  name[0] = 0;
  sprintf(name, "region %d", regionNumber);

  showSegmentationRandom(segmentation, "Segmentation");

  imshow(name, im);
  /*IplImage img = im;
  cvNamedWindow(name, 0);
  cvShowImage(name, &img);*/
  
  return;
}

/**************************************************************************/
void showAdjacencyMatrix(const Mat & image, const Mat & segmentation, const SparseMat & adjacencyMatrix){
  const int * sizes = adjacencyMatrix.size();
  if(sizes[0] != sizes[1]){
    cout << "showAdjacencyMatrix: Invalid Adjacency Matrix" << endl;
    return;
  }

  int m = sizes[0];
  vector< pair<float,float> > segCenters = getSegmentCenters(segmentation);

  vector< pair<int,int> > contours = getContours(segmentation);

  // Visualizing contours on image
  typedef Vec<uchar,3> Vec3u;

  int height = image.size().height;
  int width = image.size().width;
  Mat contourMatrix = image;
  int count = 0;
  for(vector< pair<int,int> >::iterator it = contours.begin(); it != contours.end(); it++){
    contourMatrix.at<Vec3u>(it->first, it->second)[0] = 255;
    contourMatrix.at<Vec3u>(it->first, it->second)[1] = 0;    
    contourMatrix.at<Vec3u>(it->first, it->second)[2] = 0;
  }

  // Drawing adjacency lines on image
  for(SparseMat::const_iterator it = adjacencyMatrix.begin(); it != adjacencyMatrix.end(); it++){
    int i1 = it.node()->idx[0];
    int i2 = it.node()->idx[1];
    line(contourMatrix, Point(segCenters[i1].second, segCenters[i1].first), Point(segCenters[i2].second, segCenters[i2].first), CV_RGB(255, 0, 0));
  }

  char name[200];
  name[0] = 0;
  sprintf(name, "Adjacency Visualization");

  imshow(name, contourMatrix);
  /*IplImage contourImage = contourMatrix;
  cvNamedWindow(name, 0);
  cvShowImage(name, &contourImage);*/

  return;
}

/**************************************************************************/
void showPrior(const Mat& prior, const Mat& segmentation, char * name){
  int h = segmentation.rows;
  int w = segmentation.cols;
  Mat output(h, w, CV_8UC1);

  float temp;
  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      temp = prior.at<float>(segmentation.at<int>(i,j)-1,0);
      if(temp < 0)
	temp = 0;
      if(temp > 1)
	temp = 1;
      output.at<uchar>(i,j) = (uchar)(255 * temp);
    }
  }

  imshow(name, output);

  return;
}

/**************************************************************************/
void showFloatMat(const cv::Mat& matrix, char * name){
  Mat output(matrix.rows, matrix.cols, CV_8UC1);
  float temp;
  for(int i = 0; i < matrix.rows; i++){
    for(int j = 0; j < matrix.cols; j++){
      temp = matrix.at<float>(i,j);
      if(temp < 0)
	temp = 0;
      if(temp > 1)
	temp = 1;
      output.at<uchar>(i,j) = (uchar)(temp*255);
    }
  }
  imshow(name, output);
  return;
}

/**************************************************************************/
Mat getSpatialOnContours(const Mat& image, const Mat& segmentation, const SparseMat& spatialMatrix){
  // The assumption is that spatialMatrix values are between 0 and 1. 1 means very similar, 0 means very different.

  int height = image.rows;
  int width = image.cols;
  Mat contourMatrix(height, width, CV_8UC1);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      contourMatrix.at<uchar>(i,j) = 0;

  vector< pair<int,int> > contours = getContours(segmentation);

  clock_t startC, endC;
  
  startC = clock();
  int num_ccs; 
  
  int idx[2];
  for(vector< pair<int,int> >::iterator it = contours.begin(); it != contours.end(); it++){
    int y = it->first;
    int x = it->second;
    int sv = segmentation.at<int>(y,x);
    int dv = sv;
    for(int yy = y-1; yy <= y+1; yy++){
      for(int xx = x-1; xx <= x+1; xx++){
	dv = segmentation.at<int>(yy,xx);
	if(sv != dv)
	  break;
      }
    }
    idx[0] = sv-1; idx[1] = dv-1;
    float value = spatialMatrix.value<float>(idx);
    value = 255 * value;
    contourMatrix.at<uchar>(y,x) = (uchar)(value);
  }

  endC = clock();

  //cout << "Time required for getContours2: " << (double)(endC-startC)/CLOCKS_PER_SEC << " seconds." << endl;
  
  //cout << "Total Adjacency: " << totalAdj << endl;
  
  return contourMatrix;
}

/**************************************************************************/
void showSpatialOnContours(const Mat& image, const Mat& segmentation, const SparseMat& spatialMatrix){
  Mat contourMatrix = getSpatialOnContours(image, segmentation, spatialMatrix);
  imshow("ContourMatrix", contourMatrix);
  return;
}

/**************************************************************************/
cv::Mat labelsOnContourImage(const cv::Mat & contourImage, const cv::Mat & segmentation, const cv::Mat & fLabels){
  int height = contourImage.rows;
  int width = contourImage.cols;
  Mat output(height, width, CV_8UC3);
  typedef Vec<uchar,3> Vec3u;
  for(int i = 0; i < height; i++){
    for(int j = 0; j < width; j++){
      int l = segmentation.at<int>(i,j);
      float f = fLabels.at<float>(l-1,0);
      uchar addval = 0;
      if(f > 0.99){
	output.at<Vec3u>(i,j)[0] = 0;
	output.at<Vec3u>(i,j)[1] = 0;
	output.at<Vec3u>(i,j)[2] = 255;
      }
      if(f > 0.5 && f <= 0.99){
	output.at<Vec3u>(i,j)[0] = max(contourImage.at<Vec3u>(i,j)[0] - 100, 0);
	output.at<Vec3u>(i,j)[1] = max(contourImage.at<Vec3u>(i,j)[1] - 100, 0);
	output.at<Vec3u>(i,j)[2] = min(contourImage.at<Vec3u>(i,j)[2] + 150, 255);
      }
      if(f <= 0.5)
	output.at<Vec3u>(i,j) = contourImage.at<Vec3u>(i,j);
    }
  }
  return output;
}

/**************************************************************************/
Mat labelsToMask(const Mat & segmentation, const Mat & fLabels){
  int height = segmentation.rows;
  int width = segmentation.cols;
  Mat output(height, width, CV_32S);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      if(fLabels.at<float>(segmentation.at<int>(i,j)-1, 0) > 0.5)
        output.at<int>(i,j) = 1;
      else
	output.at<int>(i,j) = 0;

  return output;    
}

/**************************************************************************/
void interactiveImageSegmentation(int event, int X, int Y, int flags, void* param){

  ImIntSeg * input = (ImIntSeg *)(param);

  imshow("FathiBMVC11", input->labelsOnContourImage);

  switch( event ){
  case CV_EVENT_MOUSEMOVE: 
    break;
    
  case CV_EVENT_LBUTTONDOWN:
    //cout << "LB: The pixel location is: " << Y << " " << X << endl;
    input->Labels.at<int>(input->segmentation.at<int>(floor(Y), floor(X))-1,0) = 1;
    //cout << input->segmentation.at<int>(floor(Y), floor(X)) << endl;
    //cout << input->Labels << endl;
    input->fLabels = harmonicSemiSupervised(input->W, input->Labels, input->numOfClasses);
    //cout << input->fLabels << endl;
    input->labelsOnContourImage = labelsOnContourImage(input->contourImage, input->segmentation, input->fLabels);
   
    break;

  case CV_EVENT_RBUTTONDOWN:
    //cout << "RB: The pixel location is: " << Y << " " << X << endl;
    input->Labels.at<int>(input->segmentation.at<int>(floor(Y), floor(X))-1,0) = 2;
    //cout << input->segmentation.at<int>(floor(Y), floor(X)) << endl;
    input->fLabels = harmonicSemiSupervised(input->W, input->Labels, input->numOfClasses);
    input->labelsOnContourImage = labelsOnContourImage(input->contourImage, input->segmentation, input->fLabels);

    break;

  case CV_EVENT_LBUTTONUP:
    break;
  }

  return;
}

/**************************************************************************/
void showLabels(const Mat& labels, const Mat& segmentation, char * name){
  int h = segmentation.rows;
  int w = segmentation.cols;

  Mat Labels(h, w, CV_32S);
  int s;
  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      s = segmentation.at<int>(i,j);
      Labels.at<int>(i,j) = labels.at<int>(s-1,0) + 1;
    }
  }

  showSegmentationRandom(Labels, name);
  
  return;
}

/**************************************************************************/
void showObjectLabels(const cv::Mat& labels, Prior& p, Dataset& db, char * name){
  /* 
     The assumption is that, labels is a mx1 matrix, where there are m segments
     objNames has length m
     objColors has size mx3 and is uchar
     bgColor has size 1x3 and is uchar
   */

  vector<string> objNames = db.getObjectNames();
  Mat objColors = db.getObjectColors(objNames);
  Scalar bgColor = db.getObjectColor(string(""));

  Mat segmentation = p.getSegmentation();
  Mat centers = p.getCenters();
  Mat sizes = p.getPrior_size();

  int h = segmentation.rows;
  int w = segmentation.cols;
  Mat Labels(h, w, CV_32S);

  int m = labels.rows;
  int n = objNames.size();

  int s;
  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      s = segmentation.at<int>(i,j);
      Labels.at<int>(i,j) = labels.at<int>(s-1,0);
    }
  }

  typedef Vec<uchar,3> Vec3u;
  int l;
  uchar b,g,r;
  Mat output(h, w, CV_8UC3);
  for(int i = 0; i < h; i++){
    for(int j = 0; j < w; j++){
      l = Labels.at<int>(i,j);
      if(l == n){
	b = bgColor(0);
	g = bgColor(1);
	r = bgColor(2);
      }
      else{
	b = objColors.at<uchar>(l,0);
	g = objColors.at<uchar>(l,1);
	r = objColors.at<uchar>(l,2);
      }	
      output.at<Vec3u>(i,j)[0] = b;
      output.at<Vec3u>(i,j)[1] = g;
      output.at<Vec3u>(i,j)[2] = r;
    }
  }

  float sizeThresh = 500;
  for(int i = 0; i < m; i++){
    if(sizes.at<float>(i,0) < sizeThresh)
      continue;
    string text;
    if(labels.at<int>(i,0) == n)
      text = "unknown";
    else
      text = objNames[labels.at<int>(i,0)];
    Point pt;
    pt.y = centers.at<float>(i,0);
    pt.x = centers.at<float>(i,1)-20;
    putText(output, text, pt, FONT_HERSHEY_SIMPLEX, 0.3, cvScalar(255,0,0));
  }

  imshow(name, output);
  
  return;
}

/**************************************************************************/
void showMaskOnImage(const Mat& mask, const Mat& image, const char * name){
  Mat newImage = maskOnImage(mask, image);
  imshow(name, newImage);
  return;
}


/**************************************************************************/
void showMaskOnImage2(const Mat& mask, const Mat& image, const char * name){
  Mat newImage = maskOnImage2(mask, image);
  imshow(name, newImage);
  return;
}

