#ifndef VISUALIZATIONS_H
#define VISUALIZATIONS_H

#include <vector>
#include <string>
#include "global.h"
#include "segment-graph.h"
#include "SegmentationStuff.h"
#include "PriorStuff.h"
#include "DatasetStuff.h"

void cvShowManyImages(char* title, int nArgs, ...);
void showSegmentation(char * imname, char * segname);
void showConfusionMatrix(cv::Mat matrix, int blockSize, std::vector<std::string> labels); //block size determines the height and width of each cell in pixels
void showSegmentationConfusionMatrix(char * segname, cv::Mat matrix);
void showContours(std::vector<std::pair<int,int> > contours, int height, int width);
void showContoursOnImage(const cv::Mat & image, std::vector<std::pair<int,int> > contours, char * name);
void showFzsbSegmentation(const cv::Mat & image, universe * u, char * name);
void showNonzeroElementsMatrix(const cv::Mat & matrix);
void showSegmentationRandom(const cv::Mat& segmentation, char * name);
void showParticularRegion(const cv::Mat& segmentation, int regionNumber);
void showAdjacencyMatrix(const cv::Mat& image, const cv::Mat& segmentation, const cv::SparseMat& adjacencyMatrix);
cv::Mat getSpatialOnContours(const cv::Mat& image, const cv::Mat& segmentation, const cv::SparseMat& spatialMatrix);
void showSpatialOnContours(const cv::Mat& image, const cv::Mat& segmentation, const cv::SparseMat& spatialMatrix);
void showPrior(const cv::Mat& prior, const cv::Mat& segmentation, char * name);
void showFloatMat(const cv::Mat& matrix, char * name);
void showLabels(const cv::Mat& labels, const cv::Mat& segmentation, char * name);
void showObjectLabels(const cv::Mat& labels, Prior& p, Dataset& db, char * name);
void showMaskOnImage(const cv::Mat& mask, const cv::Mat& image, const char * name);
void showMaskOnImage2(const cv::Mat& mask, const cv::Mat& image, const char * name);

struct ImIntSeg{
  cv::Mat image;
  cv::Mat contourImage;
  cv::Mat labelsOnContourImage;
  cv::Mat segmentation;
  cv::Mat W;
  cv::Mat Labels;
  cv::Mat fLabels;
  int numOfClasses; //For now we assume it is always 2
};

cv::Mat labelsOnContourImage(const cv::Mat & contourImage, const cv::Mat & segmentation, const cv::Mat & fLabels);
cv::Mat labelsToMask(const cv::Mat & segmentation, const cv::Mat & fLabels);
void interactiveImageSegmentation(int event, int X, int Y, int flags, void* param);

#endif 

