#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "PriorStuff.h"
#include "DatasetStuff.h"
#include "InputOutputFile.h"
#include "MyMATFileInterface.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 7) {
    cout << "usage: " << argv[0] << " <dbname> <object name> <seq number> <start frame> <steps> <k>"  << endl;
    return 1;
  }

  string dbname(argv[1]);
  string objectName(argv[2]);
  int seqNumber = atoi(argv[3]);
  int startFrame = atoi(argv[4]);
  int steps = atoi(argv[5]);
  int k = atoi(argv[6]);

  Dataset db = dataset_setup(dbname);
  DatasetSeq seq = db.getSequence(seqNumber);
  int endFrame = seq.getEndFrame();
  string dataDir = db.getDataDir();
  string seqName = db.getSequence(seqNumber).getSeqName();
  string imType = db.getImType();

  cout << "DataDir: " << dataDir.c_str() << endl;
  cout << "SeqName: " << seqName.c_str() << endl;
  cout << "ImType: " << imType.c_str() << endl;

  char buffer[200];
  buffer[0] = 0;
  sprintf(buffer, "mkdir %s/opencv/groundtruth/%s", dataDir.c_str(), seqName.c_str());
  system(buffer);
  buffer[0] = 0;
  sprintf(buffer, "mkdir %s/opencv/groundtruth/%s/%s", dataDir.c_str(), seqName.c_str(), objectName.c_str());
  system(buffer);


  ImIntSeg * myImIntSeg = new ImIntSeg();

  for(int f=startFrame; f <= endFrame; f += steps){
    Mat im = db.getIM(seqNumber, f);
    Mat ucm = db.readUCM(seqNumber, f);
    Prior p(im, ucm, k, f);
    myImIntSeg->image = im;
    myImIntSeg->segmentation = p.getSegmentation();
    SparseMat Wsp = p.getW();
    Wsp.copyTo(myImIntSeg->W);
    int m = p.getM();
    int numOfClasses = 2;
    Mat Labels = Mat::zeros(m, 1, CV_32S);
    Mat fLabels = Mat::zeros(m, numOfClasses, CV_32F);
    myImIntSeg->Labels = Labels;
    myImIntSeg->fLabels = fLabels;
    myImIntSeg->numOfClasses = numOfClasses;

    Mat contourImage_ = getSpatialOnContours(im, p.getSegmentation(), p.getAdjacencyWeights());
    int height = im.rows;
    int width = im.cols;
    Mat contourImage(height, width, CV_8UC3);
    cvtColor(contourImage_, contourImage, CV_GRAY2BGR);
    contourImage += im;
    myImIntSeg->contourImage = contourImage;
    myImIntSeg->labelsOnContourImage = contourImage;

    imshow("FathiBMVC11", contourImage);
    
    cvSetMouseCallback("FathiBMVC11", interactiveImageSegmentation, (void *) myImIntSeg);

    cout << "Frame: " << f << endl;
    
    cvWaitKey(0);
    
    Mat mask = labelsToMask(myImIntSeg->segmentation, myImIntSeg->fLabels);
    db.writeGTObjectMask(objectName, seqNumber, f, mask);
  }

  delete myImIntSeg;

  cvWaitKey(0);

  return 0;
}

