#include "MotionStuff.h"
#include "Visualizations.h"

// Namespaces
using namespace std;
using namespace cv;
using namespace cv::gpu;

//******************************************************************************

/** Main Function */
int main( int argc, char *argv[] )
{
    // Check input arguments
    if( argc != 5){
      cout << "usage: " << argv[0] << "seqNumber startFrame endFrame resizeFactor" << endl;
      return -1;
    }

    struct timeval tod1;
    double t1 = 0.0, t2 = 0.0, tdflow = 0.0, tvis = 0.0;

    int seqNumber = atoi(argv[1]);
    int startFrame = atoi(argv[2]);
    int endFrame = atoi(argv[3]);
    double resizeFactor = atof(argv[4]);

    Dataset db = dataset_setup(string("GTDA_v3"));

    // Create OpenCV windows
    cvNamedWindow("Image");
    cvNamedWindow("Dense Flow");

    for(int frame = startFrame; frame <= endFrame; frame++){
      gettimeofday(&tod1,NULL);
      t1 = tod1.tv_sec + tod1.tv_usec / 1000000.0;
      
      Mat image1 = db.getIM(seqNumber, frame);
      Mat image2 = db.getIM(seqNumber, frame+1);
      int height = image1.rows;
      int width = image1.cols;
      pair<Mat,Mat> flow = getDenseFlow_eccv04(image1, image2, resizeFactor); 
      
      cout << "Done!" << endl;
      
      gettimeofday(&tod1,NULL);
      t2 = tod1.tv_sec + tod1.tv_usec / 1000000.0;
      tdflow = (t2-t1);
      
      cout << "Time Dense Flow: " << tdflow << endl;
      
      // Visualization
      Mat flow_rgb = Mat(Size(width,height),CV_8UC3);
      drawColorField(flow.first, flow.second, flow_rgb);
      
      imshow("Dense Flow",flow_rgb);
      imshow("Image", image1);
      cvWaitKey(0);
    }

    cvWaitKey(0);

    return 0;
}

