/*
Copyright (C) 2006 Pedro Felzenszwalb

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/

#include <cstdio>
#include <cstdlib>
#include "image.h"
#include "misc.h"
#include "pnmfile.h"
#include <opencv2/opencv.hpp>
#include "segment-image.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"

using namespace std;
using namespace cv;

int main(int argc, char **argv) {

  if (argc != 6) {
    fprintf(stderr, "usage: %s resizeScale sigma k min input(image)\n", argv[0]);
    return 1;
  }

  string imname(argv[5]);
  Mat inputImage_original = imread(imname);
  double resizeScale = atof(argv[1]);
  Size who = inputImage_original.size();
  int widtho = who.width;
  int heighto = who.height;
  Mat inputImage(round(heighto/resizeScale), round(widtho/resizeScale), inputImage_original.type());
  cv::resize(inputImage_original, inputImage, inputImage.size(), 0, 0, INTER_LINEAR);
  imshow("Alireza", inputImage);

  Size wh = inputImage.size();
  int width = wh.width;
  int height = wh.height;

  float sigma = atof(argv[2]);
  float k = atof(argv[3]);
  int min_size = atoi(argv[4]);

  cout << "Processing..." << endl;

  clock_t startC, endC;
  
  startC = clock();
  int num_ccs; 
  Mat segmentation = felzenszwalb_segment(inputImage, sigma, k, min_size, &num_ccs); 
  endC = clock();

  cout << "Time required for execution: " << (double)(endC-startC)/CLOCKS_PER_SEC << " seconds." << endl;
  //Mat outputImage(Size(320, 240), CV_8UC3);
  //uchar * rawoutputPtr = outputImage.ptr();

  cout << "Creating the output image" << endl;

  int m = sortSegmentNumbers(segmentation);
  cout << "number of regions are: " << m << endl;

  showSegmentationRandom(segmentation, "Felzenszwalb");  

  Mat colorImage = myImagesc(segmentation);
  imshow("Imagesc", colorImage);  
  
  // showing the adjacency matrix
  SparseMat adjacencyMatrix = getWeightedAdjacencyMatrix(segmentation);
  showAdjacencyMatrix(inputImage, segmentation, adjacencyMatrix);

  cvWaitKey(0);
  
  return 0;
}

