#include "TrainStuff.h"
#include "ForegroundStuff.h"

#include "MotionStuff.h"
#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 5) {
    cout << "usage: " << argv[0] << "<dbname> <seqNumber> <startFrame> <endFrame>" << endl;
    return 1;
  }

  int trainVersion = 1;
  char * dbname = argv[1];
  int seqNumber = atoi(argv[2]);
  int startFrame = atoi(argv[3]);
  int endFrame = atoi(argv[4]);
  string datasetName(dbname);
  Dataset db = dataset_setup(datasetName);
  float scale = 0.2;
  db.setScale(scale);
  DatasetSeq seq = db.getSequence(seqNumber);

  int segType = FELZ_SEG;

  int seqContain = 0;

  int endFrame_ = startFrame + seqContain;
  int currentFrame = startFrame;

  Mat locationPattern = trainForegroundLocationPattern(datasetName, trainVersion, scale);
  LinearClassifier trainer;
  trainXiaofengClassifier_svmlight(datasetName, trainVersion, trainer, scale, segType);

  Sequence sq(db, seqNumber, startFrame-10, endFrame_+1, FELZ_SEG);
  do{
    Prior p = sq.getPrior(currentFrame);
    imshow("Im", p.getImage());
    showSegmentationRandom(p.getSegmentation(), "Seg");

    cout << "hello" << endl;
    cout << currentFrame << endl;
    Mat prior = fg_getUnary_svmlight(sq, trainer, currentFrame, currentFrame, locationPattern);
    Mat labels(prior.rows, prior.cols, CV_32S);
    for(int i = 0; i < prior.rows; i++){
      if(prior.at<float>(i,0) < 0)
	labels.at<int>(i,0) = 0;
      else
	labels.at<int>(i,0) = 1;
    }
    cout << "chello" << endl;


    showPrior(prior, p.getSegmentation(), "Prior");
    Mat mask = sq.getPrior(currentFrame).labelsToMask(labels);

    imshow("Mask", mask);

    vector<int> adjFrames;
    adjFrames.push_back(currentFrame-1);
    adjFrames.push_back(currentFrame+1);
    vector<pair<Mat,Mat> > flows;
    std::map<int, Prior> priors = sq.getPriors();
    for(int j = 0; j < adjFrames.size(); j++){
      Pairwise pw = sq.getPairwise(currentFrame, adjFrames[j]);
      flows.push_back(pw.getFlow(priors));
    }
    Mat motionPrior = priors[currentFrame].getPrior_motionChannel(flows);
    showPrior(motionPrior, p.getSegmentation(), "MotionPrior");

    Mat flow_rgb = Mat(p.getImage().rows, p.getImage().cols, CV_8UC3);
    drawColorField(flows[1].first, flows[1].second, flow_rgb);
    imshow("Dense Flow",flow_rgb);

    cvWaitKey(10);

    sq.push_back(db, seqNumber, FELZ_SEG);
    sq.pop_front();

    currentFrame++;
    endFrame_++;
  }
  while(endFrame_ < endFrame);
  

  cvWaitKey(0);

  return 0;
}


