/*
Copyright (C) 2006 Pedro Felzenszwalb

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
*/

#include <cstdio>
#include <cstdlib>
#include <image.h>
#include <misc.h>
#include <pnmfile.h>
#include <opencv2/opencv.hpp>
#include "segment-image.h"

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 6) {
    fprintf(stderr, "usage: %s sigma k min input(ppm) output(ppm)\n", argv[0]);
    return 1;
  }

  string imname(argv[4]);
  Mat inputImage_original = imread(imname);
  Size who = inputImage_original.size();
  int widtho = who.width;
  int heighto = who.height;
  Mat inputImage(round(heighto/2), round(widtho/2), inputImage_original.type());
  cv::resize(inputImage_original, inputImage, inputImage.size(), 0, 0, INTER_LINEAR);
  imshow("Alireza", inputImage);

  Size wh = inputImage.size();
  int width = wh.width;
  int height = wh.height;

  uchar * rawPtr = inputImage.ptr();
  //for(int i = 0; i < height*width; i++)
  //cout << (int)(rawPtr[i]) << endl;

  rgb * rawInput = new rgb[height*width];
  for(int i = 0; i < height*width; i++)
    rawInput[i] = random_rgb();
  int count = 0;
  for(int i = 0; i < height*width; i++){
    rawInput[i].b = rawPtr[count++];
    rawInput[i].g = rawPtr[count++];
    rawInput[i].r = rawPtr[count++];
    }

  cout << "Making a new image..." << endl;

  image<rgb>* input = new image<rgb>(width, height);
  for(int i = 0; i < height*width; i++)
    input->data[i] = rawInput[i];
  
  float sigma = atof(argv[1]);
  float k = atof(argv[2]);
  int min_size = atoi(argv[3]);

  cout << "Processing..." << endl;

  clock_t startC, endC;
  
  startC = clock();
  int num_ccs; 
  image<rgb> *seg = segment_image(input, sigma, k, min_size, &num_ccs); 
  endC = clock();

  cout << "Time required for execution: " << (double)(endC-startC)/CLOCKS_PER_SEC << " seconds." << endl;
  //Mat outputImage(Size(320, 240), CV_8UC3);
  //uchar * rawoutputPtr = outputImage.ptr();

  cout << "Creating the output image" << endl;

  count = 0;
  for(int i = 0; i <height*width; i++){
    rawPtr[count++] = seg->data[i].b;
    rawPtr[count++] = seg->data[i].g;
    rawPtr[count++] = seg->data[i].r;
  }
  
  cout << "Writing the image" << endl;

  imwrite(argv[5], inputImage);

  //printf("got %d components\n", num_ccs);
  //printf("done! uff...thats hard work.\n");
  
  return 0;
}

