/*
  Open an image, run Felzenswalb Segmentation on it, turn the result into a matrix, build its adjacency matrix, and vidualize it
  */

#include <cstdio>
#include <cstdlib>
#include <vector>

#include "global.h"

#include "MouseHandling.h"
#include "MakePatches.h"
#include "ImageStuff.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 4) {
    fprintf(stderr, "usage: %s <input image> <sigma> <k>\n", argv[0]);
    return 1;
  }
  
  string imname(argv[1]);
  Mat inputImage = imread(imname);
  Size wh = inputImage.size();
  int width = wh.width;
  int height = wh.height;
  int channels = inputImage.channels();

  Mat grayImage(height, width, CV_8UC1);
  cvtColor(inputImage, grayImage, CV_BGR2GRAY, 1);

  IplImage grayim = grayImage;
  cvNamedWindow("Gray", 0);
  cvShowImage("Gray", &grayim);

  cout << "The input image is " << height << " by " << width << endl;

  Mat smoothImage(height, width, CV_8UC3);
  GaussianBlur(inputImage, smoothImage, Size(3,3), 0, 0);
  /*IplImage smoothimg = smoothImage;
  cvNamedWindow("Smooth Image", 0);
  cvShowImage("Smooth Image", &smoothimg);*/

  const unsigned int patchRadius1 = 0;
  const unsigned int patchRadius2 = 5;
  const int patchRepMethod = PRM_GRADIENT;
  const int numNeighbors = 5;

  const unsigned int patchSize1 = patchRadius1*2+1;
  const unsigned int patchSize2 = patchRadius2*2+1;

  const unsigned int sfcnn_dim1 = SFCNN_DIM(patchSize1, patchRepMethod);
  const unsigned int sfcnn_dim2 = SFCNN_DIM(patchSize2, patchRepMethod);

  KNNSegmentation<sfcnn_dim1> * myKNN1 = new KNNSegmentation<sfcnn_dim1>(smoothImage, patchRadius1, patchRepMethod, numNeighbors);
  KNNSegmentation<sfcnn_dim2> * myKNN2 = new KNNSegmentation<sfcnn_dim2>(inputImage, patchRadius2, patchRepMethod, numNeighbors);

  double sigma_dif2sim = atof(argv[2]);

  //SparseMat dadjacency_traditional = image2dadjacency_traditional(myKNN1->imageData, myKNN1->patchMatrix, height, width, channels, patchRadius1, sfcnn_dim1);
  SparseMat dadjacency_traditional = image2dadjacency_traditional2(myKNN1->imageData, myKNN1->patchMatrix, height, width, channels, patchRadius1, sfcnn_dim1);

  SparseMat dadjacency_traditional_patch = image2dadjacency_traditional(myKNN2->imageData, myKNN2->patchMatrix, height, width, channels, patchRadius2, sfcnn_dim2);
  SparseMat sadjacency_traditional_patch =  dadjacency2sadjacency(dadjacency_traditional_patch, sigma_dif2sim);
  SparseMat dadjacency_modern = myKNN2->knn2dadjacency();
  SparseMat sadjacency_modern = dadjacency2sadjacency(dadjacency_modern, sigma_dif2sim);
  cout << "Cool:D" << endl;

  SparseMat sadjacency_combo = mergeSparseMatrices<float>(sadjacency_traditional_patch, sadjacency_modern);

  /*  int Y,X;
  int Ycor,Xcor;
  Y = 320; X = 146;
  long int index;
  int ind1,ind2;
  cout << "Shit" << endl;
  YXLocationToIndex(index, Y, X, height, width, 0);
  cout << "Index is: " << index << endl;
  for(SparseMat::iterator it = sadjacency_traditional_patch.begin(); it != sadjacency_traditional_patch.end(); it++){
    ind1 = it.node()->idx[0];
    ind2 = it.node()->idx[1];
    if(ind1 == index){
      indexToYXLocation(Ycor, Xcor, ind2, height, width, 0);
      cout << "(" << Y << "," << X << ") is conncted to (" << Ycor << "," << Xcor << ")" << endl;
    }
    if(ind2 == index){
      indexToYXLocation(Ycor, Xcor, ind1, height, width, 0);
      cout << "(" << Y << "," << X << ") is conncted to (" << Ycor << "," << Xcor << ")" << endl;
    }
  }
  */
  
  int edgeLength = -1;
  edge * edges = dadjacency2edges(dadjacency_traditional, edgeLength);
  //cout << edgeLength << endl;
  universe * u = segment_graph(height*width, edgeLength, edges, 500);
  /*Mat seg = universe2segmentation(u, height, width);
  Mat seg = segmentationForTest22(height, width);
  Mat segsc = myImagesc(seg);
  IplImage segim = segsc;
  cvNamedWindow("at", 0);
  cvShowImage("at", &segim);*/

  showFzsbSegmentation(inputImage, u, "Adjacency traditional");

  Mat seg = universe2segmentation(u, height, width);

  /*int edgeLength2 = -1;
  edge * edges2 = dadjacency2edges(dadjacency_modern, edgeLength2);
  cout << edgeLength2 << endl;
  universe * u2 = segment_graph(height*width, edgeLength2, edges2, 500);
  */
  //showFzsbSegmentation(inputImage, u2, "Adjacency Modern");
  //Mat sadjacency_modern_regions = sadjacency2regionSadjacency(sadjacency_modern, seg, true);
  Mat sadjacency_traditional_patch_regions = sadjacency2regionSadjacency(sadjacency_traditional_patch, seg, true);
  for(int i = 0; i < sadjacency_traditional_patch_regions.size().height; i++){
    if(sadjacency_traditional_patch_regions.at<float>(i,75) > 0.00001)
      cout << i << " -> 75" << endl;
    if(sadjacency_traditional_patch_regions.at<float>(i,76) > 0.00001)
      cout << i << " -> 76" << endl;
  }
  Mat sadjacency_combo_regions = sadjacency2regionSadjacency(sadjacency_combo, seg, true);
  //showNonzeroElementsMatrix(sadjacency_modern_regions);

  
  /*Mat D = matrixToD<float>(sadjacency_traditional_patch_regions);
  int dheight = D.size().height;
  int dwidth = D.size().width;
  cout << dheight << " " << dwidth << endl;
  Mat Dnim_(dheight, dwidth, CV_32F);
  for(int i = 0; i < dheight; i++)
    for(int j = 0; j < dwidth; j++)
      Dnim_.at<float>(i,j) = 0;
  for(int i = 0; i < dheight; i++)
    Dnim_.at<float>(i,i) = 1.0/sqrt(D.at<float>(i,i));

  int a,b;
  float maxValue = matrixMaxValue<float>(D, a, b);
  cout << "Dnim max value: " << maxValue << " at: (" << a << ", " << b << ")" << endl;
  float minValue = matrixMinValue<float>(D, a, b);
  cout << "Dnim min value: " << minValue << " at: (" << a << ", " << b << ")" << endl;
  minValue = matrixMinValueGreaterThanZero<float>(D, a, b);
  cout << "Dnim min value greater than zero: " << minValue << " at: (" << a << ", " << b << ")" << endl;
  minValue = matrixMinValueDiagonal<float>(D, a, b);
  cout << "Dnim min value on the diagonal: " << minValue << " at: (" << a << ", " << b << ")" << endl;
  */
  //showParticularRegion(seg, a);
  //showParticularRegion(seg, a+1);

  //showNonzeroElementsMatrix(Dnim_);

  //Mat laplacian = getNormalizedLaplacianMatrix(sadjacency_traditional_patch_regions);
  //showNonzeroElementsMatrix(sadjacency_traditional_patch_regions);

  /*maxValue = matrixMaxValue<float>(laplacian, a, b);
  cout << "Laplacian max value: " << maxValue << " at: (" << a << ", " << b << ")" << endl;
  minValue = matrixMinValue<float>(laplacian, a, b);
  cout << "Laplacian min value: " << minValue << " at: (" << a << ", " << b << ")" << endl;
  showNonzeroElementsMatrix(laplacian);
  */
  print<float>(sadjacency_traditional_patch_regions, "Sadjacency_traditional");
  //print<float>(sadjacency_modern_regions, "Sadjacency_modern");
  //print<float>(matrixToD<float>(sadjacency_modern_regions), "D modern");

  int k = atoi(argv[3]);
  //vector<int> clusters1 = normalizedSpectralClustering(sadjacency_modern_regions, k);
  vector<int> clusters2 = normalizedSpectralClustering(sadjacency_traditional_patch_regions, k);
  vector<int> clusters3 = normalizedSpectralClustering(sadjacency_combo_regions, k);
  
  /*for(int i = 0; i < k; i++){
    int sum = 0;
    for(vector<int>::iterator it = clusters.begin(); it != clusters.end(); it++)
    if(*it == i)
      sum++;
    cout << "Number of points in bin " << i << " is: " << sum << endl;
  }
  */

  /*Mat segFinal1(height, width, CV_32S);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      segFinal1.at<int>(i,j) = clusters1[seg.at<int>(i,j)-1]+1;
  */
  Mat segFinal2(height, width, CV_32S);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      segFinal2.at<int>(i,j) = clusters2[seg.at<int>(i,j)-1]+1;
  
  Mat segFinal3(height, width, CV_32S);
  for(int i = 0; i < height; i++)
    for(int j = 0; j < width; j++)
      segFinal3.at<int>(i,j) = clusters3[seg.at<int>(i,j)-1]+1;
  
  //showSegmentationRandom(segFinal1, "seg_modern");
  showSegmentationRandom(segFinal2, "seg_traditional");
  showSegmentationRandom(segFinal3, "seg_combo");

  showSegmentationRandom(seg, "Segments");
  cvSetMouseCallback("Segments", mouse_callback_regions, (void*) &seg);
  

  /*  Mat segsc1 = myImagesc(segFinal1);
  IplImage segim1 = segsc1;
  cvNamedWindow("seg_modern", 0);
  cvShowImage("seg_modern", &segim1);

  Mat segsc2 = myImagesc(segFinal2);
  IplImage segim2 = segsc2;
  cvNamedWindow("seg_traditional", 0);
  cvShowImage("seg_traditional", &segim2);
  
  Mat segsc3 = myImagesc(segFinal3);
  IplImage segim3 = segsc3;
  cvNamedWindow("seg_combo", 0);
  cvShowImage("seg_combo", &segim3);
  */
  
  /*  int edgeLength3 = -1;
  edge * edges3 = dadjacency2edges(adjacency_combo, edgeLength3);
  cout << edgeLength3 << endl;
  universe * u3 = segment_graph(height*width, edgeLength3, edges3, 500);

  showFzsbSegmentation(inputImage, u3, "Adjacency Combo");
  */

  /*Mat segmentation(0,0,CV_8UC3);

  IplImage img = inputImage;

  cvNamedWindow("Image", 0);

  cvSetMouseCallback( "Image", mouse_callback_test1<sfcnn_dim>, (void*) myKNN);

  cvShowImage("Image", &img);
  */
  cvWaitKey(0);

  return 0;  
}


