#include "ActionStuff.h"
#include "TrainStuff.h"

#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"

#include <list>

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 2) {
    cout << "usage: " << argv[0] << " <radius> " << endl;
    return 1;
  }

  int radius = atoi(argv[1]);
  
  string datasetName("GTDA_v3");
  Dataset db = dataset_setup(datasetName);
  int numBins_lab = 256;
  int numBins_hog = 256;
  int trainVersion = 3;
  int featureType = ACT_F_CIR | ACT_F_OBJ | ACT_F_FG;
  int pointType = ACT_GAZE;
  int k = 20;
  int k2 = 30;
  int k3 = 50;

  vector<Action> freqActions = db.frequentActions(trainVersion, 3, 2);

  for(int a = 0; a < freqActions.size(); a++){
    cout << a << ": ";
    freqActions[a].print();
    cout << " ";
    int freqTrain = db.getActionFreqInTraining(freqActions[a], trainVersion);
    int freqTest = db.getActionFreqInTesting(freqActions[a], trainVersion);
    cout << "(" << freqTrain << ", " << freqTest << ")";
    cout << endl;
  }


  /*Action tempAction = freqActions[13];
  tempAction.print();
  cout << endl;
  vector<Action> tempFreq;
  tempFreq.push_back(tempAction);
  vector<int> testSeqs = db.getTestSeqs(trainVersion);
  LinearClassifier trainer_;
  trainActionClassifier_svmlight(datasetName, tempAction, freqActions, trainVersion, trainer_, numBins_lab, numBins_hog, radius, k, k2, k3, featureType, pointType);
  for(int i = 0; i < testSeqs.size(); i++){
    int seqNumber = testSeqs[i];
    cout << seqNumber << endl;
    cout << tempFreq.size() << endl;
    vector<Action> acts = db.getSeqFreqActions(seqNumber, tempFreq);
    if(acts.size() < 1)
      continue;
    for(int j = 0; j < acts.size(); j++){
      Action act_ = acts[j];
      act_.print();
      cout << endl;
      vector<pair<int,int> > locations_;
      vector<Mat> responses;
      actionLocalization_interval(db, trainVersion, trainer_, act_, numBins_lab, numBins_hog, radius, seqNumber, act_.startFrame, act_.endFrame, k, k2, k3, featureType, locations_, responses);
      for(int frame = act_.startFrame; frame < act_.endFrame; frame++){
	showFloatMat(responses[frame-act_.startFrame], "Responses");
	imshow("Image", db.getIM(seqNumber, frame));
	cvWaitKey(0);
      }
    }
  }

  return 0;

  */

  /*for(int a = 0; a < freqActions.size(); a++){
    pair<float,float> temp = gazeTransition_1dGaussianForAction(db, freqActions[a], trainVersion);
    freqActions[a].print();
    cout << endl;
    cout << temp.first << " " << temp.second << endl;
    }*/

  /*for(int a = 0; a < freqActions.size(); a++){
    pair<Mat, Mat> temp = gazeLocation_2dGaussianForAction(db, freqActions[a], trainVersion);
    freqActions[a].print();
    cout << endl;
    cout << temp.first << endl << temp.second << endl;
    }*/

  //return 0;


  vector<LinearClassifier> trainers;
  for(int a = 0; a < freqActions.size(); a++){
    LinearClassifier trainer;
    cout << a << ": ";
    freqActions[a].print();
    cout << endl;
    trainActionClassifier_svmlight(datasetName, freqActions[a], freqActions, trainVersion, trainer, numBins_lab, numBins_hog, radius, k, k2, k3, featureType, pointType);
    trainers.push_back(trainer);
  }

  cout << "Testing now..." << endl;

  pair<Mat,Mat> results = testActionClassifier_svmlight(datasetName, trainers, freqActions, trainVersion, numBins_lab, numBins_hog, radius, k, k2, k3, featureType, pointType);
  cout << "Done with the results" << endl;
  //cout << results.first.rows << " " << results.first.cols << endl;

  cout << "Showing Confusion Matrix" << endl;
  Mat confMatrix = classificationToConfusionMatrix(results.first, results.second);
  //cout << confMatrix << endl;
  vector<string> labels;
  for(int i = 0; i < confMatrix.rows; i++)
    labels.push_back(freqActions[i].getActionName());
  showConfusionMatrix(confMatrix, 20, labels);

  for(int i = 0; i < confMatrix.rows; i++){
    cout << i << " ";
    freqActions[i].print();
    cout << " : " << confMatrix.at<float>(i,i) << endl;
  }

  float precision = 0.0;
  for(int i = 0; i < confMatrix.rows; i++)
    precision += confMatrix.at<float>(i,i);
  precision /= confMatrix.rows;
  cout << "Action recognition accuracy for " << confMatrix.rows << " classes is: " << precision << endl;

  cvWaitKey(0);

  return 0;
}

