#include "global.h"

using namespace cv;
using namespace std;

int main(){

  uchar A[2]; A[0] = 15; A[1] = 25;
  uchar B[2]; B[0] = 16; B[1] = 23;
  double n = myUcharNorm(A, B, 2);
  cout << "My Uchar Norm:"  << n << endl;

  Mat C(2, 3, CV_8UC1);
  uchar * C_ptr = C.ptr();
  C_ptr[0] = 15; C_ptr[1] = 7; C_ptr[2] = 9;
  C_ptr[3] = 82; C_ptr[4] = 3; C_ptr[5] = 5;
  int maxX, maxY;
  uchar mx = matrixMaxValue<uchar>(C, maxY, maxX);
  cout << "heightest value: " << (int)(mx) << ", max Y:"  << maxY << ", maxX: " << maxX << endl;


  Mat adjacency(3,3, CV_32F);
  adjacency.at<float>(0,1) = 1.0;
  adjacency.at<float>(1,0) = 1.0;
  adjacency.at<float>(1,2) = 3.0;
  adjacency.at<float>(2,1) = 3.0;
  Mat D = matrixToD<float>(adjacency);
  cout << "D:" << endl;
  for(int i = 0; i < 3; i++){
    for(int j = 0; j < 3; j++){
      cout << D.at<float>(i,j) << " ";
    }
    cout << endl;
  }
  Mat L = D - adjacency;
  
  Mat eigenvectors(3,3, CV_32F);
  Mat eigenvalues(3,3, CV_32F);
  eigen(L, eigenvalues, eigenvectors, 0, 2);
  cout << "Eigenvalues: " << endl;
  for(int i = 0; i < 3; i++)
    cout << eigenvalues.at<float>(i,0) << " ";
  cout << endl;

  Mat tMat(3, 3, CV_32F);
  tMat.at<float>(0,0) = 1;   tMat.at<float>(0,1) = 3;   tMat.at<float>(0,2) = 2;
  tMat.at<float>(1,0) = 1.3;   tMat.at<float>(1,1) = 0.4;   tMat.at<float>(1,2) = 0.5;
  tMat.at<float>(2,0) = 1.9;   tMat.at<float>(2,1) = 3.7;   tMat.at<float>(2,2) = 2.5;

  cout << "tMat " << endl << tMat << endl;

  cout << "colRange 0-0: " << endl << tMat.colRange(0,0) << endl;
  cout << "colRange 0-1: " << endl << tMat.colRange(0,1) << endl;
  cout << "colRange 0-2: " << endl << tMat.colRange(0,2) << endl;
  cout << "colRange 1-2: " << endl << tMat.colRange(1,2) << endl;

  Mat tMatSub = subMatrix<float>(tMat, 2);
  cout << "tMatSub" << endl << tMatSub << endl;

  Mat tMatX, tMatY;
  gradient(tMat, tMatY, tMatX);
  cout << "tMatX:" << endl << tMatX << endl;
  cout << "tMatY:" << endl << tMatY << endl;

  Mat prU(3, 1, CV_32S);
  prU.at<int>(0,0) = 0;
  prU.at<int>(1,0) = 1;
  prU.at<int>(2,0) = 1;

  Mat prV(3, 1, CV_32S);
  prV.at<int>(0,0) = 1;
  prV.at<int>(1,0) = 0;
  prV.at<int>(2,0) = 1;

  Mat tMat_cropped = subMatrix<float>(tMat, prU, prV);
  cout << "tMat Cropped" << endl << tMat_cropped << endl;

  vector<int> indices;
  indices.push_back(2);
  indices.push_back(0);
  Mat tMatIndRows = indMatrixRows<float>(tMat, indices);
  cout << "tMat Row Indices" << endl << tMatIndRows << endl;
  
  Mat tMatIndCols = indMatrixCols<float>(tMat, indices);
  cout << "tMat Col Indices" << endl << tMatIndCols << endl;
  
  Mat repMat = repmat(tMatIndCols, 3, 2);
  cout << "repMat_tMatIndCols" << endl << repMat << endl;



  /*vector<int> answer = normalizedSpectralClustering(adjacency, 2);
  cout << "clusters:" << endl;
  for(int i = 0; i < 3; i++)
    cout << answer[i] << " ";
    cout << endl;*/

  // Writing 3 random numbers between 1 to 10
  vector<int> nums = randsample(10, 3);
  cout << "Writing Random Numbers:" << endl;
  for(int i = 0; i < nums.size(); i++)
    cout << nums[i] << endl;

  

  return 0;
}

