#include "FeatureStuff.h"
#include "ImageStuff.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "GCoptimization.h"

using namespace std;
using namespace cv;

int main(int argc, char **argv) {

  if (argc != 3) {
    cout << "usage: " << argv[0] << " image binSize" << endl;
    return 1;
  }

  string imname(argv[1]);
  Mat inputImage = imread(imname);
  int binSize = atoi(argv[2]);

  int height = inputImage.rows;
  int width = inputImage.cols;

  imshow("inputImage", inputImage);

  Mat colorHist = getColorHistogram(inputImage, binSize);
  
  Mat colorImage = myImagesc(colorHist);
  cout << colorImage.rows << " " << colorImage.cols << endl;
  imshow("Imagesc", colorImage);  

  float sigma = 0.4;
  float k = 100;
  int min_size = 100;
  int num_ccs;
  Mat segmentation = felzenszwalb_segment(inputImage, sigma, k, min_size, &num_ccs);
  sortSegmentNumbers(segmentation);

  //showSegmentationRandom(segmentation, "Felzenszwalb");  

  vector< vector<float> > featureVectors = getFeatureVectors_colorHist(inputImage, binSize, segmentation);
  /*vector<float> temp = featureVectors[0];
  for(int i = 0; i < temp.size(); i++)
    cout << temp[i] << " ";
    cout << endl;*/


  SparseMat adjacencyMatrix = getWeightedAdjacencyMatrix(segmentation);

  SparseMat spatialMatrix = getSpatial_colorHist(featureVectors, adjacencyMatrix, 1, 1);

  /*  int height = inputImage.rows;
  int width = inputImage.cols;
  typedef Vec<uchar,3> Vec3u;
  for(int i = 0; i < height; i++){
    for(int j = 0; j< width; j++){
      cout << (int)(inputImage.at<Vec3u>(i,j)[0]) << endl;
      cout << (int)(inputImage.at<Vec3u>(i,j)[1]) << endl;
      cout << (int)(inputImage.at<Vec3u>(i,j)[2]) << endl;
    }
    }*/

  showSpatialOnContours(inputImage, segmentation, spatialMatrix);

  // testing graphcut
  GCoptimizationGridGraph gc(height, width, 2);
  

  cvWaitKey(0);

  return 0;

}


