#include "TrainStuff.h"
#include "ObjectStuff.h"

#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 1) {
    cout << "usage: " << argv[0] << " " << endl;
    return 1;
  }
  
  string datasetName("GTDA_v3");
  Dataset db = dataset_setup(datasetName);
  int numBins_lab = 256;
  int numBins_hog = 256;
  int k = 20;
  int k2 = 30;
  int k3 = 50;
  int trainVersion = 3;

  vector<string> objNames = db.getObjectNames();
  Mat objColors = db.getObjectColors(objNames);
  cout << objColors.rows << " " << objColors.cols << endl;
  cout << objColors << endl;
  Scalar bgColor = db.getObjectColor(string(""));

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);
  cout << "hello" << endl;

  vector<LinearClassifier> linearTrainers;
  for(int o = 0; o < objNames.size(); o++){
    string objectName = objNames[o];
    cout << objectName.c_str() << endl;

    LinearClassifier linearTrainer;
    //trainHierarchicalObjectClassifier_linear(datasetName, objectName, trainVersion, linearTrainer, numBins_lab, numBins_hog, k, k2, k3);
    trainHierarchicalObjectClassifier_svmlight(datasetName, objectName, trainVersion, linearTrainer, numBins_lab, numBins_hog, k, k2, k3);
    linearTrainers.push_back(linearTrainer);
  }
  cout << endl;

  cout << "Number of trainers: " << linearTrainers.size() << endl;

  vector<Action> freqActions = db.frequentActions(trainVersion, 3, 2);

  vector<int> seqs = db.getTrainSeqs(trainVersion);
  //  vector<int> seqs;
  vector<int> testSeqs = db.getTestSeqs(trainVersion);
  for(int i = 0; i < testSeqs.size(); i++)
    seqs.push_back(testSeqs[i]);

  for(int s = 0; s < seqs.size(); s++){
    int seqNumber = seqs[s];
    vector<Action> actions = db.getSeqFreqActions(seqNumber, freqActions);
    cout << "Number of actions: " << actions.size() << endl;
    for(int a = 0; a < actions.size(); a++){
      int f = (actions[a].startFrame + actions[a].endFrame) / 2;
      Mat im = db.getIM(seqNumber, f);
      Mat ucm = db.readUCM(seqNumber, f);
      PriorH ph(im, ucm, k, k2, k3, f);
      imshow("Image", im);
      Mat classificationMatrix = hierarchicalObjectClassification(ph, linearTrainers, clusterCenters_lab, clusterCenters_hog);

      Mat classLabels = classificationToLabels(classificationMatrix);
      showObjectLabels(classLabels, ph, db, "Labels");
      cvWaitKey(0);
    }
    
  }

  cvWaitKey(0);

  return 0;
}


