#include "PairwiseStuff.h"

#include "MotionStuff.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 5) {
    cout << "usage: " << argv[0] << " image1 image2 image3  binSize" << endl;
    return 1;
  }

  string imname1(argv[1]);
  Mat inputImage1 = imread(imname1);

  string imname2(argv[2]);
  Mat inputImage2 = imread(imname2);

  string imname3(argv[3]);
  Mat inputImage3 = imread(imname3);

  map<int, Prior> priors;
  Prior p1(inputImage1, 1);
  Prior p2(inputImage2, 2);
  Prior p3(inputImage3, 3);

  priors[1] = p1;
  priors[2] = p2;
  priors[3] = p3;
  
  cout << priors[1].getImage().rows << " " << priors[1].getImage().cols << endl;

  int binSize = atoi(argv[4]);

  Pairwise pw(2, 1);
  
  vector<int> adjFrames;
  adjFrames.push_back(1);
  adjFrames.push_back(3);
  Mat prior2 = p2.getPrior_motionChannel(priors, adjFrames);
  showPrior(prior2, p2.getSegmentation(), "Prior2");

  Mat prior1 = pw.transferFrom1to2(prior2, priors);
  showPrior(prior1, p1.getSegmentation(), "Prior1");
  
  imshow("Image1", pw.getImage1(priors));
  imshow("Image2", pw.getImage2(priors));

  cvWaitKey(0);

  return 0;
}

