#include "ActionStuff.h"
#include "TrainStuff.h"
#include "ForegroundStuff.h"

#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 3) {
    cout << "usage: " << argv[0] << " <dbname> <trainVersion>" <<  endl;
    return 1;
  }

  string datasetName(argv[1]);
  int trainVersion = atoi(argv[2]);
  Dataset db = dataset_setup(datasetName);

  string dataDir = db.getDataDir();

  vector<Action> freqActions = db.frequentActions(trainVersion, 3, 2);

  vector<int> seqs = db.getTrainSeqs(trainVersion);
  vector<int> testSeqs = db.getTestSeqs(trainVersion);
  for(int i = 0; i < testSeqs.size(); i++)
    seqs.push_back(testSeqs[i]);

  for(int s = 0; s < seqs.size(); s++){
    int seqNumber = seqs[s];
    vector<Action> actions = db.getSeqFreqActions(seqNumber, freqActions);
    for(int a = 0; a < actions.size(); a++){
      bool anyLeft = false;
      for(int frame = actions[a].startFrame; frame <= actions[a].endFrame; frame++){
	Mat mask = db.readForegroundMask(seqNumber, frame, trainVersion);
	if(mask.rows < 1)
	  anyLeft = true;
	if(anyLeft)
	  break;
      }
      if(anyLeft){
	cout << seqNumber << " " << actions[a].startFrame << "-" << actions[a].endFrame << endl;
	actions[a].print();
	cout << endl;
	cout << "*******************************************************" << endl;
	batch_fgSegmentation(datasetName, seqNumber, actions[a].startFrame-50, actions[a].endFrame+50, trainVersion, 1, BERKELEY_SEG);
      }
    }
  }
  

  /*  vector<int> seqs = db.getTrainSeqs(trainVersion);
  for(int s = 0; s < seqs.size(); s++){
    int seqNumber = seqs[s];
    int startFrame = 101;
    int endFrame = db.getSequence(seqNumber).getSeqLength()- 101;

    DatasetSeq seq = db.getSequence(seqNumber);
    string seqName = seq.getSeqName();

    for(int frame = startFrame; frame <= endFrame; frame++){
      Mat mask = readForegroundMask(db, seqNumber, frame, trainVersion);
      printf("%d %d\n", mask.rows, mask.cols);
      Mat im = db.getIM(seqNumber, frame);
      mask = cleanForegroundMask(im, mask);
      showMaskOnImage2(mask, im, "FG Mask");
      cvWaitKey(10);
    }
    }*/


  
  /*
  Mat im = db.getIM(seqNumber, f);
  Mat ucm = db.readUCM(seqNumber, f);
  Prior p(im, ucm, k, f);

  Mat prior_location = p.getPrior_location(locationPattern);
  showPrior(prior_location, p.getSegmentation(), "prior_location");

  CvSVM trainer;
  trainXiaofengClassifier(datasetName, 1, trainer);
  cout << "Number of support vectors: " << trainer.get_support_vector_count() << endl;
  cout << trainer.get_support_vector(10)[0] << " " << trainer.get_support_vector(10)[1] << endl;
  CvSVMParams params = trainer.get_params();
  cout << params.svm_type << " " << params.kernel_type << " " << params.C << endl;

  string dataDir = db.getDataDir();
  string seqName = db.getSequence(seqNumber).getSeqName();
  string imType = db.getImType();
  int startFrame = 3000;
  int endFrame_ = 3020;
  int endFrame = 4000;
  int currentFrame = startFrame;
  Sequence sq(dataDir.c_str(), seqName.c_str(), imType.c_str(), startFrame-10, endFrame_, BERKELEY_SEG, 1);
  while(endFrame_ < endFrame){
    cout << "CurrentFrame: " << currentFrame << endl;
    Mat fv = sq.xiaofengFeatureVector(currentFrame, locationPattern);
    //cout << "fv: " << fv.rows << " " << fv.cols << endl;
    int m = fv.rows;
    Mat prior(m, 1, CV_32F);
    //trainer.predict(fv, prior);
    //cout << prior << endl;
    Mat temp;
    for(int i = 0; i < m; i++){
      temp = fv.rowRange(i,i+1);
      prior.at<float>(i,0) = -trainer.predict(temp, true);
    }
    //cout << "prior: " << prior.rows << " " << prior.cols << endl;
    showPrior(prior, sq.getPrior(currentFrame).getSegmentation(), "FG");
    imshow("Image", sq.getPrior(currentFrame).getImage());
    
    sq.push_back(dataDir.c_str(), seqName.c_str(), imType.c_str(), BERKELEY_SEG, 1);
    sq.pop_front();
    sq.computeW(currentFrame-5, currentFrame+5, 0.1);

    cout << "Sequence: " << sq.getStartFrame() << " " << sq.getEndFrame() << endl;

    currentFrame++;
    endFrame_++;
    cvWaitKey(0);
  }
  */

  cvWaitKey(0);

  return 0;
}


