#include "TrainStuff.h"
#include "ObjectStuff.h"

#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 2) {
    cout << "usage: " << argv[0] << " ObjName" << endl;
    return 1;
  }

  //char * modelFile = "/home/alireza/Data/GTDA_v3/opencv/svm_light/trainHierarchicalObjectClassifier_svmlight_modelFile_Bagel_Container_3_256_256_20_30_50.bin";
  //LinearClassifier lc = svm_light_read_model(modelFile);
  //lc.save("/home/alireza/Data/GTDA_v3/opencv/svm_light/temp.txt");

  string objName = string(argv[1]);

  string datasetName("GTDA_v3");
  Dataset db = dataset_setup(datasetName);
  int numBins_lab = 256;
  int numBins_hog = 256;
  int k = 20;
  int k2 = 30;
  int k3 = 50;
  int trainVersion = 3;

  Mat clusterCenters_lab = getLabClusters(datasetName, trainVersion, numBins_lab);
  cout << clusterCenters_lab.rows << " " << clusterCenters_lab.cols << endl;

  Mat clusterCenters_hog = getHOGClusters(datasetName, trainVersion, numBins_hog);
  cout << clusterCenters_hog.rows << " " << clusterCenters_hog.cols << endl;

  vector<string> objectNames = db.getObjectNames();

  LinearClassifier trainer;
  CvSVM * trainer_ = new CvSVM();

  for(int o = 0; o < objectNames.size(); o++){
    string objectName = objectNames[o];
    if(strcmp(objectName.c_str(), objName.c_str()) == 0){
      trainHierarchicalObjectClassifier(datasetName, objectName, trainVersion, trainer_, numBins_lab, numBins_hog, k, k2, k3, CvSVM::RBF);
      trainHierarchicalObjectClassifier_svmlight(datasetName, objectName, trainVersion, trainer, numBins_lab, numBins_hog, k, k2, k3);
    }
  }

  //trainer.print();

  vector<Action> freqActions = db.frequentActions(trainVersion, 3, 2);

  vector<int> seqs = db.getTrainSeqs(trainVersion);
  //  vector<int> seqs;
  vector<int> testSeqs = db.getTestSeqs(trainVersion);
  for(int i = 0; i < testSeqs.size(); i++)
    seqs.push_back(testSeqs[i]);

  for(int s = 0; s < seqs.size(); s++){
    int seqNumber = seqs[s];
    vector<Action> actions = db.getSeqFreqActions(seqNumber, freqActions);
    cout << "Number of actions: " << actions.size() << endl;
    for(int a = 0; a < actions.size(); a++){
      int f = (actions[a].startFrame + actions[a].endFrame) / 2;

      Mat im = db.getIM(seqNumber, f);
      Mat ucm = db.readUCM(seqNumber, f);
      PriorH ph(im, ucm, k, k2, k3, f);

      imshow("Image", ph.getImage());
      
      Mat prior = hierarchicalObjectClassification(ph, trainer, clusterCenters_lab, clusterCenters_hog);
      Mat prior_ = hierarchicalObjectClassification(ph, trainer_, clusterCenters_lab, clusterCenters_hog);
      showPrior(prior, ph.getSegmentation(), "Classification");
      showPrior(prior_, ph.getSegmentation(), "Classification_");

      cvWaitKey(0);
    }
  }

  return 0;
}


