#include "TrainStuff.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"
#include "FeatureStuff.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 1) {
    cout << "usage: " << argv[0]  <<  endl;
    return 1;
  }

  string datasetName("GTDA_v3");
  Dataset db = dataset_setup(datasetName);
  int seqNumber = 2;
  int frameNumber = 3000;
  int k = 20;

  Mat inputImage = db.getIM(seqNumber, frameNumber);
  imshow("Image", inputImage);
  Mat ucm = db.readUCM(seqNumber, frameNumber);
  Prior p(inputImage, ucm, k, frameNumber);

  int nb = 128;
  Mat centers = getHOGClusters(string("GTDA_v3"), 1, nb);
  Mat fv = getHOG(inputImage);
  Mat bins = getFeatureMap(fv, centers);
  Mat featureMap = featureMapHOG_reshape(bins, inputImage.rows, inputImage.cols);
  showSegmentationRandom(featureMap, "HOG Map");

  int nb_lab = 256;
  Mat centers_lab = getLabClusters(string("GTDA_v3"), 1, nb_lab);
  Mat fv_lab = getColorLab(inputImage);
  Mat bins_lab = getFeatureMap(fv_lab, centers_lab);
  Mat featureMap_lab = featureMapLab_reshape(bins_lab, inputImage.rows, inputImage.cols);
  showSegmentationRandom(featureMap_lab, "Color Map");

  Mat fv_regions = featureMap2regionFeatures(featureMap_lab, p.getSegmentation(), nb_lab);

  PriorH ph(inputImage, ucm, 20, 30, 50, frameNumber);
  Mat fvh_lab = ph.getHierarchicalFeature_CLab(centers_lab);
  Mat fvh_hog = ph.getHierarchicalFeature_HOG(centers);
  //cout << ph.getM() << " " << fvh_lab.rows << " " << fvh_lab.cols << endl;
  //cout << fvh_hog << endl;
  //cout << ph.getM() << " " << fvh_hog.rows << " " << fvh_hog.cols << endl;
  

  Mat fv_circle = featureAroundPoint(featureMap_lab, 200, 300, 100, nb_lab);
  cout << fv_circle << endl;

  cvWaitKey(0);

  return 0;
}
