#include "TrainStuff.h"
#include "ForegroundStuff.h"

#include "InputOutputFile.h"
#include "SegmentationStuff.h"
#include "Visualizations.h"
#include "MyMATFileInterface.h"

#include <stdio.h>

using namespace std;
using namespace cv;

int main(int argc, char **argv) {
  if (argc != 5) {
    cout << "usage: " << argv[0] << " doFG actionVerb actionNoun1,actionNoun2,... FPS" << endl;
    return 1;
  }

  Dataset db = dataset_setup("GTDA_v3");
  int trainVersion = 3;

  bool doFG = (bool)(atoi(argv[1]));

  Action action;
  action.verb = string(argv[2]);

  char * tok = strtok(argv[3], ",");
  while(tok != NULL){
    action.names.push_back(string(tok));
    tok = strtok(NULL, ",");
  }

  action.print();
  cout << endl;

  int fps = atoi(argv[4]);
  
  vector<int> sequences = db.getTrainSeqs(trainVersion);
  vector<int> testSeqs = db.getTestSeqs(trainVersion);
  for(int i = 0; i < testSeqs.size(); i++)
    sequences.push_back(testSeqs[i]);

  //writeGTActions(action, db.getDatasetName(), trainVersion, seqs, true);

  string dataDir = db.getDataDir();

  char actName[500];
  actName[0] = 0;
  sprintf(actName, "%s", action.verb.c_str());
  for(int n = 0; n < action.names.size(); n++)
    sprintf(actName, "%s_%s", actName, action.names[n].c_str());
  
  char rootDir[500];
  rootDir[0] = 0;
  sprintf(rootDir, "%s/opencv/Results/%s", dataDir.c_str(), actName);
  
  char command[500];
  command[0] = 0;
  sprintf(command, "mkdir %s", rootDir);
  system(command);

  for(int s = 0; s < sequences.size(); s++){
    int seqNumber = sequences[s];
    string seqName = db.getSequence(seqNumber).getSeqName();
    vector<Action> actions = db.getActions(seqNumber);
    for(int a = 0; a < actions.size(); a++){
      if(!areEqualActions(action, actions[a]))
	continue;
      char fileName[500];
      fileName[0] = 0;
      if(doFG)
	sprintf(fileName, "%s/FG_%s_%s_%d_%d.avi", rootDir, actName, seqName.c_str(), actions[a].startFrame, actions[a].endFrame);
      else
	sprintf(fileName, "%s/%s_%s_%d_%d.avi", rootDir, actName, seqName.c_str(), actions[a].startFrame, actions[a].endFrame);
      //cout << fileName << endl;
      Mat im = db.getIM(seqNumber, actions[a].startFrame);
      Size ss = im.size();
      VideoWriter vw;
      vw.open(fileName, CV_FOURCC('M', 'J', 'P', 'G'), fps, ss);
      for(int f = actions[a].startFrame-20; f <= actions[a].endFrame+20; f++){
	Mat im = db.getIM(seqNumber, f);
	if(doFG){
	  Mat mask = db.readForegroundMask(seqNumber, f, trainVersion);
	  mask = cleanForegroundMask(im, mask);
	  im = maskOnImage2(mask, im);
	}
	vw << im;
      }
    }
  }


  return 0;
}

